@@ -2,15 +2,20 @@
 '''
 The acl module handles publisher_acl operations
 
-Additional information on client_acl can be
+Additional information on publisher_acl can be
 found by reading the salt documentation:
 
     http://docs.saltstack.com/en/latest/ref/publisheracl.html
 '''
 
 # Import python libraries
-from __future__ import absolute_import
-import salt.utils
+from __future__ import absolute_import, print_function, unicode_literals
+
+# Import salt libs
+import salt.utils.stringutils
+
+# Import 3rd-party libs
+from salt.ext import six
 
 
 class PublisherACL(object):
@@ -26,13 +31,13 @@ class PublisherACL(object):
         Takes a username as a string and returns a boolean. True indicates that
         the provided user has been blacklisted
         '''
-        return not salt.utils.check_whitelist_blacklist(user, blacklist=self.blacklist.get('users', []))
+        return not salt.utils.stringutils.check_whitelist_blacklist(user, blacklist=self.blacklist.get('users', []))
 
     def cmd_is_blacklisted(self, cmd):
         # If this is a regular command, it is a single function
-        if isinstance(cmd, str):
+        if isinstance(cmd, six.string_types):
             cmd = [cmd]
         for fun in cmd:
-            if not salt.utils.check_whitelist_blacklist(fun, blacklist=self.blacklist.get('modules', [])):
+            if not salt.utils.stringutils.check_whitelist_blacklist(fun, blacklist=self.blacklist.get('modules', [])):
                 return True
         return False
