@@ -36,19 +36,18 @@ authenticated against.  This defaults to `login`
 '''
 
 # Import Python Libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 from ctypes import CDLL, POINTER, Structure, CFUNCTYPE, cast, pointer, sizeof
 from ctypes import c_void_p, c_uint, c_char_p, c_char, c_int
 from ctypes.util import find_library
 
 # Import Salt libs
-from salt.utils import get_group_list
+import salt.utils.user
 from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 
-LIBPAM = CDLL(find_library('pam'))
 LIBC = CDLL(find_library('c'))
 
 CALLOC = LIBC.calloc
@@ -121,6 +120,7 @@ class PamConv(Structure):
 
 
 try:
+    LIBPAM = CDLL(find_library('pam'))
     PAM_START = LIBPAM.pam_start
     PAM_START.restype = c_int
     PAM_START.argtypes = [c_char_p, c_char_p, POINTER(PamConv),
@@ -214,4 +214,4 @@ def groups(username, *args, **kwargs):
 
     Uses system groups
     '''
-    return get_group_list(username)
+    return salt.utils.user.get_group_list(username)
