@@ -27,28 +27,27 @@ Set up the cloud configuration at ``/etc/salt/cloud.providers`` or
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import time
 import pprint
 import logging
 import re
-import json
 
 # Import salt libs
-import salt.ext.six as six
-import salt.utils
+import salt.utils.cloud
+import salt.utils.json
 
 # Import salt cloud libs
-import salt.utils.cloud
 import salt.config as config
 from salt.exceptions import (
     SaltCloudSystemExit,
     SaltCloudExecutionFailure,
     SaltCloudExecutionTimeout
 )
-from salt.ext.six.moves import range
 
-# Import Third Party Libs
+# Import 3rd-party Libs
+from salt.ext import six
+from salt.ext.six.moves import range
 try:
     import requests
     HAS_REQUESTS = True
@@ -138,7 +137,7 @@ def _authenticate():
         full_url, verify=verify_ssl, data=connect_data).json()
 
     ticket = {'PVEAuthCookie': returned_data['data']['ticket']}
-    csrf = str(returned_data['data']['CSRFPreventionToken'])
+    csrf = six.text_type(returned_data['data']['CSRFPreventionToken'])
 
 
 def query(conn_type, option, post_data=None):
@@ -151,7 +150,7 @@ def query(conn_type, option, post_data=None):
 
     full_url = 'https://{0}:8006/api2/json/{1}'.format(url, option)
 
-    log.debug('{0}: {1} ({2})'.format(conn_type, full_url, post_data))
+    log.debug('%s: %s (%s)', conn_type, full_url, post_data)
 
     httpheaders = {'Accept': 'application/json',
                    'Content-Type': 'application/x-www-form-urlencoded',
@@ -200,7 +199,7 @@ def _get_vm_by_name(name, allDetails=False):
     if name in vms:
         return vms[name]
 
-    log.info('VM with name "{0}" could not be found.'.format(name))
+    log.info('VM with name "%s" could not be found.', name)
     return False
 
 
@@ -209,10 +208,10 @@ def _get_vm_by_id(vmid, allDetails=False):
     Retrieve a VM based on the ID.
     '''
     for vm_name, vm_details in six.iteritems(get_resources_vms(includeConfig=allDetails)):
-        if str(vm_details['vmid']) == str(vmid):
+        if six.text_type(vm_details['vmid']) == six.text_type(vmid):
             return vm_details
 
-    log.info('VM with ID "{0}" could not be found.'.format(vmid))
+    log.info('VM with ID "%s" could not be found.', vmid)
     return False
 
 
@@ -233,10 +232,10 @@ def _check_ip_available(ip_addr):
     for vm_name, vm_details in six.iteritems(get_resources_vms(includeConfig=True)):
         vm_config = vm_details['config']
         if ip_addr in vm_config['ip_address'] or vm_config['ip_address'] == ip_addr:
-            log.debug('IP "{0}" is already defined'.format(ip_addr))
+            log.debug('IP "%s" is already defined', ip_addr)
             return False
 
-    log.debug('IP \'{0}\' is available to be defined'.format(ip_addr))
+    log.debug('IP \'%s\' is available to be defined', ip_addr)
     return True
 
 
@@ -251,18 +250,18 @@ def _parse_proxmox_upid(node, vm_=None):
     # Parse node response
     node = node.split(':')
     if node[0] == 'UPID':
-        ret['node'] = str(node[1])
-        ret['pid'] = str(node[2])
-        ret['pstart'] = str(node[3])
-        ret['starttime'] = str(node[4])
-        ret['type'] = str(node[5])
-        ret['vmid'] = str(node[6])
-        ret['user'] = str(node[7])
+        ret['node'] = six.text_type(node[1])
+        ret['pid'] = six.text_type(node[2])
+        ret['pstart'] = six.text_type(node[3])
+        ret['starttime'] = six.text_type(node[4])
+        ret['type'] = six.text_type(node[5])
+        ret['vmid'] = six.text_type(node[6])
+        ret['user'] = six.text_type(node[7])
         # include the upid again in case we'll need it again
-        ret['upid'] = str(upid)
+        ret['upid'] = six.text_type(upid)
 
         if vm_ is not None and 'technology' in vm_:
-            ret['technology'] = str(vm_['technology'])
+            ret['technology'] = six.text_type(vm_['technology'])
 
     return ret
 
@@ -272,13 +271,13 @@ def _lookup_proxmox_task(upid):
     Retrieve the (latest) logs and retrieve the status for a UPID.
     This can be used to verify whether a task has completed.
     '''
-    log.debug('Getting creation status for upid: {0}'.format(upid))
+    log.debug('Getting creation status for upid: %s', upid)
     tasks = query('get', 'cluster/tasks')
 
     if tasks:
         for task in tasks:
             if task['upid'] == upid:
-                log.debug('Found upid task: {0}'.format(task))
+                log.debug('Found upid task: %s', task)
                 return task
 
     return False
@@ -293,7 +292,7 @@ def get_resources_nodes(call=None, resFilter=None):
 
         salt-cloud -f get_resources_nodes my-proxmox-config
     '''
-    log.debug('Getting resource: nodes.. (filter: {0})'.format(resFilter))
+    log.debug('Getting resource: nodes.. (filter: %s)', resFilter)
     resources = query('get', 'cluster/resources')
 
     ret = {}
@@ -303,11 +302,11 @@ def get_resources_nodes(call=None, resFilter=None):
             ret[name] = resource
 
     if resFilter is not None:
-        log.debug('Filter given: {0}, returning requested '
-                  'resource: nodes'.format(resFilter))
+        log.debug('Filter given: %s, returning requested '
+                  'resource: nodes', resFilter)
         return ret[resFilter]
 
-    log.debug('Filter not given: {0}, returning all resource: nodes'.format(ret))
+    log.debug('Filter not given: %s, returning all resource: nodes', ret)
     return ret
 
 
@@ -321,7 +320,7 @@ def get_resources_vms(call=None, resFilter=None, includeConfig=True):
 
         salt-cloud -f get_resources_vms my-proxmox-config
     '''
-    log.debug('Getting resource: vms.. (filter: {0})'.format(resFilter))
+    log.debug('Getting resource: vms.. (filter: %s)', resFilter)
     resources = query('get', 'cluster/resources')
 
     ret = {}
@@ -339,11 +338,11 @@ def get_resources_vms(call=None, resFilter=None, includeConfig=True):
                 )
 
     if resFilter is not None:
-        log.debug('Filter given: {0}, returning requested '
-                  'resource: nodes'.format(resFilter))
+        log.debug('Filter given: %s, returning requested '
+                  'resource: nodes', resFilter)
         return ret[resFilter]
 
-    log.debug('Filter not given: {0}, returning all resource: nodes'.format(ret))
+    log.debug('Filter not given: %s, returning all resource: nodes', ret)
     return ret
 
 
@@ -432,15 +431,15 @@ def list_nodes(call=None):
 
     ret = {}
     for vm_name, vm_details in six.iteritems(get_resources_vms(includeConfig=True)):
-        log.debug('VM_Name: {0}'.format(vm_name))
-        log.debug('vm_details: {0}'.format(vm_details))
+        log.debug('VM_Name: %s', vm_name)
+        log.debug('vm_details: %s', vm_details)
 
         # Limit resultset on what Salt-cloud demands:
         ret[vm_name] = {}
-        ret[vm_name]['id'] = str(vm_details['vmid'])
-        ret[vm_name]['image'] = str(vm_details['vmid'])
-        ret[vm_name]['size'] = str(vm_details['disk'])
-        ret[vm_name]['state'] = str(vm_details['status'])
+        ret[vm_name]['id'] = six.text_type(vm_details['vmid'])
+        ret[vm_name]['image'] = six.text_type(vm_details['vmid'])
+        ret[vm_name]['size'] = six.text_type(vm_details['disk'])
+        ret[vm_name]['state'] = six.text_type(vm_details['status'])
 
         # Figure out which is which to put it in the right column
         private_ips = []
@@ -450,9 +449,9 @@ def list_nodes(call=None):
             ips = vm_details['config']['ip_address'].split(' ')
             for ip_ in ips:
                 if IP(ip_).iptype() == 'PRIVATE':
-                    private_ips.append(str(ip_))
+                    private_ips.append(six.text_type(ip_))
                 else:
-                    public_ips.append(str(ip_))
+                    public_ips.append(six.text_type(ip_))
 
         ret[vm_name]['private_ips'] = private_ips
         ret[vm_name]['public_ips'] = public_ips
@@ -524,29 +523,28 @@ def create(vm_):
         transport=__opts__['transport']
     )
 
-    log.info('Creating Cloud VM {0}'.format(vm_['name']))
+    log.info('Creating Cloud VM %s', vm_['name'])
 
     if 'use_dns' in vm_ and 'ip_address' not in vm_:
         use_dns = vm_['use_dns']
         if use_dns:
             from socket import gethostbyname, gaierror
             try:
-                ip_address = gethostbyname(str(vm_['name']))
+                ip_address = gethostbyname(six.text_type(vm_['name']))
             except gaierror:
-                log.debug('Resolving of {hostname} failed'.format(hostname=str(vm_['name'])))
+                log.debug('Resolving of %s failed', vm_['name'])
             else:
-                vm_['ip_address'] = str(ip_address)
+                vm_['ip_address'] = six.text_type(ip_address)
 
     try:
         newid = _get_next_vmid()
         data = create_node(vm_, newid)
     except Exception as exc:
         log.error(
-            'Error creating {0} on PROXMOX\n\n'
+            'Error creating %s on PROXMOX\n\n'
             'The following exception was thrown when trying to '
-            'run the initial deployment: \n{1}'.format(
-                vm_['name'], str(exc)
-            ),
+            'run the initial deployment: \n%s',
+            vm_['name'], exc,
             # Show the traceback if the debug logging level is enabled
             exc_info_on_loglevel=logging.DEBUG
         )
@@ -563,15 +561,15 @@ def create(vm_):
 
     # Determine which IP to use in order of preference:
     if 'ip_address' in vm_:
-        ip_address = str(vm_['ip_address'])
+        ip_address = six.text_type(vm_['ip_address'])
     elif 'public_ips' in data:
-        ip_address = str(data['public_ips'][0])  # first IP
+        ip_address = six.text_type(data['public_ips'][0])  # first IP
     elif 'private_ips' in data:
-        ip_address = str(data['private_ips'][0])  # first IP
+        ip_address = six.text_type(data['private_ips'][0])  # first IP
     else:
         raise SaltCloudExecutionFailure  # err.. not a good idea i reckon
 
-    log.debug('Using IP address {0}'.format(ip_address))
+    log.debug('Using IP address %s', ip_address)
 
     # wait until the vm has been created so we can start it
     if not wait_for_created(data['upid'], timeout=300):
@@ -579,11 +577,11 @@ def create(vm_):
 
     # VM has been created. Starting..
     if not start(name, vmid, call='action'):
-        log.error('Node {0} ({1}) failed to start!'.format(name, vmid))
+        log.error('Node %s (%s) failed to start!', name, vmid)
         raise SaltCloudExecutionFailure
 
     # Wait until the VM has fully started
-    log.debug('Waiting for state "running" for vm {0} on {1}'.format(vmid, host))
+    log.debug('Waiting for state "running" for vm %s on %s', vmid, host)
     if not wait_for_state(vmid, 'running'):
         return {'Error': 'Unable to start {0}, command timed out'.format(name)}
 
@@ -603,11 +601,10 @@ def create(vm_):
     ret = __utils__['cloud.bootstrap'](vm_, __opts__)
 
     # Report success!
-    log.info('Created Cloud VM \'{0[name]}\''.format(vm_))
+    log.info('Created Cloud VM \'%s\'', vm_['name'])
     log.debug(
-        '\'{0[name]}\' VM creation details:\n{1}'.format(
-            vm_, pprint.pformat(data)
-        )
+        '\'%s\' VM creation details:\n%s',
+        vm_['name'], pprint.pformat(data)
     )
 
     __utils__['cloud.fire_event'](
@@ -633,7 +630,7 @@ def _import_api():
 
     re_filter = re.compile('(?<=pveapi =)(.*)(?=^;)', re.DOTALL | re.MULTILINE)
     api_json = re_filter.findall(returned_data.text)[0]
-    api = json.loads(api_json)
+    api = salt.utils.json.loads(api_json)
 
 
 def _get_properties(path="", method="GET", forced_params=None):
@@ -660,7 +657,7 @@ def _get_properties(path="", method="GET", forced_params=None):
         # get list of properties for requested method
         props = sub['info'][method]['parameters']['properties'].keys()
     except KeyError as exc:
-        log.error('method not found: "{0}"'.format(str(exc)))
+        log.error('method not found: "%s"', exc)
     except:
         raise
     for prop in props:
@@ -669,7 +666,7 @@ def _get_properties(path="", method="GET", forced_params=None):
         # "prop[n]"
         if numerical:
             for i in range(10):
-                parameters.add(numerical.group(1) + str(i))
+                parameters.add(numerical.group(1) + six.text_type(i))
         else:
             parameters.add(prop)
     return parameters
@@ -723,7 +720,7 @@ def create_node(vm_, newid):
         newnode['hostname'] = vm_['name']
         newnode['ostemplate'] = vm_['image']
 
-        static_props = ('cpuunits', 'description', 'memory', 'onboot', 'net0',
+        static_props = ('cpuunits', 'cpulimit', 'rootfs', 'cores', 'description', 'memory', 'onboot', 'net0',
                         'password', 'nameserver', 'swap', 'storage', 'rootfs')
         for prop in _get_properties('/nodes/{node}/lxc',
                                     'POST',
@@ -767,8 +764,7 @@ def create_node(vm_, newid):
         sock_dir=__opts__['sock_dir'],
     )
 
-    log.debug('Preparing to generate a node using these parameters: {0} '.format(
-              newnode))
+    log.debug('Preparing to generate a node using these parameters: %s ', newnode)
     if 'clone' in vm_ and vm_['clone'] is True and vm_['technology'] == 'qemu':
         postParams = {}
         postParams['newid'] = newnode['vmid']
@@ -848,18 +844,16 @@ def wait_for_state(vmid, state, timeout=300):
 
     while True:
         if node['status'] == state:
-            log.debug('Host {0} is now in "{1}" state!'.format(
-                node['name'], state
-            ))
+            log.debug('Host %s is now in "%s" state!', node['name'], state)
             return True
         time.sleep(1)
         if time.time() - start_time > timeout:
-            log.debug('Timeout reached while waiting for {0} to '
-                      'become {1}'.format(node['name'], state))
+            log.debug('Timeout reached while waiting for %s to become %s',
+                      node['name'], state)
             return False
         node = get_vm_status(vmid=vmid)
-        log.debug('State for {0} is: "{1}" instead of "{2}"'.format(
-                  node['name'], node['status'], state))
+        log.debug('State for %s is: "%s" instead of "%s"',
+                  node['name'], node['status'], state)
 
 
 def destroy(name, call=None):
@@ -922,30 +916,30 @@ def set_vm_status(status, name=None, vmid=None):
     '''
     Convenience function for setting VM status
     '''
-    log.debug('Set status to {0} for {1} ({2})'.format(status, name, vmid))
+    log.debug('Set status to %s for %s (%s)', status, name, vmid)
 
     if vmid is not None:
-        log.debug('set_vm_status: via ID - VMID {0} ({1}): {2}'.format(
-                  vmid, name, status))
+        log.debug('set_vm_status: via ID - VMID %s (%s): %s',
+                  vmid, name, status)
         vmobj = _get_vm_by_id(vmid)
     else:
-        log.debug('set_vm_status: via name - VMID {0} ({1}): {2}'.format(
-                  vmid, name, status))
+        log.debug('set_vm_status: via name - VMID %s (%s): %s',
+                  vmid, name, status)
         vmobj = _get_vm_by_name(name)
 
     if not vmobj or 'node' not in vmobj or 'type' not in vmobj or 'vmid' not in vmobj:
-        log.error('Unable to set status {0} for {1} ({2})'.format(
-                  status, name, vmid))
+        log.error('Unable to set status %s for %s (%s)',
+                  status, name, vmid)
         raise SaltCloudExecutionTimeout
 
-    log.debug("VM_STATUS: Has desired info ({0}). Setting status..".format(vmobj))
+    log.debug("VM_STATUS: Has desired info (%s). Setting status..", vmobj)
     data = query('post', 'nodes/{0}/{1}/{2}/status/{3}'.format(
                  vmobj['node'], vmobj['type'], vmobj['vmid'], status))
 
     result = _parse_proxmox_upid(data, vmobj)
 
     if result is not False and result is not None:
-        log.debug('Set_vm_status action result: {0}'.format(result))
+        log.debug('Set_vm_status action result: %s', result)
         return True
 
     return False
@@ -956,20 +950,20 @@ def get_vm_status(vmid=None, name=None):
     Get the status for a VM, either via the ID or the hostname
     '''
     if vmid is not None:
-        log.debug('get_vm_status: VMID {0}'.format(vmid))
+        log.debug('get_vm_status: VMID %s', vmid)
         vmobj = _get_vm_by_id(vmid)
     elif name is not None:
-        log.debug('get_vm_status: name {0}'.format(name))
+        log.debug('get_vm_status: name %s', name)
         vmobj = _get_vm_by_name(name)
     else:
         log.debug("get_vm_status: No ID or NAME given")
         raise SaltCloudExecutionFailure
 
-    log.debug('VM found: {0}'.format(vmobj))
+    log.debug('VM found: %s', vmobj)
 
     if vmobj is not None and 'node' in vmobj:
-        log.debug("VM_STATUS: Has desired info. Retrieving.. ({0})".format(
-                  vmobj['name']))
+        log.debug("VM_STATUS: Has desired info. Retrieving.. (%s)",
+                  vmobj['name'])
         data = query('get', 'nodes/{0}/{1}/{2}/status/current'.format(
                      vmobj['node'], vmobj['type'], vmobj['vmid']))
         return data
@@ -993,9 +987,9 @@ def start(name, vmid=None, call=None):
             'The start action must be called with -a or --action.'
         )
 
-    log.debug('Start: {0} ({1}) = Start'.format(name, vmid))
+    log.debug('Start: %s (%s) = Start', name, vmid)
     if not set_vm_status('start', name, vmid=vmid):
-        log.error('Unable to bring VM {0} ({1}) up..'.format(name, vmid))
+        log.error('Unable to bring VM %s (%s) up..', name, vmid)
         raise SaltCloudExecutionFailure
 
     # xxx: TBD: Check here whether the status was actually changed to 'started'
@@ -1019,7 +1013,7 @@ def stop(name, vmid=None, call=None):
         )
 
     if not set_vm_status('stop', name, vmid=vmid):
-        log.error('Unable to bring VM {0} ({1}) down..'.format(name, vmid))
+        log.error('Unable to bring VM %s (%s) down..', name, vmid)
         raise SaltCloudExecutionFailure
 
     # xxx: TBD: Check here whether the status was actually changed to 'stopped'
@@ -1043,7 +1037,7 @@ def shutdown(name=None, vmid=None, call=None):
         )
 
     if not set_vm_status('shutdown', name, vmid=vmid):
-        log.error('Unable to shut VM {0} ({1}) down..'.format(name, vmid))
+        log.error('Unable to shut VM %s (%s) down..', name, vmid)
         raise SaltCloudExecutionFailure
 
     # xxx: TBD: Check here whether the status was actually changed to 'stopped'
