@@ -4,7 +4,7 @@ File server pluggable modules and generic backend functions
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import collections
 import errno
 import fnmatch
@@ -15,12 +15,16 @@ import time
 
 # Import salt libs
 import salt.loader
-import salt.utils
-import salt.utils.locales
+import salt.utils.data
+import salt.utils.files
+import salt.utils.path
+import salt.utils.url
+import salt.utils.versions
 from salt.utils.args import get_function_argspec as _argspec
+from salt.utils.decorators import ensure_unicode_args
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 
 
 log = logging.getLogger(__name__)
@@ -38,7 +42,7 @@ def _unlock_cache(w_lock):
         elif os.path.isfile(w_lock):
             os.unlink(w_lock)
     except (OSError, IOError) as exc:
-        log.trace('Error removing lockfile {0}: {1}'.format(w_lock, exc))
+        log.trace('Error removing lockfile %s: %s', w_lock, exc)
 
 
 def _lock_cache(w_lock):
@@ -49,7 +53,7 @@ def _lock_cache(w_lock):
             raise
         return False
     else:
-        log.trace('Lockfile {0} created'.format(w_lock))
+        log.trace('Lockfile %s created', w_lock)
         return True
 
 
@@ -92,9 +96,10 @@ def wait_lock(lk_fn, dest, wait_timeout=0):
         if timeout:
             if time.time() > timeout:
                 raise ValueError(
-                    'Timeout({0}s) for {1} '
-                    '(lock: {2}) elapsed'.format(
-                        wait_timeout, dest, lk_fn))
+                    'Timeout({0}s) for {1} (lock: {2}) elapsed'.format(
+                        wait_timeout, dest, lk_fn
+                    )
+                )
     return False
 
 
@@ -126,10 +131,12 @@ def check_file_list_cache(opts, form, list_cache, w_lock):
                     age = opts.get('fileserver_list_cache_time', 20) + 1
                 if age < opts.get('fileserver_list_cache_time', 20):
                     # Young enough! Load this sucker up!
-                    with salt.utils.fopen(list_cache, 'rb') as fp_:
-                        log.trace('Returning file_lists cache data from '
-                                  '{0}'.format(list_cache))
-                        return serial.load(fp_).get(form, []), False, False
+                    with salt.utils.files.fopen(list_cache, 'rb') as fp_:
+                        log.trace(
+                            'Returning file_lists cache data from %s',
+                            list_cache
+                        )
+                        return salt.utils.data.decode(serial.load(fp_).get(form, [])), False, False
                 elif _lock_cache(w_lock):
                     # Set the w_lock and go
                     refresh_cache = True
@@ -151,10 +158,10 @@ def write_file_list_cache(opts, data, list_cache, w_lock):
     backend to determine if the cache needs to be refreshed/written).
     '''
     serial = salt.payload.Serial(opts)
-    with salt.utils.fopen(list_cache, 'w+b') as fp_:
+    with salt.utils.files.fopen(list_cache, 'w+b') as fp_:
         fp_.write(serial.dumps(data))
         _unlock_cache(w_lock)
-        log.trace('Lockfile {0} removed'.format(w_lock))
+        log.trace('Lockfile %s removed', w_lock)
 
 
 def check_env_cache(opts, env_cache):
@@ -164,10 +171,10 @@ def check_env_cache(opts, env_cache):
     if not os.path.isfile(env_cache):
         return None
     try:
-        with salt.utils.fopen(env_cache, 'rb') as fp_:
-            log.trace('Returning env cache data from {0}'.format(env_cache))
+        with salt.utils.files.fopen(env_cache, 'rb') as fp_:
+            log.trace('Returning env cache data from %s', env_cache)
             serial = salt.payload.Serial(opts)
-            return serial.load(fp_)
+            return salt.utils.data.decode(serial.load(fp_))
     except (IOError, OSError):
         pass
     return None
@@ -180,7 +187,7 @@ def generate_mtime_map(opts, path_map):
     file_map = {}
     for saltenv, path_list in six.iteritems(path_map):
         for path in path_list:
-            for directory, dirnames, filenames in os.walk(path):
+            for directory, dirnames, filenames in salt.utils.path.os_walk(path):
                 # Don't walk any directories that match file_ignore_regex or glob
                 dirnames[:] = [d for d in dirnames if not is_file_ignored(opts, d)]
                 for item in filenames:
@@ -190,8 +197,9 @@ def generate_mtime_map(opts, path_map):
                     except (OSError, IOError):
                         # skip dangling symlinks
                         log.info(
-                            'Failed to get mtime on {0}, '
-                            'dangling symlink ?'.format(file_path))
+                            'Failed to get mtime on %s, dangling symlink?',
+                            file_path
+                        )
                         continue
     return file_map
 
@@ -223,7 +231,7 @@ def reap_fileserver_cache_dir(cache_base, find_func):
     '''
     for saltenv in os.listdir(cache_base):
         env_base = os.path.join(cache_base, saltenv)
-        for root, dirs, files in os.walk(env_base):
+        for root, dirs, files in salt.utils.path.os_walk(env_base):
             # if we have an empty directory, lets cleanup
             # This will only remove the directory on the second time
             # "_reap_cache" is called (which is intentional)
@@ -239,10 +247,10 @@ def reap_fileserver_cache_dir(cache_base, find_func):
                 try:
                     filename, _, hash_type = file_rel_path.rsplit('.', 2)
                 except ValueError:
-                    log.warning((
-                        'Found invalid hash file [{0}] when attempting to reap'
-                        ' cache directory.'
-                    ).format(file_))
+                    log.warning(
+                        'Found invalid hash file [%s] when attempting to reap '
+                        'cache directory', file_
+                    )
                     continue
                 # do we have the file?
                 ret = find_func(filename, saltenv=saltenv)
@@ -262,9 +270,8 @@ def is_file_ignored(opts, fname):
         for regex in opts['file_ignore_regex']:
             if re.search(regex, fname):
                 log.debug(
-                    'File matching file_ignore_regex. Skipping: {0}'.format(
-                        fname
-                    )
+                    'File matching file_ignore_regex. Skipping: %s',
+                    fname
                 )
                 return True
 
@@ -272,9 +279,8 @@ def is_file_ignored(opts, fname):
         for glob in opts['file_ignore_glob']:
             if fnmatch.fnmatch(fname, glob):
                 log.debug(
-                    'File matching file_ignore_glob. Skipping: {0}'.format(
-                        fname
-                    )
+                    'File matching file_ignore_glob. Skipping: %s',
+                    fname
                 )
                 return True
     return False
@@ -320,7 +326,7 @@ class Fileserver(object):
         self.opts = opts
         self.servers = salt.loader.fileserver(opts, opts['fileserver_backend'])
 
-    def _gen_back(self, back):
+    def backends(self, back=None):
         '''
         Return the backend list
         '''
@@ -386,13 +392,13 @@ class Fileserver(object):
         Clear the cache of all of the fileserver backends that support the
         clear_cache function or the named backend(s) only.
         '''
-        back = self._gen_back(back)
+        back = self.backends(back)
         cleared = []
         errors = []
         for fsb in back:
             fstr = '{0}.clear_cache'.format(fsb)
             if fstr in self.servers:
-                log.debug('Clearing {0} fileserver cache'.format(fsb))
+                log.debug('Clearing %s fileserver cache', fsb)
                 failed = self.servers[fstr]()
                 if failed:
                     errors.extend(failed)
@@ -409,7 +415,7 @@ class Fileserver(object):
         information, or a pattern. If the latter, then remotes for which the URL
         matches the pattern will be locked.
         '''
-        back = self._gen_back(back)
+        back = self.backends(back)
         locked = []
         errors = []
         for fsb in back:
@@ -443,7 +449,7 @@ class Fileserver(object):
             If specified, then any remotes which contain the passed string will
             have their lock cleared.
         '''
-        back = self._gen_back(back)
+        back = self.backends(back)
         cleared = []
         errors = []
         for fsb in back:
@@ -461,18 +467,31 @@ class Fileserver(object):
         Update all of the enabled fileserver backends which support the update
         function, or
         '''
-        back = self._gen_back(back)
+        back = self.backends(back)
         for fsb in back:
             fstr = '{0}.update'.format(fsb)
             if fstr in self.servers:
-                log.debug('Updating {0} fileserver cache'.format(fsb))
+                log.debug('Updating %s fileserver cache', fsb)
                 self.servers[fstr]()
 
+    def update_intervals(self, back=None):
+        '''
+        Return the update intervals for all of the enabled fileserver backends
+        which support variable update intervals.
+        '''
+        back = self.backends(back)
+        ret = {}
+        for fsb in back:
+            fstr = '{0}.update_intervals'.format(fsb)
+            if fstr in self.servers:
+                ret[fsb] = self.servers[fstr]()
+        return ret
+
     def envs(self, back=None, sources=False):
         '''
         Return the environments for the named backend or all backends
         '''
-        back = self._gen_back(back)
+        back = self.backends(back)
         ret = set()
         if sources:
             ret = {}
@@ -494,7 +513,7 @@ class Fileserver(object):
         '''
         Initialize the backend, only do so if the fs supports an init function
         '''
-        back = self._gen_back(back)
+        back = self.backends(back)
         for fsb in back:
             fstr = '{0}.init'.format(fsb)
             if fstr in self.servers:
@@ -527,7 +546,9 @@ class Fileserver(object):
         Find the path and return the fnd structure, this structure is passed
         to other backend interfaces.
         '''
-        back = self._gen_back(back)
+        path = salt.utils.stringutils.to_unicode(path)
+        saltenv = salt.utils.stringutils.to_unicode(saltenv)
+        back = self.backends(back)
         kwargs = {}
         fnd = {'path': '',
                'rel': ''}
@@ -551,12 +572,7 @@ class Fileserver(object):
                     kwargs[args[0]] = args[1]
 
         if 'env' in kwargs:
-            salt.utils.warn_until(
-                'Oxygen',
-                'Parameter \'env\' has been detected in the argument list.  This '
-                'parameter is no longer used and has been replaced by \'saltenv\' '
-                'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-                )
+            # "env" is not supported; Use "saltenv".
             kwargs.pop('env')
         if 'saltenv' in kwargs:
             saltenv = kwargs.pop('saltenv')
@@ -581,12 +597,7 @@ class Fileserver(object):
                'dest': ''}
 
         if 'env' in load:
-            salt.utils.warn_until(
-                'Oxygen',
-                'Parameter \'env\' has been detected in the argument list.  This '
-                'parameter is no longer used and has been replaced by \'saltenv\' '
-                'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-                )
+            # "env" is not supported; Use "saltenv".
             load.pop('env')
 
         if 'path' not in load or 'loc' not in load or 'saltenv' not in load:
@@ -607,13 +618,7 @@ class Fileserver(object):
         Common code for hashing and stating files
         '''
         if 'env' in load:
-            salt.utils.warn_until(
-                'Oxygen',
-                'Parameter \'env\' has been detected in the argument list. '
-                'This parameter is no longer used and has been replaced by '
-                '\'saltenv\' as of Salt 2016.11.0. This warning will be removed '
-                'in Salt Oxygen.'
-            )
+            # "env" is not supported; Use "saltenv".
             load.pop('env')
 
         if 'path' not in load or 'saltenv' not in load:
@@ -621,7 +626,7 @@ class Fileserver(object):
         if not isinstance(load['saltenv'], six.string_types):
             load['saltenv'] = six.text_type(load['saltenv'])
 
-        fnd = self.find_file(salt.utils.locales.sdecode(load['path']),
+        fnd = self.find_file(salt.utils.stringutils.to_unicode(load['path']),
                 load['saltenv'])
         if not fnd.get('back'):
             return '', None
@@ -654,12 +659,7 @@ class Fileserver(object):
         Deletes the file_lists cache files
         '''
         if 'env' in load:
-            salt.utils.warn_until(
-                'Oxygen',
-                'Parameter \'env\' has been detected in the argument list.  This '
-                'parameter is no longer used and has been replaced by \'saltenv\' '
-                'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-                )
+            # "env" is not supported; Use "saltenv".
             load.pop('env')
 
         saltenv = load.get('saltenv', [])
@@ -668,14 +668,14 @@ class Fileserver(object):
                 try:
                     saltenv = [x.strip() for x in saltenv.split(',')]
                 except AttributeError:
-                    saltenv = [x.strip() for x in str(saltenv).split(',')]
+                    saltenv = [x.strip() for x in six.text_type(saltenv).split(',')]
 
             for idx, val in enumerate(saltenv):
                 if not isinstance(val, six.string_types):
                     saltenv[idx] = six.text_type(val)
 
         ret = {}
-        fsb = self._gen_back(load.pop('fsbackend', None))
+        fsb = self.backends(load.pop('fsbackend', None))
         list_cachedir = os.path.join(self.opts['cachedir'], 'file_lists')
         try:
             file_list_backends = os.listdir(list_cachedir)
@@ -731,17 +731,13 @@ class Fileserver(object):
                     )
         return ret
 
+    @ensure_unicode_args
     def file_list(self, load):
         '''
         Return a list of files from the dominant environment
         '''
         if 'env' in load:
-            salt.utils.warn_until(
-                'Oxygen',
-                'Parameter \'env\' has been detected in the argument list.  This '
-                'parameter is no longer used and has been replaced by \'saltenv\' '
-                'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-                )
+            # "env" is not supported; Use "saltenv".
             load.pop('env')
 
         ret = set()
@@ -750,29 +746,23 @@ class Fileserver(object):
         if not isinstance(load['saltenv'], six.string_types):
             load['saltenv'] = six.text_type(load['saltenv'])
 
-        for fsb in self._gen_back(load.pop('fsbackend', None)):
+        for fsb in self.backends(load.pop('fsbackend', None)):
             fstr = '{0}.file_list'.format(fsb)
             if fstr in self.servers:
                 ret.update(self.servers[fstr](load))
-        # upgrade all set elements to a common encoding
-        ret = [salt.utils.locales.sdecode(f) for f in ret]
         # some *fs do not handle prefix. Ensure it is filtered
         prefix = load.get('prefix', '').strip('/')
         if prefix != '':
             ret = [f for f in ret if f.startswith(prefix)]
         return sorted(ret)
 
+    @ensure_unicode_args
     def file_list_emptydirs(self, load):
         '''
         List all emptydirs in the given environment
         '''
         if 'env' in load:
-            salt.utils.warn_until(
-                'Oxygen',
-                'Parameter \'env\' has been detected in the argument list.  This '
-                'parameter is no longer used and has been replaced by \'saltenv\' '
-                'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-                )
+            # "env" is not supported; Use "saltenv".
             load.pop('env')
 
         ret = set()
@@ -781,29 +771,23 @@ class Fileserver(object):
         if not isinstance(load['saltenv'], six.string_types):
             load['saltenv'] = six.text_type(load['saltenv'])
 
-        for fsb in self._gen_back(None):
+        for fsb in self.backends(None):
             fstr = '{0}.file_list_emptydirs'.format(fsb)
             if fstr in self.servers:
                 ret.update(self.servers[fstr](load))
-        # upgrade all set elements to a common encoding
-        ret = [salt.utils.locales.sdecode(f) for f in ret]
         # some *fs do not handle prefix. Ensure it is filtered
         prefix = load.get('prefix', '').strip('/')
         if prefix != '':
             ret = [f for f in ret if f.startswith(prefix)]
         return sorted(ret)
 
+    @ensure_unicode_args
     def dir_list(self, load):
         '''
         List all directories in the given environment
         '''
         if 'env' in load:
-            salt.utils.warn_until(
-                'Oxygen',
-                'Parameter \'env\' has been detected in the argument list.  This '
-                'parameter is no longer used and has been replaced by \'saltenv\' '
-                'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-                )
+            # "env" is not supported; Use "saltenv".
             load.pop('env')
 
         ret = set()
@@ -812,29 +796,23 @@ class Fileserver(object):
         if not isinstance(load['saltenv'], six.string_types):
             load['saltenv'] = six.text_type(load['saltenv'])
 
-        for fsb in self._gen_back(load.pop('fsbackend', None)):
+        for fsb in self.backends(load.pop('fsbackend', None)):
             fstr = '{0}.dir_list'.format(fsb)
             if fstr in self.servers:
                 ret.update(self.servers[fstr](load))
-        # upgrade all set elements to a common encoding
-        ret = [salt.utils.locales.sdecode(f) for f in ret]
         # some *fs do not handle prefix. Ensure it is filtered
         prefix = load.get('prefix', '').strip('/')
         if prefix != '':
             ret = [f for f in ret if f.startswith(prefix)]
         return sorted(ret)
 
+    @ensure_unicode_args
     def symlink_list(self, load):
         '''
         Return a list of symlinked files and dirs
         '''
         if 'env' in load:
-            salt.utils.warn_until(
-                'Oxygen',
-                'Parameter \'env\' has been detected in the argument list.  This '
-                'parameter is no longer used and has been replaced by \'saltenv\' '
-                'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-                )
+            # "env" is not supported; Use "saltenv".
             load.pop('env')
 
         ret = {}
@@ -843,14 +821,10 @@ class Fileserver(object):
         if not isinstance(load['saltenv'], six.string_types):
             load['saltenv'] = six.text_type(load['saltenv'])
 
-        for fsb in self._gen_back(load.pop('fsbackend', None)):
+        for fsb in self.backends(load.pop('fsbackend', None)):
             symlstr = '{0}.symlink_list'.format(fsb)
             if symlstr in self.servers:
                 ret = self.servers[symlstr](load)
-        # upgrade all set elements to a common encoding
-        ret = dict([
-            (salt.utils.locales.sdecode(x), salt.utils.locales.sdecode(y)) for x, y in ret.items()
-        ])
         # some *fs do not handle prefix. Ensure it is filtered
         prefix = load.get('prefix', '').strip('/')
         if prefix != '':
@@ -876,14 +850,14 @@ class FSChan(object):
                 self.opts['__fs_update'] = True
         else:
             self.fs.update()
-        self.cmd_stub = {'ext_nodes': {}}
+        self.cmd_stub = {'master_tops': {}}
 
     def send(self, load, tries=None, timeout=None, raw=False):  # pylint: disable=unused-argument
         '''
         Emulate the channel send method, the tries and timeout are not used
         '''
         if 'cmd' not in load:
-            log.error('Malformed request, no cmd: {0}'.format(load))
+            log.error('Malformed request, no cmd: %s', load)
             return {}
         cmd = load['cmd'].lstrip('_')
         if cmd in self.cmd_stub:
@@ -891,6 +865,6 @@ class FSChan(object):
         if cmd == 'file_envs':
             return self.fs.envs()
         if not hasattr(self.fs, cmd):
-            log.error('Malformed request, invalid cmd: {0}'.format(load))
+            log.error('Malformed request, invalid cmd: %s', load)
             return {}
         return getattr(self.fs, cmd)(load)
