@@ -2,6 +2,8 @@
 '''
 Amazon S3 Fileserver Backend
 
+.. versionadded:: 0.16.0
+
 This backend exposes directories in S3 buckets as Salt environments. To enable
 this backend, add ``s3fs`` to the :conf_master:`fileserver_backend` option in the
 Master config file.
@@ -58,7 +60,7 @@ structure::
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import datetime
 import os
 import time
@@ -68,11 +70,14 @@ import logging
 # Import salt libs
 import salt.fileserver as fs
 import salt.modules
-import salt.utils
+import salt.utils.files
+import salt.utils.gzip_util
+import salt.utils.hashutils
+import salt.utils.versions
 
 # Import 3rd-party libs
 # pylint: disable=import-error,no-name-in-module,redefined-builtin
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import filter
 from salt.ext.six.moves.urllib.parse import quote as _quote
 # pylint: enable=import-error,no-name-in-module,redefined-builtin
@@ -108,7 +113,7 @@ def update():
             for bucket, files in six.iteritems(_find_files(env_meta)):
                 for file_path in files:
                     cached_file_path = _get_cached_file_name(bucket, saltenv, file_path)
-                    log.info('{0} - {1} : {2}'.format(bucket, saltenv, file_path))
+                    log.info('%s - %s : %s', bucket, saltenv, file_path)
 
                     # load the file from S3 if it's not in the cache or it's old
                     _get_file_from_s3(metadata, saltenv, bucket, file_path, cached_file_path)
@@ -123,12 +128,7 @@ def find_file(path, saltenv='base', **kwargs):
     is missing, or if the MD5 does not match.
     '''
     if 'env' in kwargs:
-        salt.utils.warn_until(
-            'Oxygen',
-            'Parameter \'env\' has been detected in the argument list.  This '
-            'parameter is no longer used and has been replaced by \'saltenv\' '
-            'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-            )
+        # "env" is not supported; Use "saltenv".
         kwargs.pop('env')
 
     fnd = {'bucket': None,
@@ -165,12 +165,7 @@ def file_hash(load, fnd):
     Return an MD5 file hash
     '''
     if 'env' in load:
-        salt.utils.warn_until(
-            'Oxygen',
-            'Parameter \'env\' has been detected in the argument list.  This '
-            'parameter is no longer used and has been replaced by \'saltenv\' '
-            'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-            )
+        # "env" is not supported; Use "saltenv".
         load.pop('env')
 
     ret = {}
@@ -187,7 +182,7 @@ def file_hash(load, fnd):
             fnd['path'])
 
     if os.path.isfile(cached_file_path):
-        ret['hsum'] = salt.utils.get_hash(cached_file_path)
+        ret['hsum'] = salt.utils.hashutils.get_hash(cached_file_path)
         ret['hash_type'] = 'md5'
 
     return ret
@@ -198,12 +193,7 @@ def serve_file(load, fnd):
     Return a chunk from a file based on the data received
     '''
     if 'env' in load:
-        salt.utils.warn_until(
-            'Oxygen',
-            'Parameter \'env\' has been detected in the argument list.  This '
-            'parameter is no longer used and has been replaced by \'saltenv\' '
-            'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-            )
+        # "env" is not supported; Use "saltenv".
         load.pop('env')
 
     ret = {'data': '',
@@ -225,9 +215,11 @@ def serve_file(load, fnd):
 
     ret['dest'] = _trim_env_off_path([fnd['path']], load['saltenv'])[0]
 
-    with salt.utils.fopen(cached_file_path, 'rb') as fp_:
+    with salt.utils.files.fopen(cached_file_path, 'rb') as fp_:
         fp_.seek(load['loc'])
         data = fp_.read(__opts__['file_buffer_size'])
+        if data and six.PY3 and not salt.utils.files.is_binary(cached_file_path):
+            data = data.decode(__salt_system_encoding__)
         if gzip and data:
             data = salt.utils.gzip_util.compress(data, gzip)
             ret['gzip'] = gzip
@@ -240,12 +232,7 @@ def file_list(load):
     Return a list of all files on the file server in a specified environment
     '''
     if 'env' in load:
-        salt.utils.warn_until(
-            'Oxygen',
-            'Parameter \'env\' has been detected in the argument list.  This '
-            'parameter is no longer used and has been replaced by \'saltenv\' '
-            'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-            )
+        # "env" is not supported; Use "saltenv".
         load.pop('env')
 
     ret = []
@@ -281,12 +268,7 @@ def dir_list(load):
     Return a list of all directories on the master
     '''
     if 'env' in load:
-        salt.utils.warn_until(
-            'Oxygen',
-            'Parameter \'env\' has been detected in the argument list.  This '
-            'parameter is no longer used and has been replaced by \'saltenv\' '
-            'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-            )
+        # "env" is not supported; Use "saltenv".
         load.pop('env')
 
     ret = []
@@ -327,8 +309,14 @@ def _get_s3_key():
     location = __opts__['s3.location'] \
         if 's3.location' in __opts__ \
         else None
+    path_style = __opts__['s3.path_style'] \
+        if 's3.path_style' in __opts__ \
+        else None
+    https_enable = __opts__['s3.https_enable'] \
+        if 's3.https_enable' in __opts__ \
+        else None
 
-    return key, keyid, service_url, verify_ssl, kms_keyid, location
+    return key, keyid, service_url, verify_ssl, kms_keyid, location, path_style, https_enable
 
 
 def _init():
@@ -398,17 +386,23 @@ def _refresh_buckets_cache_file(cache_file):
 
     log.debug('Refreshing buckets cache file')
 
-    key, keyid, service_url, verify_ssl, kms_keyid, location = _get_s3_key()
+    key, keyid, service_url, verify_ssl, kms_keyid, location, path_style, https_enable = _get_s3_key()
     metadata = {}
 
     # helper s3 query function
     def __get_s3_meta(bucket, key=key, keyid=keyid):
         ret, marker = [], ''
         while True:
-            tmp = __utils__['s3.query'](key=key, keyid=keyid, kms_keyid=keyid,
-                                        bucket=bucket, service_url=service_url,
+            tmp = __utils__['s3.query'](key=key,
+                                        keyid=keyid,
+                                        kms_keyid=keyid,
+                                        bucket=bucket,
+                                        service_url=service_url,
                                         verify_ssl=verify_ssl,
-                                        location=location, return_bin=False,
+                                        location=location,
+                                        return_bin=False,
+                                        path_style=path_style,
+                                        https_enable=https_enable,
                                         params={'marker': marker})
             headers = []
             for header in tmp:
@@ -444,15 +438,15 @@ def _refresh_buckets_cache_file(cache_file):
                             meta_response.update(k)
                     # attempt use of human readable output first.
                     try:
-                        log.warning("'{0}' response for bucket '{1}'".format(meta_response['Message'], bucket_name))
+                        log.warning("'%s' response for bucket '%s'", meta_response['Message'], bucket_name)
                         continue
                     except KeyError:
                         # no human readable error message provided
                         if 'Code' in meta_response:
                             log.warning(
-                                ("'{0}' response for "
-                                "bucket '{1}'").format(meta_response['Code'],
-                                                       bucket_name))
+                                "'%s' response for bucket '%s'",
+                                meta_response['Code'], bucket_name
+                            )
                             continue
                         else:
                             log.warning(
@@ -483,15 +477,15 @@ def _refresh_buckets_cache_file(cache_file):
                         meta_response.update(k)
                 # attempt use of human readable output first.
                 try:
-                    log.warning("'{0}' response for bucket '{1}'".format(meta_response['Message'], bucket_name))
+                    log.warning("'%s' response for bucket '%s'", meta_response['Message'], bucket_name)
                     continue
                 except KeyError:
                     # no human readable error message provided
                     if 'Code' in meta_response:
                         log.warning(
-                            ("'{0}' response for "
-                            "bucket '{1}'").format(meta_response['Code'],
-                                                   bucket_name))
+                            "'%s' response for bucket '%s'",
+                            meta_response['Code'], bucket_name
+                        )
                         continue
                     else:
                         log.warning(
@@ -521,7 +515,7 @@ def _refresh_buckets_cache_file(cache_file):
 
     log.debug('Writing buckets cache file')
 
-    with salt.utils.fopen(cache_file, 'w') as fp_:
+    with salt.utils.files.fopen(cache_file, 'w') as fp_:
         pickle.dump(metadata, fp_)
 
     return metadata
@@ -534,7 +528,7 @@ def _read_buckets_cache_file(cache_file):
 
     log.debug('Reading buckets cache file')
 
-    with salt.utils.fopen(cache_file, 'rb') as fp_:
+    with salt.utils.files.fopen(cache_file, 'rb') as fp_:
         try:
             data = pickle.load(fp_)
         except (pickle.UnpicklingError, AttributeError, EOFError, ImportError,
@@ -617,7 +611,7 @@ def _get_file_from_s3(metadata, saltenv, bucket_name, path, cached_file_path):
     Checks the local cache for the file, if it's old or missing go grab the
     file from S3 and update the cache
     '''
-    key, keyid, service_url, verify_ssl, kms_keyid, location = _get_s3_key()
+    key, keyid, service_url, verify_ssl, kms_keyid, location, path_style, https_enable = _get_s3_key()
 
     # check the local cache...
     if os.path.isfile(cached_file_path):
@@ -627,7 +621,7 @@ def _get_file_from_s3(metadata, saltenv, bucket_name, path, cached_file_path):
 
             if file_etag.find('-') == -1:
                 file_md5 = file_etag
-                cached_md5 = salt.utils.get_hash(cached_file_path, 'md5')
+                cached_md5 = salt.utils.hashutils.get_hash(cached_file_path, 'md5')
 
                 # hashes match we have a cache hit
                 if cached_md5 == file_md5:
@@ -656,23 +650,26 @@ def _get_file_from_s3(metadata, saltenv, bucket_name, path, cached_file_path):
                         location=location,
                         path=_quote(path),
                         local_file=cached_file_path,
-                        full_headers=True
+                        full_headers=True,
+                        path_style=path_style,
+                        https_enable=https_enable
                     )
                     if ret is not None:
                         for header_name, header_value in ret['headers'].items():
                             name = header_name.strip()
                             value = header_value.strip()
-                            if str(name).lower() == 'last-modified':
+                            if six.text_type(name).lower() == 'last-modified':
                                 s3_file_mtime = datetime.datetime.strptime(
                                     value, '%a, %d %b %Y %H:%M:%S %Z')
-                            elif str(name).lower() == 'content-length':
+                            elif six.text_type(name).lower() == 'content-length':
                                 s3_file_size = int(value)
                         if (cached_file_size == s3_file_size and
                                 cached_file_mtime > s3_file_mtime):
                             log.info(
-                                '{0} - {1} : {2} skipped download since cached file size '
-                                'equal to and mtime after s3 values'.format(
-                                    bucket_name, saltenv, path))
+                                '%s - %s : %s skipped download since cached file size '
+                                'equal to and mtime after s3 values',
+                                bucket_name, saltenv, path
+                            )
                             return
 
     # ... or get the file from S3
@@ -685,7 +682,9 @@ def _get_file_from_s3(metadata, saltenv, bucket_name, path, cached_file_path):
         verify_ssl=verify_ssl,
         location=location,
         path=_quote(path),
-        local_file=cached_file_path
+        local_file=cached_file_path,
+        path_style=path_style,
+        https_enable=https_enable,
     )
 
 
