@@ -4,10 +4,11 @@ A module to wrap (non-Windows) archive calls
 
 .. versionadded:: 2014.1.0
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import contextlib  # For < 2.7 compat
 import copy
 import errno
+import glob
 import logging
 import os
 import re
@@ -21,6 +22,9 @@ try:
 except ImportError:
     from pipes import quote as _quote
 
+# Import third party libs
+from salt.ext import six
+from salt.ext.six.moves.urllib.parse import urlparse as _urlparse  # pylint: disable=no-name-in-module
 try:
     import rarfile
     HAS_RARFILE = True
@@ -29,10 +33,16 @@ except ImportError:
 
 # Import salt libs
 from salt.exceptions import SaltInvocationError, CommandExecutionError
-from salt.ext.six import string_types, integer_types
-from salt.ext.six.moves.urllib.parse import urlparse as _urlparse  # pylint: disable=no-name-in-module
-import salt.utils
-import salt.utils.itertools
+import salt.utils.decorators
+import salt.utils.decorators.path
+import salt.utils.files
+import salt.utils.path
+import salt.utils.platform
+import salt.utils.stringutils
+import salt.utils.templates
+
+if salt.utils.platform.is_windows():
+    import win32file
 
 # TODO: Check that the passed arguments are correct
 
@@ -51,7 +61,8 @@ def list_(name,
           strip_components=None,
           clean=False,
           verbose=False,
-          saltenv='base'):
+          saltenv='base',
+          source_hash=None):
     '''
     .. versionadded:: 2016.11.0
     .. versionchanged:: 2016.11.2
@@ -140,6 +151,14 @@ def list_(name,
         ``archive``. This is only applicable when ``archive`` is a file from
         the ``salt://`` fileserver.
 
+    source_hash
+        If ``name`` is an http(s)/ftp URL and the file exists in the minion's
+        file cache, this option can be passed to keep the minion from
+        re-downloading the archive if the cached copy matches the specified
+        hash.
+
+        .. versionadded:: 2018.3.0
+
     .. _tarfile: https://docs.python.org/2/library/tarfile.html
     .. _xz: http://tukaani.org/xz/
 
@@ -151,9 +170,13 @@ def list_(name,
             salt '*' archive.list /path/to/myfile.tar.gz strip_components=1
             salt '*' archive.list salt://foo.tar.gz
             salt '*' archive.list https://domain.tld/myfile.zip
+            salt '*' archive.list https://domain.tld/myfile.zip source_hash=f1d2d2f924e986ac86fdf7b36c94bcdf32beec15
             salt '*' archive.list ftp://10.1.2.3/foo.rar
     '''
     def _list_tar(name, cached, decompress_cmd, failhard=False):
+        '''
+        List the contents of a tar archive.
+        '''
         dirs = []
         files = []
         links = []
@@ -181,7 +204,7 @@ def list_(name,
                             info={'error': stderr}
                         )
             else:
-                if not salt.utils.which('tar'):
+                if not salt.utils.path.which('tar'):
                     raise CommandExecutionError('\'tar\' command not available')
                 if decompress_cmd is not None:
                     # Guard against shell injection
@@ -192,8 +215,8 @@ def list_(name,
                     except AttributeError:
                         raise CommandExecutionError('Invalid CLI options')
                 else:
-                    if salt.utils.which('xz') \
-                            and __salt__['cmd.retcode'](['xz', '-l', cached],
+                    if salt.utils.path.which('xz') \
+                            and __salt__['cmd.retcode'](['xz', '-t', cached],
                                                         python_shell=False,
                                                         ignore_retcode=True) == 0:
                         decompress_cmd = 'xz --decompress --stdout'
@@ -216,6 +239,7 @@ def list_(name,
 
     def _list_zip(name, cached):
         '''
+        List the contents of a zip archive.
         Password-protected ZIP archives can still be listed by zipfile, so
         there is no reason to invoke the unzip command.
         '''
@@ -226,7 +250,7 @@ def list_(name,
             with contextlib.closing(zipfile.ZipFile(cached)) as zip_archive:
                 for member in zip_archive.infolist():
                     path = member.filename
-                    if salt.utils.is_windows():
+                    if salt.utils.platform.is_windows():
                         if path.endswith('/'):
                             # zipfile.ZipInfo objects on windows use forward
                             # slash at end of the directory name.
@@ -257,6 +281,9 @@ def list_(name,
             raise CommandExecutionError('{0} is not a ZIP file'.format(name))
 
     def _list_rar(name, cached):
+        '''
+        List the contents of a rar archive.
+        '''
         dirs = []
         files = []
         if HAS_RARFILE:
@@ -268,7 +295,7 @@ def list_(name,
                     else:
                         files.append(path)
         else:
-            if not salt.utils.which('rar'):
+            if not salt.utils.path.which('rar'):
                 raise CommandExecutionError(
                     'rar command not available, is it installed?'
                 )
@@ -293,7 +320,7 @@ def list_(name,
                 )
         return dirs, files, []
 
-    cached = __salt__['cp.cache_file'](name, saltenv)
+    cached = __salt__['cp.cache_file'](name, saltenv, source_hash=source_hash)
     if not cached:
         raise CommandExecutionError('Failed to cache {0}'.format(name))
 
@@ -313,6 +340,9 @@ def list_(name,
         path = parsed.path or parsed.netloc
 
         def _unsupported_format(archive_format):
+            '''
+            Raise the proper exception message for the given archive format.
+            '''
             if archive_format is None:
                 raise CommandExecutionError(
                     'Unable to guess archive format, please pass an '
@@ -401,7 +431,37 @@ def list_(name,
         raise CommandExecutionError(exc.error, info=info)
 
 
-@salt.utils.decorators.which('tar')
+_glob_wildcards = re.compile('[*?[]')
+
+
+def _glob(pathname):
+    '''
+    In case ``pathname`` contains glob wildcards, performs expansion and returns
+    the possibly empty list of matching pathnames. Otherwise returns a list that
+    contains only ``pathname`` itself.
+    '''
+    if _glob_wildcards.search(pathname) is None:
+        return [pathname]
+    else:
+        return glob.glob(pathname)
+
+
+def _expand_sources(sources):
+    '''
+    Expands a user-provided specification of source files into a list of paths.
+    '''
+    if sources is None:
+        return []
+    if isinstance(sources, six.string_types):
+        sources = [x.strip() for x in sources.split(',')]
+    elif isinstance(sources, (float, six.integer_types)):
+        sources = [six.text_type(sources)]
+    return [path
+            for source in sources
+            for path in _glob(source)]
+
+
+@salt.utils.decorators.path.which('tar')
 def tar(options, tarfile, sources=None, dest=None,
         cwd=None, template=None, runas=None):
     '''
@@ -437,6 +497,9 @@ def tar(options, tarfile, sources=None, dest=None,
         Comma delimited list of files to **pack** into the tarfile. Can also be
         passed as a Python list.
 
+        .. versionchanged:: 2017.7.0
+            Globbing is now supported for this argument
+
     dest
         The destination directory into which to **unpack** the tarfile
 
@@ -451,14 +514,16 @@ def tar(options, tarfile, sources=None, dest=None,
 
         .. code-block:: bash
 
-            salt '*' archive.tar -cjvf /tmp/salt.tar.bz2 {{grains.saltpath}} template=jinja
+            salt '*' archive.tar cjvf /tmp/salt.tar.bz2 {{grains.saltpath}} template=jinja
 
     CLI Examples:
 
     .. code-block:: bash
 
         # Create a tarfile
-        salt '*' archive.tar -cjvf /tmp/tarfile.tar.bz2 /tmp/file_1,/tmp/file_2
+        salt '*' archive.tar cjvf /tmp/tarfile.tar.bz2 /tmp/file_1,/tmp/file_2
+        # Create a tarfile using globbing (2017.7.0 and later)
+        salt '*' archive.tar cjvf /tmp/tarfile.tar.bz2 '/tmp/file_*'
         # Unpack a tarfile
         salt '*' archive.tar xf foo.tar dest=/target/directory
     '''
@@ -468,20 +533,14 @@ def tar(options, tarfile, sources=None, dest=None,
         # should at least let them know of their silliness.
         raise SaltInvocationError('Tar options can not be empty')
 
-    if isinstance(sources, string_types):
-        sources = [s.strip() for s in sources.split(',')]
-
     cmd = ['tar']
-    if dest:
-        cmd.extend(['-C', '{0}'.format(dest)])
-
     if options:
         cmd.extend(options.split())
 
     cmd.extend(['{0}'.format(tarfile)])
-
-    if sources is not None:
-        cmd.extend(sources)
+    cmd.extend(_expand_sources(sources))
+    if dest:
+        cmd.extend(['-C', '{0}'.format(dest)])
 
     return __salt__['cmd.run'](cmd,
                                cwd=cwd,
@@ -490,7 +549,7 @@ def tar(options, tarfile, sources=None, dest=None,
                                python_shell=False).splitlines()
 
 
-@salt.utils.decorators.which('gzip')
+@salt.utils.decorators.path.which('gzip')
 def gzip(sourcefile, template=None, runas=None, options=None):
     '''
     Uses the gzip command to create gzip files
@@ -530,7 +589,7 @@ def gzip(sourcefile, template=None, runas=None, options=None):
                                python_shell=False).splitlines()
 
 
-@salt.utils.decorators.which('gunzip')
+@salt.utils.decorators.path.which('gunzip')
 def gunzip(gzipfile, template=None, runas=None, options=None):
     '''
     Uses the gunzip command to unpack gzip files
@@ -570,7 +629,7 @@ def gunzip(gzipfile, template=None, runas=None, options=None):
                                python_shell=False).splitlines()
 
 
-@salt.utils.decorators.which('zip')
+@salt.utils.decorators.path.which('zip')
 def cmd_zip(zip_file, sources, template=None, cwd=None, runas=None):
     '''
     .. versionadded:: 2015.5.0
@@ -589,6 +648,9 @@ def cmd_zip(zip_file, sources, template=None, cwd=None, runas=None):
         Comma-separated list of sources to include in the zip file. Sources can
         also be passed in a Python list.
 
+        .. versionchanged:: 2017.7.0
+            Globbing is now supported for this argument
+
     template : None
         Can be set to 'jinja' or another supported template engine to render
         the command arguments before execution:
@@ -623,12 +685,12 @@ def cmd_zip(zip_file, sources, template=None, cwd=None, runas=None):
     .. code-block:: bash
 
         salt '*' archive.cmd_zip /tmp/zipfile.zip /tmp/sourcefile1,/tmp/sourcefile2
+        # Globbing for sources (2017.7.0 and later)
+        salt '*' archive.cmd_zip /tmp/zipfile.zip '/tmp/sourcefile*'
     '''
-    if isinstance(sources, string_types):
-        sources = [s.strip() for s in sources.split(',')]
     cmd = ['zip', '-r']
     cmd.append('{0}'.format(zip_file))
-    cmd.extend(sources)
+    cmd.extend(_expand_sources(sources))
     return __salt__['cmd.run'](cmd,
                                cwd=cwd,
                                template=template,
@@ -655,6 +717,9 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
         Comma-separated list of sources to include in the zip file. Sources can
         also be passed in a Python list.
 
+        .. versionchanged:: 2017.7.0
+            Globbing is now supported for this argument
+
     template : None
         Can be set to 'jinja' or another supported template engine to render
         the command arguments before execution:
@@ -685,6 +750,8 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
     .. code-block:: bash
 
         salt '*' archive.zip /tmp/zipfile.zip /tmp/sourcefile1,/tmp/sourcefile2
+        # Globbing for sources (2017.7.0 and later)
+        salt '*' archive.zip /tmp/zipfile.zip '/tmp/sourcefile*'
     '''
     if runas:
         euid = os.geteuid()
@@ -696,11 +763,7 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
             )
 
     zip_file, sources = _render_filenames(zip_file, sources, None, template)
-
-    if isinstance(sources, string_types):
-        sources = [x.strip() for x in sources.split(',')]
-    elif isinstance(sources, (float, integer_types)):
-        sources = [str(sources)]
+    sources = _expand_sources(sources)
 
     if not cwd:
         for src in sources:
@@ -709,13 +772,12 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
                     'Relative paths require the \'cwd\' parameter'
                 )
     else:
-        def _bad_cwd():
-            raise SaltInvocationError('cwd must be absolute')
+        err_msg = 'cwd must be absolute'
         try:
             if not os.path.isabs(cwd):
-                _bad_cwd()
+                raise SaltInvocationError(err_msg)
         except AttributeError:
-            _bad_cwd()
+            raise SaltInvocationError(err_msg)
 
     if runas and (euid != uinfo['uid'] or egid != uinfo['gid']):
         # Change the egid first, as changing it after the euid will fail
@@ -736,12 +798,11 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
                     else:
                         rel_root = cwd if cwd is not None else '/'
                     if os.path.isdir(src):
-                        for dir_name, sub_dirs, files in os.walk(src):
+                        for dir_name, sub_dirs, files in salt.utils.path.os_walk(src):
                             if cwd and dir_name.startswith(cwd):
-                                arc_dir = salt.utils.relpath(dir_name, cwd)
+                                arc_dir = os.path.relpath(dir_name, cwd)
                             else:
-                                arc_dir = salt.utils.relpath(dir_name,
-                                                             rel_root)
+                                arc_dir = os.path.relpath(dir_name, rel_root)
                             if arc_dir:
                                 archived_files.append(arc_dir + '/')
                                 zfile.write(dir_name, arc_dir)
@@ -752,9 +813,9 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
                                 zfile.write(abs_name, arc_name)
                     else:
                         if cwd and src.startswith(cwd):
-                            arc_name = salt.utils.relpath(src, cwd)
+                            arc_name = os.path.relpath(src, cwd)
                         else:
-                            arc_name = salt.utils.relpath(src, rel_root)
+                            arc_name = os.path.relpath(src, rel_root)
                         archived_files.append(arc_name)
                         zfile.write(src, arc_name)
     except Exception as exc:
@@ -774,7 +835,7 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
     return archived_files
 
 
-@salt.utils.decorators.which('unzip')
+@salt.utils.decorators.path.which('unzip')
 def cmd_unzip(zip_file,
               dest,
               excludes=None,
@@ -851,10 +912,10 @@ def cmd_unzip(zip_file,
 
         salt '*' archive.cmd_unzip /tmp/zipfile.zip /home/strongbad/ excludes=file_1,file_2
     '''
-    if isinstance(excludes, string_types):
+    if isinstance(excludes, six.string_types):
         excludes = [x.strip() for x in excludes.split(',')]
-    elif isinstance(excludes, (float, integer_types)):
-        excludes = [str(excludes)]
+    elif isinstance(excludes, (float, six.integer_types)):
+        excludes = [six.text_type(excludes)]
 
     cmd = ['unzip']
     if password:
@@ -989,7 +1050,6 @@ def unzip(zip_file,
         os.seteuid(uinfo['uid'])
 
     try:
-        exc = None
         # Define cleaned_files here so that an exception will not prevent this
         # variable from being defined and cause a NameError in the return
         # statement at the end of the function.
@@ -997,15 +1057,15 @@ def unzip(zip_file,
         with contextlib.closing(zipfile.ZipFile(zip_file, "r")) as zfile:
             files = zfile.namelist()
 
-            if isinstance(excludes, string_types):
+            if isinstance(excludes, six.string_types):
                 excludes = [x.strip() for x in excludes.split(',')]
-            elif isinstance(excludes, (float, integer_types)):
-                excludes = [str(excludes)]
+            elif isinstance(excludes, (float, six.integer_types)):
+                excludes = [six.text_type(excludes)]
 
             cleaned_files.extend([x for x in files if x not in excludes])
             for target in cleaned_files:
                 if target not in excludes:
-                    if salt.utils.is_windows() is False:
+                    if salt.utils.platform.is_windows() is False:
                         info = zfile.getinfo(target)
                         # Check if zipped file is a symbolic link
                         if stat.S_ISLNK(info.external_attr >> 16):
@@ -1014,33 +1074,38 @@ def unzip(zip_file,
                             continue
                     zfile.extract(target, dest, password)
                     if extract_perms:
-                        perm = zfile.getinfo(target).external_attr >> 16
-                        if perm == 0:
-                            umask_ = os.umask(0)
-                            os.umask(umask_)
-                            if target.endswith('/'):
-                                perm = 0o777 & ~umask_
-                            else:
-                                perm = 0o666 & ~umask_
-                        os.chmod(os.path.join(dest, target), perm)
+                        if not salt.utils.platform.is_windows():
+                            perm = zfile.getinfo(target).external_attr >> 16
+                            if perm == 0:
+                                umask_ = os.umask(0)
+                                os.umask(umask_)
+                                if target.endswith('/'):
+                                    perm = 0o777 & ~umask_
+                                else:
+                                    perm = 0o666 & ~umask_
+                            os.chmod(os.path.join(dest, target), perm)
+                        else:
+                            win32_attr = zfile.getinfo(target).external_attr & 0xFF
+                            win32file.SetFileAttributes(os.path.join(dest, target), win32_attr)
     except Exception as exc:
-        pass
+        if runas:
+            os.seteuid(euid)
+            os.setegid(egid)
+        # Wait to raise the exception until euid/egid are restored to avoid
+        # permission errors in writing to minion log.
+        raise CommandExecutionError(
+            'Exception encountered unpacking zipfile: {0}'.format(exc)
+        )
     finally:
         # Restore the euid/egid
         if runas:
             os.seteuid(euid)
             os.setegid(egid)
-        if exc is not None:
-            # Wait to raise the exception until euid/egid are restored to avoid
-            # permission errors in writing to minion log.
-            raise CommandExecutionError(
-                'Exception encountered unpacking zipfile: {0}'.format(exc)
-            )
 
     return _trim_files(cleaned_files, trim_output)
 
 
-def is_encrypted(name, clean=False, saltenv='base'):
+def is_encrypted(name, clean=False, saltenv='base', source_hash=None):
     '''
     .. versionadded:: 2016.11.0
 
@@ -1059,6 +1124,18 @@ def is_encrypted(name, clean=False, saltenv='base'):
             If there is an error listing the archive's contents, the cached
             file will not be removed, to allow for troubleshooting.
 
+    saltenv : base
+        Specifies the fileserver environment from which to retrieve
+        ``archive``. This is only applicable when ``archive`` is a file from
+        the ``salt://`` fileserver.
+
+    source_hash
+        If ``name`` is an http(s)/ftp URL and the file exists in the minion's
+        file cache, this option can be passed to keep the minion from
+        re-downloading the archive if the cached copy matches the specified
+        hash.
+
+        .. versionadded:: 2018.3.0
 
     CLI Examples:
 
@@ -1068,9 +1145,10 @@ def is_encrypted(name, clean=False, saltenv='base'):
             salt '*' archive.is_encrypted salt://foo.zip
             salt '*' archive.is_encrypted salt://foo.zip saltenv=dev
             salt '*' archive.is_encrypted https://domain.tld/myfile.zip clean=True
+            salt '*' archive.is_encrypted https://domain.tld/myfile.zip source_hash=f1d2d2f924e986ac86fdf7b36c94bcdf32beec15
             salt '*' archive.is_encrypted ftp://10.1.2.3/foo.zip
     '''
-    cached = __salt__['cp.cache_file'](name, saltenv)
+    cached = __salt__['cp.cache_file'](name, saltenv, source_hash=source_hash)
     if not cached:
         raise CommandExecutionError('Failed to cache {0}'.format(name))
 
@@ -1103,7 +1181,7 @@ def is_encrypted(name, clean=False, saltenv='base'):
     return ret
 
 
-@salt.utils.decorators.which('rar')
+@salt.utils.decorators.path.which('rar')
 def rar(rarfile, sources, template=None, cwd=None, runas=None):
     '''
     Uses `rar for Linux`_ to create rar files
@@ -1117,6 +1195,9 @@ def rar(rarfile, sources, template=None, cwd=None, runas=None):
         Comma-separated list of sources to include in the rar file. Sources can
         also be passed in a Python list.
 
+        .. versionchanged:: 2017.7.0
+            Globbing is now supported for this argument
+
     cwd : None
         Run the rar command from the specified directory. Use this argument
         along with relative file paths to create rar files which do not
@@ -1139,11 +1220,11 @@ def rar(rarfile, sources, template=None, cwd=None, runas=None):
     .. code-block:: bash
 
         salt '*' archive.rar /tmp/rarfile.rar /tmp/sourcefile1,/tmp/sourcefile2
+        # Globbing for sources (2017.7.0 and later)
+        salt '*' archive.rar /tmp/rarfile.rar '/tmp/sourcefile*'
     '''
-    if isinstance(sources, string_types):
-        sources = [s.strip() for s in sources.split(',')]
     cmd = ['rar', 'a', '-idp', '{0}'.format(rarfile)]
-    cmd.extend(sources)
+    cmd.extend(_expand_sources(sources))
     return __salt__['cmd.run'](cmd,
                                cwd=cwd,
                                template=template,
@@ -1151,7 +1232,7 @@ def rar(rarfile, sources, template=None, cwd=None, runas=None):
                                python_shell=False).splitlines()
 
 
-@salt.utils.decorators.which_bin(('unrar', 'rar'))
+@salt.utils.decorators.path.which_bin(('unrar', 'rar'))
 def unrar(rarfile, dest, excludes=None, template=None, runas=None, trim_output=False):
     '''
     Uses `rar for Linux`_ to unpack rar files
@@ -1183,19 +1264,19 @@ def unrar(rarfile, dest, excludes=None, template=None, runas=None, trim_output=F
         salt '*' archive.unrar /tmp/rarfile.rar /home/strongbad/ excludes=file_1,file_2
 
     '''
-    if isinstance(excludes, string_types):
+    if isinstance(excludes, six.string_types):
         excludes = [entry.strip() for entry in excludes.split(',')]
 
-    cmd = [salt.utils.which_bin(('unrar', 'rar')),
+    cmd = [salt.utils.path.which_bin(('unrar', 'rar')),
            'x', '-idp', '{0}'.format(rarfile)]
     if excludes is not None:
         for exclude in excludes:
             cmd.extend(['-x', '{0}'.format(exclude)])
     cmd.append('{0}'.format(dest))
     files = __salt__['cmd.run'](cmd,
-                               template=template,
-                               runas=runas,
-                               python_shell=False).splitlines()
+                                template=template,
+                                runas=runas,
+                                python_shell=False).splitlines()
 
     return _trim_files(files, trim_output)
 
@@ -1229,15 +1310,15 @@ def _render_filenames(filenames, zip_file, saltenv, template):
         temp file, rendering that file, and returning the result.
         '''
         # write out path to temp file
-        tmp_path_fn = salt.utils.mkstemp()
-        with salt.utils.fopen(tmp_path_fn, 'w+') as fp_:
-            fp_.write(contents)
+        tmp_path_fn = salt.utils.files.mkstemp()
+        with salt.utils.files.fopen(tmp_path_fn, 'w+') as fp_:
+            fp_.write(salt.utils.stringutils.to_str(contents))
         data = salt.utils.templates.TEMPLATE_REGISTRY[template](
             tmp_path_fn,
             to_str=True,
             **kwargs
         )
-        salt.utils.safe_rm(tmp_path_fn)
+        salt.utils.files.safe_rm(tmp_path_fn)
         if not data['result']:
             # Failed to render the template
             raise CommandExecutionError(
@@ -1254,7 +1335,9 @@ def _render_filenames(filenames, zip_file, saltenv, template):
 
 
 def _trim_files(files, trim_output):
-    # Trim the file list for output
+    '''
+    Trim the file list for output.
+    '''
     count = 100
     if not isinstance(trim_output, bool):
         count = trim_output
