@@ -28,23 +28,31 @@ For an item only one field should be provided. Either a `data` or a `file` entry
 In case both are provided the `file` entry is prefered.
 
 .. code-block:: bash
+
     salt '*' kubernetes.nodes api_url=http://k8s-api-server:port api_user=myuser api_password=pass
 
 .. versionadded: 2017.7.0
 '''
 
 # Import Python Futures
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
+import sys
 import os.path
 import base64
 import logging
-import yaml
 import tempfile
+import signal
+from time import sleep
+from contextlib import contextmanager
 
 from salt.exceptions import CommandExecutionError
 from salt.ext.six import iteritems
-import salt.utils
+from salt.ext import six
+import salt.utils.files
 import salt.utils.templates
+import salt.utils.yaml
+from salt.exceptions import TimeoutError
+from salt.ext.six.moves import range  # pylint: disable=import-error
 
 try:
     import kubernetes  # pylint: disable=import-self
@@ -78,6 +86,21 @@ def __virtual__():
     return False, 'python kubernetes library not found'
 
 
+if not salt.utils.platform.is_windows():
+    @contextmanager
+    def _time_limit(seconds):
+        def signal_handler(signum, frame):
+            raise TimeoutError
+        signal.signal(signal.SIGALRM, signal_handler)
+        signal.alarm(seconds)
+        try:
+            yield
+        finally:
+            signal.alarm(0)
+
+    POLLING_TIME_LIMIT = 30
+
+
 # pylint: disable=no-member
 def _setup_conn(**kwargs):
     '''
@@ -123,6 +146,9 @@ def _setup_conn(**kwargs):
     kubernetes.client.configuration.host = host
     kubernetes.client.configuration.user = username
     kubernetes.client.configuration.passwd = password
+    if __salt__['config.option']('kubernetes.api_key'):
+        kubernetes.client.configuration.api_key = {'authorization': __salt__['config.option']('kubernetes.api_key')}
+        kubernetes.client.configuration.api_key_prefix = {'authorization': __salt__['config.option']('kubernetes.api_key_prefix')}
 
     if ca_cert_file:
         kubernetes.client.configuration.ssl_ca_cert = ca_cert_file
@@ -144,24 +170,27 @@ def _setup_conn(**kwargs):
 
     if client_key_file:
         kubernetes.client.configuration.key_file = client_key_file
-    if client_key:
+    elif client_key:
         with tempfile.NamedTemporaryFile(prefix='salt-kube-', delete=False) as k:
             k.write(base64.b64decode(client_key))
             kubernetes.client.configuration.key_file = k.name
     else:
         kubernetes.client.configuration.key_file = None
 
+    # The return makes unit testing easier
+    return vars(kubernetes.client.configuration)
+
 
 def _cleanup(**kwargs):
     ca = kubernetes.client.configuration.ssl_ca_cert
     cert = kubernetes.client.configuration.cert_file
     key = kubernetes.client.configuration.key_file
     if cert and os.path.exists(cert) and os.path.basename(cert).startswith('salt-kube-'):
-        salt.utils.safe_rm(cert)
+        salt.utils.files.safe_rm(cert)
     if key and os.path.exists(key) and os.path.basename(key).startswith('salt-kube-'):
-        salt.utils.safe_rm(key)
+        salt.utils.files.safe_rm(key)
     if ca and os.path.exists(ca) and os.path.basename(ca).startswith('salt-kube-'):
-        salt.utils.safe_rm(ca)
+        salt.utils.files.safe_rm(ca)
 
 
 def ping(**kwargs):
@@ -200,9 +229,7 @@ def nodes(**kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->list_node: {0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->list_node')
             raise CommandExecutionError(exc)
     finally:
         _cleanup()
@@ -224,9 +251,7 @@ def node(name, **kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->list_node: {0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->list_node')
             raise CommandExecutionError(exc)
     finally:
         _cleanup()
@@ -249,7 +274,7 @@ def node_labels(name, **kwargs):
     match = node(name, **kwargs)
 
     if match is not None:
-        return match.metadata.labels
+        return match['metadata']['labels']
 
     return {}
 
@@ -280,9 +305,7 @@ def node_add_label(node_name, label_name, label_value, **kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->patch_node: {0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->patch_node')
             raise CommandExecutionError(exc)
     finally:
         _cleanup()
@@ -315,9 +338,7 @@ def node_remove_label(node_name, label_name, **kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->patch_node: {0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->patch_node')
             raise CommandExecutionError(exc)
     finally:
         _cleanup()
@@ -344,10 +365,7 @@ def namespaces(**kwargs):
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
-            log.exception(
-                'Exception when calling CoreV1Api->list_namespace: '
-                '{0}'.format(exc)
-            )
+            log.exception('Exception when calling CoreV1Api->list_namespace')
             raise CommandExecutionError(exc)
     finally:
         _cleanup()
@@ -374,8 +392,7 @@ def deployments(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->list_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->list_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -403,7 +420,7 @@ def services(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->list_namespaced_service: {0}'.format(exc)
+                'CoreV1Api->list_namespaced_service'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -431,7 +448,7 @@ def pods(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->list_namespaced_pod: {0}'.format(exc)
+                'CoreV1Api->list_namespaced_pod'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -459,7 +476,7 @@ def secrets(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->list_namespaced_secret: {0}'.format(exc)
+                'CoreV1Api->list_namespaced_secret'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -487,7 +504,7 @@ def configmaps(namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->list_namespaced_config_map: {0}'.format(exc)
+                'CoreV1Api->list_namespaced_config_map'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -515,8 +532,7 @@ def show_deployment(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->read_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->read_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -544,7 +560,7 @@ def show_service(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespaced_service: {0}'.format(exc)
+                'CoreV1Api->read_namespaced_service'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -572,7 +588,7 @@ def show_pod(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespaced_pod: {0}'.format(exc)
+                'CoreV1Api->read_namespaced_pod'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -599,7 +615,7 @@ def show_namespace(name, **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespace: {0}'.format(exc)
+                'CoreV1Api->read_namespace'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -635,8 +651,7 @@ def show_secret(name, namespace='default', decode=False, **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespaced_secret: '
-                '{0}'.format(exc)
+                'CoreV1Api->read_namespaced_secret'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -666,8 +681,7 @@ def show_configmap(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->read_namespaced_config_map: '
-                '{0}'.format(exc)
+                'CoreV1Api->read_namespaced_config_map'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -692,15 +706,37 @@ def delete_deployment(name, namespace='default', **kwargs):
             name=name,
             namespace=namespace,
             body=body)
-        return api_response.to_dict()
+        mutable_api_response = api_response.to_dict()
+        if not salt.utils.platform.is_windows():
+            try:
+                with _time_limit(POLLING_TIME_LIMIT):
+                    while show_deployment(name, namespace) is not None:
+                        sleep(1)
+                    else:  # pylint: disable=useless-else-on-loop
+                        mutable_api_response['code'] = 200
+            except TimeoutError:
+                pass
+        else:
+            # Windows has not signal.alarm implementation, so we are just falling
+            # back to loop-counting.
+            for i in range(60):
+                if show_deployment(name, namespace) is None:
+                    mutable_api_response['code'] = 200
+                    break
+                else:
+                    sleep(1)
+        if mutable_api_response['code'] != 200:
+            log.warning('Reached polling time limit. Deployment is not yet '
+                        'deleted, but we are backing off. Sorry, but you\'ll '
+                        'have to check manually.')
+        return mutable_api_response
     except (ApiException, HTTPError) as exc:
         if isinstance(exc, ApiException) and exc.status == 404:
             return None
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->delete_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->delete_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -730,8 +766,7 @@ def delete_service(name, namespace='default', **kwargs):
             return None
         else:
             log.exception(
-                'Exception when calling CoreV1Api->delete_namespaced_service: '
-                '{0}'.format(exc)
+                'Exception when calling CoreV1Api->delete_namespaced_service'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -764,7 +799,7 @@ def delete_pod(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->delete_namespaced_pod: {0}'.format(exc)
+                'CoreV1Api->delete_namespaced_pod'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -793,8 +828,7 @@ def delete_namespace(name, **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->delete_namespace: '
-                '{0}'.format(exc)
+                'CoreV1Api->delete_namespace'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -826,8 +860,7 @@ def delete_secret(name, namespace='default', **kwargs):
             return None
         else:
             log.exception(
-                'Exception when calling CoreV1Api->delete_namespaced_secret: '
-                '{0}'.format(exc)
+                'Exception when calling CoreV1Api->delete_namespaced_secret'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -860,8 +893,7 @@ def delete_configmap(name, namespace='default', **kwargs):
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->delete_namespaced_config_map: '
-                '{0}'.format(exc)
+                'CoreV1Api->delete_namespaced_config_map'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -906,8 +938,7 @@ def create_deployment(
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->create_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->create_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -952,8 +983,7 @@ def create_pod(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespaced_pod: '
-                '{0}'.format(exc)
+                'CoreV1Api->create_namespaced_pod'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -998,7 +1028,7 @@ def create_service(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespaced_service: {0}'.format(exc)
+                'CoreV1Api->create_namespaced_service'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -1007,14 +1037,22 @@ def create_service(
 
 def create_secret(
         name,
-        namespace,
-        data,
-        source,
-        template,
-        saltenv,
+        namespace='default',
+        data=None,
+        source=None,
+        template=None,
+        saltenv='base',
         **kwargs):
     '''
     Creates the kubernetes secret as defined by the user.
+
+    CLI Examples::
+
+        salt 'minion1' kubernetes.create_secret \
+            passwords default '{"db": "letmein"}'
+
+        salt 'minion2' kubernetes.create_secret \
+            name=passwords namespace=default data='{"db": "letmein"}'
     '''
     if source:
         data = __read_and_render_yaml_file(source, template, saltenv)
@@ -1045,7 +1083,7 @@ def create_secret(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespaced_secret: {0}'.format(exc)
+                'CoreV1Api->create_namespaced_secret'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -1056,12 +1094,20 @@ def create_configmap(
         name,
         namespace,
         data,
-        source,
-        template,
-        saltenv,
+        source=None,
+        template=None,
+        saltenv='base',
         **kwargs):
     '''
     Creates the kubernetes configmap as defined by the user.
+
+    CLI Examples::
+
+        salt 'minion1' kubernetes.create_configmap \
+            settings default '{"example.conf": "# example file"}'
+
+        salt 'minion2' kubernetes.create_configmap \
+            name=settings namespace=default data='{"example.conf": "# example file"}'
     '''
     if source:
         data = __read_and_render_yaml_file(source, template, saltenv)
@@ -1088,7 +1134,7 @@ def create_configmap(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespaced_config_map: {0}'.format(exc)
+                'CoreV1Api->create_namespaced_config_map'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -1123,8 +1169,7 @@ def create_namespace(
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->create_namespace: '
-                '{0}'.format(exc)
+                'CoreV1Api->create_namespace'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -1169,8 +1214,7 @@ def replace_deployment(name,
         else:
             log.exception(
                 'Exception when calling '
-                'ExtensionsV1beta1Api->replace_namespaced_deployment: '
-                '{0}'.format(exc)
+                'ExtensionsV1beta1Api->replace_namespaced_deployment'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -1221,7 +1265,7 @@ def replace_service(name,
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->replace_namespaced_service: {0}'.format(exc)
+                'CoreV1Api->replace_namespaced_service'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -1230,14 +1274,22 @@ def replace_service(name,
 
 def replace_secret(name,
                    data,
-                   source,
-                   template,
-                   saltenv,
+                   source=None,
+                   template=None,
+                   saltenv='base',
                    namespace='default',
                    **kwargs):
     '''
     Replaces an existing secret with a new one defined by name and namespace,
     having the specificed data.
+
+    CLI Examples::
+
+        salt 'minion1' kubernetes.replace_secret \
+            name=passwords data='{"db": "letmein"}'
+
+        salt 'minion2' kubernetes.replace_secret \
+            name=passwords namespace=saltstack data='{"db": "passw0rd"}'
     '''
     if source:
         data = __read_and_render_yaml_file(source, template, saltenv)
@@ -1268,7 +1320,7 @@ def replace_secret(name,
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->replace_namespaced_secret: {0}'.format(exc)
+                'CoreV1Api->replace_namespaced_secret'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -1277,14 +1329,22 @@ def replace_secret(name,
 
 def replace_configmap(name,
                       data,
-                      source,
-                      template,
-                      saltenv,
+                      source=None,
+                      template=None,
+                      saltenv='base',
                       namespace='default',
                       **kwargs):
     '''
     Replaces an existing configmap with a new one defined by name and
-    namespace, having the specificed data.
+    namespace with the specified data.
+
+    CLI Examples::
+
+        salt 'minion1' kubernetes.replace_configmap \
+            settings default '{"example.conf": "# example file"}'
+
+        salt 'minion2' kubernetes.replace_configmap \
+            name=settings namespace=default data='{"example.conf": "# example file"}'
     '''
     if source:
         data = __read_and_render_yaml_file(source, template, saltenv)
@@ -1309,7 +1369,7 @@ def replace_configmap(name,
         else:
             log.exception(
                 'Exception when calling '
-                'CoreV1Api->replace_namespaced_configmap: {0}'.format(exc)
+                'CoreV1Api->replace_namespaced_configmap'
             )
             raise CommandExecutionError(exc)
     finally:
@@ -1361,7 +1421,7 @@ def __read_and_render_yaml_file(source,
         raise CommandExecutionError(
             'Source file \'{0}\' not found'.format(source))
 
-    with salt.utils.fopen(sfn, 'r') as src:
+    with salt.utils.files.fopen(sfn, 'r') as src:
         contents = src.read()
 
         if template:
@@ -1392,7 +1452,7 @@ def __read_and_render_yaml_file(source,
                     'Unknown template specified: {0}'.format(
                         template))
 
-        return yaml.load(contents)
+        return salt.utils.yaml.safe_load(contents)
 
 
 def __dict_to_object_meta(name, namespace, metadata):
@@ -1401,6 +1461,13 @@ def __dict_to_object_meta(name, namespace, metadata):
     '''
     meta_obj = kubernetes.client.V1ObjectMeta()
     meta_obj.namespace = namespace
+
+    # Replicate `kubectl [create|replace|apply] --record`
+    if 'annotations' not in metadata:
+        metadata['annotations'] = {}
+    if 'kubernetes.io/change-cause' not in metadata['annotations']:
+        metadata['annotations']['kubernetes.io/change-cause'] = ' '.join(sys.argv)
+
     for key, value in iteritems(metadata):
         if hasattr(meta_obj, key):
             setattr(meta_obj, key, value)
@@ -1468,6 +1535,6 @@ def __enforce_only_strings_dict(dictionary):
     ret = {}
 
     for key, value in iteritems(dictionary):
-        ret[str(key)] = str(value)
+        ret[six.text_type(key)] = six.text_type(value)
 
     return ret
