@@ -11,7 +11,7 @@ This is an alternative to the ``ldap`` interface provided by the
 :depends: - ``ldap`` Python module
 '''
 
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 available_backends = set()
 try:
@@ -23,7 +23,7 @@ try:
 except ImportError:
     pass
 import logging
-import salt.ext.six as six
+from salt.ext import six
 import sys
 
 log = logging.getLogger(__name__)
@@ -79,6 +79,20 @@ def _bind(l, bind=None):
                          + '"; supported bind methods: simple sasl')
 
 
+def _format_unicode_password(pwd):
+    '''Formats a string per Microsoft AD password specifications.
+    The string must be enclosed in double quotes and UTF-16 encoded.
+    See: https://msdn.microsoft.com/en-us/library/cc223248.aspx
+
+    :param pwd:
+       The desired password as a string
+
+    :returns:
+        A unicode string
+    '''
+    return '"{0}"'.format(pwd).encode('utf-16-le')
+
+
 class _connect_ctx(object):
     def __init__(self, c):
         self.c = c
@@ -388,8 +402,11 @@ def add(connect_spec, dn, attributes):
     # are not modified)
     attributes = dict(((attr, list(vals))
                        for attr, vals in six.iteritems(attributes)))
-    log.info('adding entry: dn: {0} attributes: {1}'.format(
-        repr(dn), repr(attributes)))
+    log.info('adding entry: dn: %s attributes: %s', repr(dn), repr(attributes))
+
+    if 'unicodePwd' in attributes:
+        attributes['unicodePwd'] = [_format_unicode_password(x) for x in attributes['unicodePwd']]
+
     modlist = ldap.modlist.addModlist(attributes)
     try:
         l.c.add_s(dn, modlist)
@@ -423,7 +440,7 @@ def delete(connect_spec, dn):
         }" dn='cn=admin,dc=example,dc=com'
     '''
     l = connect(connect_spec)
-    log.info('deleting entry: dn: {0}'.format(repr(dn)))
+    log.info('deleting entry: dn: %s', repr(dn))
     try:
         l.c.delete_s(dn)
     except ldap.LDAPError as e:
@@ -484,6 +501,12 @@ def modify(connect_spec, dn, directives):
     # not modified)
     modlist = [(getattr(ldap, 'MOD_' + op.upper()), attr, list(vals))
                for op, attr, vals in directives]
+
+    for idx, mod in enumerate(modlist):
+        if mod[1] == 'unicodePwd':
+            modlist[idx] = (mod[0], mod[1],
+                [_format_unicode_password(x) for x in mod[2]])
+
     try:
         l.c.modify_s(dn, modlist)
     except ldap.LDAPError as e:
@@ -546,6 +569,10 @@ def change(connect_spec, dn, before, after):
                    for attr, vals in six.iteritems(before)))
     after = dict(((attr, list(vals))
                   for attr, vals in six.iteritems(after)))
+
+    if 'unicodePwd' in after:
+        after['unicodePwd'] = [_format_unicode_password(x) for x in after['unicodePwd']]
+
     modlist = ldap.modlist.modifyModlist(before, after)
     try:
         l.c.modify_s(dn, modlist)
