@@ -20,7 +20,7 @@ CLI Example:
 
     salt '*' log.error 'Please dont do that, this module is not for CLI use!'
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import logging
@@ -35,36 +35,48 @@ def __virtual__():
 
 
 def debug(message):
-    '''Log message at level DEBUG.'''
+    '''
+    Log message at level DEBUG.
+    '''
     log.debug(message)
     return True
 
 
 def info(message):
-    '''Log message at level INFO.'''
+    '''
+    Log message at level INFO.
+    '''
     log.info(message)
     return True
 
 
 def warning(message):
-    '''Log message at level WARNING.'''
+    '''
+    Log message at level WARNING.
+    '''
     log.warning(message)
     return True
 
 
 def error(message):
-    '''Log message at level ERROR.'''
+    '''
+    Log message at level ERROR.
+    '''
     log.error(message)
     return True
 
 
 def critical(message):
-    '''Log message at level CRITICAL.'''
+    '''
+    Log message at level CRITICAL.
+    '''
     log.critical(message)
     return True
 
 
 def exception(message):
-    '''Log message at level EXCEPTION.'''
+    '''
+    Log message at level EXCEPTION.
+    '''
     log.exception(message)
     return True
