@@ -2,21 +2,26 @@
 '''
 Extract the pillar data for this minion
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import collections
 
 # Import third party libs
+import copy
 import os
 import copy
 import logging
-import yaml
-import salt.ext.six as six
+from salt.ext import six
 
 # Import salt libs
 import salt.pillar
-import salt.utils
+import salt.utils.crypt
+import salt.utils.data
+import salt.utils.dictupdate
+import salt.utils.functools
+import salt.utils.odict
+import salt.utils.yaml
 from salt.defaults import DEFAULT_TARGET_DELIM
 from salt.exceptions import CommandExecutionError
 
@@ -30,6 +35,7 @@ def get(key,
         merge=False,
         merge_nested_lists=None,
         delimiter=DEFAULT_TARGET_DELIM,
+        pillarenv=None,
         saltenv=None):
     '''
     .. versionadded:: 0.14
@@ -58,7 +64,7 @@ def get(key,
         dictionaries, the dictionaries will be recursively merged.
 
         .. versionadded:: 2014.7.0
-        .. versionchanged:: 2016.3.7,2016.11.4,Nitrogen
+        .. versionchanged:: 2016.3.7,2016.11.4,2017.7.0
             If the default and the retrieved value are not of the same type,
             then merging will be skipped and the retrieved value will be
             returned. Earlier releases raised an error in these cases.
@@ -82,7 +88,7 @@ def get(key,
 
         .. versionadded:: 2014.7.0
 
-    saltenv
+    pillarenv
         If specified, this function will query the master to generate fresh
         pillar data on the fly, specifically from the requested pillar
         environment. Note that this can produce different pillar data than
@@ -96,7 +102,13 @@ def get(key,
         data. This tradeoff in performance however allows for the use case
         where pillar data is desired only from a single environment.
 
-        .. versionadded:: Nitrogen
+        .. versionadded:: 2017.7.0
+
+    saltenv
+        Included only for compatibility with
+        :conf_minion:`pillarenv_from_saltenv`, and is otherwise ignored.
+
+        .. versionadded:: 2017.7.0
 
     CLI Example:
 
@@ -110,11 +122,13 @@ def get(key,
             default = ''
     opt_merge_lists = __opts__.get('pillar_merge_lists', False) if \
         merge_nested_lists is None else merge_nested_lists
-    pillar_dict = __pillar__ if saltenv is None else items(saltenv=saltenv)
+    pillar_dict = __pillar__ \
+        if all(x is None for x in (saltenv, pillarenv)) \
+        else items(saltenv=saltenv, pillarenv=pillarenv)
 
     if merge:
         if isinstance(default, dict):
-            ret = salt.utils.traverse_dict_and_list(
+            ret = salt.utils.data.traverse_dict_and_list(
                 pillar_dict,
                 key,
                 {},
@@ -132,7 +146,7 @@ def get(key,
                     'skipped.', default, ret, type(ret).__name__
                 )
         elif isinstance(default, list):
-            ret = salt.utils.traverse_dict_and_list(
+            ret = salt.utils.data.traverse_dict_and_list(
                 pillar_dict,
                 key,
                 [],
@@ -154,10 +168,11 @@ def get(key,
                 default, type(default).__name__
             )
 
-    ret = salt.utils.traverse_dict_and_list(pillar_dict,
-                                            key,
-                                            default,
-                                            delimiter)
+    ret = salt.utils.data.traverse_dict_and_list(
+        pillar_dict,
+        key,
+        default,
+        delimiter)
     if ret is KeyError:
         raise KeyError('Pillar key not found: {0}'.format(key))
 
@@ -173,13 +188,29 @@ def items(*args, **kwargs):
     currently loaded into the minion.
 
     pillar
-        if specified, allows for a dictionary of pillar data to be made
+        If specified, allows for a dictionary of pillar data to be made
         available to pillar and ext_pillar rendering. these pillar variables
         will also override any variables of the same name in pillar or
         ext_pillar.
 
         .. versionadded:: 2015.5.0
 
+    pillar_enc
+        If specified, the data passed in the ``pillar`` argument will be passed
+        through this renderer to decrypt it.
+
+        .. note::
+            This will decrypt on the minion side, so the specified renderer
+            must be set up on the minion for this to work. Alternatively,
+            pillar data can be decrypted master-side. For more information, see
+            the :ref:`Pillar Encryption <pillar-encryption>` documentation.
+            Pillar data that is decrypted master-side, is not decrypted until
+            the end of pillar compilation though, so minion-side decryption
+            will be necessary if the encrypted pillar data must be made
+            available in an decrypted state pillar/ext_pillar rendering.
+
+        .. versionadded:: 2017.7.0
+
     pillarenv
         Pass a specific pillar environment from which to compile pillar data.
         If not specified, then the minion's :conf_minion:`pillarenv` option is
@@ -189,6 +220,10 @@ def items(*args, **kwargs):
 
         .. versionadded:: 2016.11.2
 
+    saltenv
+        Included only for compatibility with
+        :conf_minion:`pillarenv_from_saltenv`, and is otherwise ignored.
+
     CLI Example:
 
     .. code-block:: bash
@@ -199,17 +234,40 @@ def items(*args, **kwargs):
     if args:
         return item(*args)
 
+    pillarenv = kwargs.get('pillarenv')
+    if pillarenv is None:
+        if __opts__.get('pillarenv_from_saltenv', False):
+            pillarenv = kwargs.get('saltenv') or __opts__['saltenv']
+        else:
+            pillarenv = __opts__['pillarenv']
+
+    pillar_override = kwargs.get('pillar')
+    pillar_enc = kwargs.get('pillar_enc')
+
+    if pillar_override and pillar_enc:
+        try:
+            pillar_override = salt.utils.crypt.decrypt(
+                pillar_override,
+                pillar_enc,
+                translate_newlines=True,
+                opts=__opts__,
+                valid_rend=__opts__['decrypt_pillar_renderers'])
+        except Exception as exc:
+            raise CommandExecutionError(
+                'Failed to decrypt pillar override: {0}'.format(exc)
+            )
+
     pillar = salt.pillar.get_pillar(
         __opts__,
         __grains__,
         __opts__['id'],
-        pillar=kwargs.get('pillar'),
-        pillarenv=kwargs.get('pillarenv') or __opts__['pillarenv'])
+        pillar_override=pillar_override,
+        pillarenv=pillarenv)
 
     return pillar.compile_pillar()
 
 # Allow pillar.data to also be used to return pillar data
-data = salt.utils.alias_function(items, 'data')
+data = salt.utils.functools.alias_function(items, 'data')
 
 
 def _obfuscate_inner(var):
@@ -315,10 +373,11 @@ def item(*args, **kwargs):
 
     try:
         for arg in args:
-            ret[arg] = salt.utils.traverse_dict_and_list(__pillar__,
-                                                         arg,
-                                                         default,
-                                                         delimiter)
+            ret[arg] = salt.utils.data.traverse_dict_and_list(
+                __pillar__,
+                arg,
+                default,
+                delimiter)
     except KeyError:
         pass
 
@@ -353,7 +412,7 @@ def raw(key=None):
 
 def ext(external, pillar=None):
     '''
-    .. versionchanged:: 2016.3.6,2016.11.3,Nitrogen
+    .. versionchanged:: 2016.3.6,2016.11.3,2017.7.0
         The supported ext_pillar types are now tunable using the
         :conf_master:`on_demand_ext_pillar` config option. Earlier releases
         used a hard-coded default.
@@ -370,8 +429,8 @@ def ext(external, pillar=None):
 
         .. code-block:: python
 
-            >>> import yaml
-            >>> ext_pillar = yaml.safe_load("""
+            >>> import salt.utils.yaml
+            >>> ext_pillar = salt.utils.yaml.safe_load("""
             ... ext_pillar:
             ...   - git:
             ...     - issue38440 https://github.com/terminalmage/git_pillar:
@@ -404,14 +463,14 @@ def ext(external, pillar=None):
         salt '*' pillar.ext "{'git': [{'mybranch https://github.com/myuser/myrepo': [{'env': 'base'}]}]}"
     '''
     if isinstance(external, six.string_types):
-        external = yaml.safe_load(external)
+        external = salt.utils.yaml.safe_load(external)
     pillar_obj = salt.pillar.get_pillar(
         __opts__,
         __grains__,
         __opts__['id'],
-        __opts__['environment'],
+        __opts__['saltenv'],
         ext=external,
-        pillar=pillar)
+        pillar_override=pillar)
 
     ret = pillar_obj.compile_pillar()
 
@@ -436,7 +495,7 @@ def keys(key, delimiter=DEFAULT_TARGET_DELIM):
 
         salt '*' pillar.keys web:sites
     '''
-    ret = salt.utils.traverse_dict_and_list(
+    ret = salt.utils.data.traverse_dict_and_list(
         __pillar__, key, KeyError, delimiter)
 
     if ret is KeyError:
@@ -498,3 +557,63 @@ def file_exists(path, saltenv=None):
 
 # Provide a jinja function call compatible get aliased as fetch
 fetch = get
+
+
+def filter_by(lookup_dict,
+              pillar,
+              merge=None,
+              default='default',
+              base=None):
+    '''
+    .. versionadded:: 2017.7.0
+
+    Look up the given pillar in a given dictionary and return the result
+
+    :param lookup_dict: A dictionary, keyed by a pillar, containing a value or
+        values relevant to systems matching that pillar. For example, a key
+        could be a pillar for a role and the value could the name of a package
+        on that particular OS.
+
+        The dictionary key can be a globbing pattern. The function will return
+        the corresponding ``lookup_dict`` value where the pilalr value matches
+        the  pattern. For example:
+
+        .. code-block:: bash
+
+            # this will render 'got some salt' if ``role`` begins with 'salt'
+            salt '*' pillar.filter_by '{salt*: got some salt, default: salt is not here}' role
+
+    :param pillar: The name of a pillar to match with the system's pillar. For
+        example, the value of the "role" pillar could be used to pull values
+        from the ``lookup_dict`` dictionary.
+
+        The pillar value can be a list. The function will return the
+        ``lookup_dict`` value for a first found item in the list matching
+        one of the ``lookup_dict`` keys.
+
+    :param merge: A dictionary to merge with the results of the pillar
+        selection from ``lookup_dict``. This allows another dictionary to
+        override the values in the ``lookup_dict``.
+
+    :param default: default lookup_dict's key used if the pillar does not exist
+        or if the pillar value has no match on lookup_dict.  If unspecified
+        the value is "default".
+
+    :param base: A lookup_dict key to use for a base dictionary.  The
+        pillar-selected ``lookup_dict`` is merged over this and then finally
+        the ``merge`` dictionary is merged.  This allows common values for
+        each case to be collected in the base and overridden by the pillar
+        selection dictionary and the merge dictionary.  Default is unset.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' pillar.filter_by '{web: Serve it up, db: I query, default: x_x}' role
+    '''
+    return salt.utils.data.filter_by(lookup_dict=lookup_dict,
+                                     lookup=pillar,
+                                     traverse=__pillar__,
+                                     merge=merge,
+                                     default=default,
+                                     base=base)
