@@ -36,7 +36,7 @@ file, in order to use this module to manage packages, like so:
       pkg: pkgng
 
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import copy
@@ -44,9 +44,15 @@ import logging
 import os
 
 # Import salt libs
-import salt.utils
+import salt.utils.data
+import salt.utils.files
+import salt.utils.functools
+import salt.utils.itertools
+import salt.utils.pkg
+import salt.utils.stringutils
+import salt.utils.versions
 from salt.exceptions import CommandExecutionError, MinionError
-import salt.ext.six as six
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -65,12 +71,11 @@ def __virtual__():
         return __virtualname__
     if __grains__['os'] == 'FreeBSD' and float(__grains__['osrelease']) >= 10:
         return __virtualname__
-    if __grains__['os'] == 'FreeBSD' and \
-            float(__grains__['osmajorrelease']) == 9:
+    if __grains__['os'] == 'FreeBSD' and int(__grains__['osmajorrelease']) == 9:
         providers = {}
         if 'providers' in __opts__:
             providers = __opts__['providers']
-        log.debug('__opts__.providers: {0}'.format(providers))
+        log.debug('__opts__.providers: %s', providers)
         if providers and 'pkg' in providers and providers['pkg'] == 'pkgng':
             log.debug('Configuration option \'providers:pkg\' is set to '
                 '\'pkgng\', using \'pkgng\' in favor of \'freebsdpkg\'.')
@@ -80,7 +85,7 @@ def __virtual__():
             'on FreeBSD 10 or FreeBSD 9 with providers.pkg set to pkgng.')
 
 
-def _pkg(jail=None, chroot=None):
+def _pkg(jail=None, chroot=None, root=None):
     '''
     Returns the prefix for a pkg command, using -j if a jail is specified, or
     -c if chroot is specified.
@@ -90,14 +95,16 @@ def _pkg(jail=None, chroot=None):
         ret.extend(['-j', jail])
     elif chroot:
         ret.extend(['-c', chroot])
+    elif root:
+        ret.extend(['-r', root])
     return ret
 
 
-def _get_pkgng_version(jail=None, chroot=None):
+def _get_pkgng_version(jail=None, chroot=None, root=None):
     '''
     return the version of 'pkg'
     '''
-    cmd = _pkg(jail, chroot) + ['--version']
+    cmd = _pkg(jail, chroot, root) + ['--version']
     return __salt__['cmd.run'](cmd).strip()
 
 
@@ -118,16 +125,18 @@ def _get_version(name, results):
     return None
 
 
-def _contextkey(jail=None, chroot=None, prefix='pkg.list_pkgs'):
+def _contextkey(jail=None, chroot=None, root=None, prefix='pkg.list_pkgs'):
     '''
     As this module is designed to manipulate packages in jails and chroots, use
     the passed jail/chroot to ensure that a key in the __context__ dict that is
     unique to that jail/chroot is used.
     '''
     if jail:
-        return str(prefix) + '.jail_{0}'.format(jail)
+        return six.text_type(prefix) + '.jail_{0}'.format(jail)
     elif chroot:
-        return str(prefix) + '.chroot_{0}'.format(chroot)
+        return six.text_type(prefix) + '.chroot_{0}'.format(chroot)
+    elif root:
+        return six.text_type(prefix) + '.root_{0}'.format(root)
     return prefix
 
 
@@ -147,8 +156,9 @@ def parse_config(file_name='/usr/local/etc/pkg.conf'):
     if not os.path.isfile(file_name):
         return 'Unable to find {0} on file system'.format(file_name)
 
-    with salt.utils.fopen(file_name) as ifile:
+    with salt.utils.files.fopen(file_name) as ifile:
         for line in ifile:
+            line = salt.utils.stringutils.to_unicode(line)
             if line.startswith('#') or line.startswith('\n'):
                 pass
             else:
@@ -176,6 +186,10 @@ def version(*names, **kwargs):
         Get package version information for the specified chroot (ignored if
         ``jail`` is specified)
 
+    root
+        Get package version information for the specified root (ignored if
+        ``jail`` is specified)
+
     with_origin : False
         Return a nested dictionary containing both the origin name and version
         for each specified package.
@@ -193,7 +207,7 @@ def version(*names, **kwargs):
     '''
     with_origin = kwargs.pop('with_origin', False)
     ret = __salt__['pkg_resource.version'](*names, **kwargs)
-    if not salt.utils.is_true(with_origin):
+    if not salt.utils.data.is_true(with_origin):
         return ret
     # Put the return value back into a dict since we're adding a subdict
     if len(names) == 1:
@@ -205,10 +219,10 @@ def version(*names, **kwargs):
     ])
 
 # Support pkg.info get version info, since this is the CLI usage
-info = salt.utils.alias_function(version, 'info')
+info = salt.utils.functools.alias_function(version, 'info')
 
 
-def refresh_db(jail=None, chroot=None, force=False):
+def refresh_db(jail=None, chroot=None, root=None, force=False):
     '''
     Refresh PACKAGESITE contents
 
@@ -230,6 +244,10 @@ def refresh_db(jail=None, chroot=None, force=False):
         Refresh the pkg database within the specified chroot (ignored if
         ``jail`` is specified)
 
+    root
+        Refresh the pkg database within the specified root (ignored if
+        ``jail`` is specified)
+
     force
         Force a full download of the repository catalog without regard to the
         respective ages of the local and remote copies of the catalog.
@@ -240,7 +258,9 @@ def refresh_db(jail=None, chroot=None, force=False):
 
             salt '*' pkg.refresh_db force=True
     '''
-    cmd = _pkg(jail, chroot)
+    # Remove rtag file to keep multiple refreshes from happening in pkg states
+    salt.utils.pkg.clear_rtag(__opts__)
+    cmd = _pkg(jail, chroot, root)
     cmd.append('update')
     if force:
         cmd.append('-f')
@@ -248,7 +268,7 @@ def refresh_db(jail=None, chroot=None, force=False):
 
 
 # Support pkg.update to refresh the db, since this is the CLI usage
-update = salt.utils.alias_function(refresh_db, 'update')
+update = salt.utils.functools.alias_function(refresh_db, 'update')
 
 
 def latest_version(*names, **kwargs):
@@ -278,23 +298,24 @@ def latest_version(*names, **kwargs):
     jail = kwargs.get('jail')
     chroot = kwargs.get('chroot')
     refresh = kwargs.get('refresh')
-    pkgs = list_pkgs(versions_as_list=True, jail=jail, chroot=chroot)
+    root = kwargs.get('root')
+    pkgs = list_pkgs(versions_as_list=True, jail=jail, chroot=chroot, root=root)
 
-    if salt.utils.compare_versions(_get_pkgng_version(jail, chroot), '>=', '1.6.0'):
+    if salt.utils.versions.compare(_get_pkgng_version(jail, chroot, root), '>=', '1.6.0'):
         quiet = True
     else:
         quiet = False
 
-    cmd_prefix = _pkg(jail, chroot) + ['search']
+    cmd_prefix = _pkg(jail, chroot, root) + ['search']
     for name in names:
         # FreeBSD supports packages in format java/openjdk7
         if '/' in name:
-            cmd = _pkg(jail, chroot) + ['search']
+            cmd = _pkg(jail, chroot, root) + ['search']
         else:
-            cmd = _pkg(jail, chroot) + ['search', '-S', 'name', '-Q', 'version', '-e']
+            cmd = _pkg(jail, chroot, root) + ['search', '-S', 'name', '-Q', 'version', '-e']
         if quiet:
             cmd.append('-q')
-        if not salt.utils.is_true(refresh):
+        if not salt.utils.data.is_true(refresh):
             cmd.append('-U')
         cmd.append(name)
 
@@ -312,7 +333,7 @@ def latest_version(*names, **kwargs):
                 ret[name] = pkgver
             else:
                 if not any(
-                    (salt.utils.compare_versions(ver1=x,
+                    (salt.utils.versions.compare(ver1=x,
                                                  oper='>=',
                                                  ver2=pkgver)
                      for x in installed)
@@ -326,12 +347,13 @@ def latest_version(*names, **kwargs):
 
 
 # available_version is being deprecated
-available_version = salt.utils.alias_function(latest_version, 'available_version')
+available_version = salt.utils.functools.alias_function(latest_version, 'available_version')
 
 
 def list_pkgs(versions_as_list=False,
               jail=None,
               chroot=None,
+              root=None,
               with_origin=False,
               **kwargs):
     '''
@@ -346,6 +368,10 @@ def list_pkgs(versions_as_list=False,
         List the packages in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        List the packages in the specified root (ignored if ``jail`` is
+        specified)
+
     with_origin : False
         Return a nested dictionary containing both the origin name and version
         for each installed package.
@@ -361,19 +387,19 @@ def list_pkgs(versions_as_list=False,
         salt '*' pkg.list_pkgs chroot=/path/to/chroot
     '''
     # not yet implemented or not applicable
-    if any([salt.utils.is_true(kwargs.get(x))
+    if any([salt.utils.data.is_true(kwargs.get(x))
             for x in ('removed', 'purge_desired')]):
         return {}
 
-    versions_as_list = salt.utils.is_true(versions_as_list)
-    contextkey_pkg = _contextkey(jail, chroot)
-    contextkey_origins = _contextkey(jail, chroot, prefix='pkg.origin')
+    versions_as_list = salt.utils.data.is_true(versions_as_list)
+    contextkey_pkg = _contextkey(jail, chroot, root)
+    contextkey_origins = _contextkey(jail, chroot, root, prefix='pkg.origin')
 
     if contextkey_pkg in __context__:
         ret = copy.deepcopy(__context__[contextkey_pkg])
         if not versions_as_list:
             __salt__['pkg_resource.stringify'](ret)
-        if salt.utils.is_true(with_origin):
+        if salt.utils.data.is_true(with_origin):
             origins = __context__.get(contextkey_origins, {})
             return dict([
                 (x, {'origin': origins.get(x, ''), 'version': y})
@@ -384,7 +410,7 @@ def list_pkgs(versions_as_list=False,
     ret = {}
     origins = {}
     out = __salt__['cmd.run_stdout'](
-        _pkg(jail, chroot) + ['info', '-ao'],
+        _pkg(jail, chroot, root) + ['info', '-ao'],
         output_loglevel='trace',
         python_shell=False)
     for line in salt.utils.itertools.split(out, '\n'):
@@ -403,7 +429,7 @@ def list_pkgs(versions_as_list=False,
     __context__[contextkey_origins] = origins
     if not versions_as_list:
         __salt__['pkg_resource.stringify'](ret)
-    if salt.utils.is_true(with_origin):
+    if salt.utils.data.is_true(with_origin):
         return dict([
             (x, {'origin': origins.get(x, ''), 'version': y})
             for x, y in six.iteritems(ret)
@@ -432,7 +458,7 @@ def update_package_site(new_url):
     return True
 
 
-def stats(local=False, remote=False, jail=None, chroot=None):
+def stats(local=False, remote=False, jail=None, chroot=None, root=None):
     '''
     Return pkgng stats.
 
@@ -475,6 +501,10 @@ def stats(local=False, remote=False, jail=None, chroot=None):
         Retrieve stats from the specified chroot (ignored if ``jail`` is
         specified).
 
+    root
+        Retrieve stats from the specified root (ignored if ``jail`` is
+        specified).
+
         CLI Example:
 
         .. code-block:: bash
@@ -490,7 +520,7 @@ def stats(local=False, remote=False, jail=None, chroot=None):
     if remote:
         opts += 'r'
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.append('stats')
     if opts:
         cmd.append('-' + opts)
@@ -498,7 +528,7 @@ def stats(local=False, remote=False, jail=None, chroot=None):
     return [x.strip('\t') for x in salt.utils.itertools.split(out, '\n')]
 
 
-def backup(file_name, jail=None, chroot=None):
+def backup(file_name, jail=None, chroot=None, root=None):
     '''
     Export installed packages into yaml+mtree file
 
@@ -525,6 +555,12 @@ def backup(file_name, jail=None, chroot=None):
         so the path to the backup file will be relative to the root of the
         chroot.
 
+    root
+        Backup packages from the specified root (ignored if ``jail`` is
+        specified). Note that this will run the command within the root, and
+        so the path to the backup file will be relative to the root of the
+        root.
+
         CLI Example:
 
         .. code-block:: bash
@@ -532,14 +568,14 @@ def backup(file_name, jail=None, chroot=None):
             salt '*' pkg.backup /tmp/pkg chroot=/path/to/chroot
     '''
     ret = __salt__['cmd.run'](
-        _pkg(jail, chroot) + ['backup', '-d', file_name],
+        _pkg(jail, chroot, root) + ['backup', '-d', file_name],
         output_loglevel='trace',
         python_shell=False
     )
     return ret.split('...')[1]
 
 
-def restore(file_name, jail=None, chroot=None):
+def restore(file_name, jail=None, chroot=None, root=None):
     '''
     Reads archive created by pkg backup -d and recreates the database.
 
@@ -566,6 +602,12 @@ def restore(file_name, jail=None, chroot=None):
         so the path to the file from which the pkg database will be restored is
         relative to the root of the chroot.
 
+    root
+        Restore database to the specified root (ignored if ``jail`` is
+        specified). Note that this will run the command within the root, and
+        so the path to the file from which the pkg database will be restored is
+        relative to the root of the root.
+
         CLI Example:
 
         .. code-block:: bash
@@ -573,13 +615,13 @@ def restore(file_name, jail=None, chroot=None):
             salt '*' pkg.restore /tmp/pkg chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        _pkg(jail, chroot) + ['backup', '-r', file_name],
+        _pkg(jail, chroot, root) + ['backup', '-r', file_name],
         output_loglevel='trace',
         python_shell=False
     )
 
 
-def audit(jail=None, chroot=None):
+def audit(jail=None, chroot=None, root=None):
     '''
     Audits installed packages against known vulnerabilities
 
@@ -602,6 +644,10 @@ def audit(jail=None, chroot=None):
         Audit packages within the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Audit packages within the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -609,7 +655,7 @@ def audit(jail=None, chroot=None):
             salt '*' pkg.audit chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        _pkg(jail, chroot) + ['audit', '-F'],
+        _pkg(jail, chroot, root) + ['audit', '-F'],
         output_loglevel='trace',
         python_shell=False
     )
@@ -621,6 +667,7 @@ def install(name=None,
             sources=None,
             jail=None,
             chroot=None,
+            root=None,
             orphan=False,
             force=False,
             glob=False,
@@ -650,6 +697,10 @@ def install(name=None,
         Install the package into the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Install the package into the specified root (ignored if ``jail`` is
+        specified)
+
     orphan
         Mark the installed package as orphan. Will be automatically removed
         if no other packages depend on them. For more information please
@@ -762,26 +813,26 @@ def install(name=None,
         return {}
 
     opts = 'y'
-    if salt.utils.is_true(orphan):
+    if salt.utils.data.is_true(orphan):
         opts += 'A'
-    if salt.utils.is_true(force):
+    if salt.utils.data.is_true(force):
         opts += 'f'
-    if salt.utils.is_true(glob):
+    if salt.utils.data.is_true(glob):
         opts += 'g'
-    if salt.utils.is_true(local):
+    if salt.utils.data.is_true(local):
         opts += 'U'
-    if salt.utils.is_true(dryrun):
+    if salt.utils.data.is_true(dryrun):
         opts += 'n'
-    if salt.utils.is_true(quiet):
+    if salt.utils.data.is_true(quiet):
         opts += 'q'
-    if salt.utils.is_true(reinstall_requires):
+    if salt.utils.data.is_true(reinstall_requires):
         opts += 'R'
-    if salt.utils.is_true(regex):
+    if salt.utils.data.is_true(regex):
         opts += 'x'
-    if salt.utils.is_true(pcre):
+    if salt.utils.data.is_true(pcre):
         opts += 'X'
 
-    old = list_pkgs(jail=jail, chroot=chroot)
+    old = list_pkgs(jail=jail, chroot=chroot, root=root)
 
     if pkg_type == 'file':
         pkg_cmd = 'add'
@@ -801,7 +852,7 @@ def install(name=None,
             else:
                 targets.append('{0}-{1}'.format(param, version_num))
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.append(pkg_cmd)
     if fromrepo:
         cmd.extend(['-r', fromrepo])
@@ -809,7 +860,7 @@ def install(name=None,
         cmd.append('-' + opts)
     cmd.extend(targets)
 
-    if pkg_cmd == 'add' and salt.utils.is_true(dryrun):
+    if pkg_cmd == 'add' and salt.utils.data.is_true(dryrun):
         # pkg add doesn't have a dryrun mode, so echo out what will be run
         return ' '.join(cmd)
 
@@ -824,10 +875,10 @@ def install(name=None,
     else:
         errors = []
 
-    __context__.pop(_contextkey(jail, chroot), None)
-    __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
-    new = list_pkgs(jail=jail, chroot=chroot)
-    ret = salt.utils.compare_dicts(old, new)
+    __context__.pop(_contextkey(jail, chroot, root), None)
+    __context__.pop(_contextkey(jail, chroot, root, prefix='pkg.origin'), None)
+    new = list_pkgs(jail=jail, chroot=chroot, root=root)
+    ret = salt.utils.data.compare_dicts(old, new)
 
     if errors:
         raise CommandExecutionError(
@@ -842,6 +893,7 @@ def remove(name=None,
            pkgs=None,
            jail=None,
            chroot=None,
+           root=None,
            all_installed=False,
            force=False,
            glob=False,
@@ -874,6 +926,10 @@ def remove(name=None,
         Delete the package from the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Delete the package from the specified root (ignored if ``jail`` is
+        specified)
+
     all_installed
         Deletes all installed packages from the system and empties the
         database. USE WITH CAUTION!
@@ -946,12 +1002,12 @@ def remove(name=None,
         raise CommandExecutionError(exc)
 
     targets = []
-    old = list_pkgs(jail=jail, chroot=chroot, with_origin=True)
+    old = list_pkgs(jail=jail, chroot=chroot, root=root, with_origin=True)
     for pkg in pkg_params.items():
         # FreeBSD pkg supports `openjdk` and `java/openjdk7` package names
         if pkg[0].find("/") > 0:
             origin = pkg[0]
-            pkg = [k for k, v in old.iteritems() if v['origin'] == origin][0]
+            pkg = [k for k, v in six.iteritems(old) if v['origin'] == origin][0]
 
         if pkg[0] in old:
             targets.append(pkg[0])
@@ -960,24 +1016,24 @@ def remove(name=None,
         return {}
 
     opts = ''
-    if salt.utils.is_true(all_installed):
+    if salt.utils.data.is_true(all_installed):
         opts += 'a'
-    if salt.utils.is_true(force):
+    if salt.utils.data.is_true(force):
         opts += 'f'
-    if salt.utils.is_true(glob):
+    if salt.utils.data.is_true(glob):
         opts += 'g'
-    if salt.utils.is_true(dryrun):
+    if salt.utils.data.is_true(dryrun):
         opts += 'n'
-    if not salt.utils.is_true(dryrun):
+    if not salt.utils.data.is_true(dryrun):
         opts += 'y'
-    if salt.utils.is_true(recurse):
+    if salt.utils.data.is_true(recurse):
         opts += 'R'
-    if salt.utils.is_true(regex):
+    if salt.utils.data.is_true(regex):
         opts += 'x'
-    if salt.utils.is_true(pcre):
+    if salt.utils.data.is_true(pcre):
         opts += 'X'
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.append('delete')
     if opts:
         cmd.append('-' + opts)
@@ -994,10 +1050,10 @@ def remove(name=None,
     else:
         errors = []
 
-    __context__.pop(_contextkey(jail, chroot), None)
-    __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
-    new = list_pkgs(jail=jail, chroot=chroot, with_origin=True)
-    ret = salt.utils.compare_dicts(old, new)
+    __context__.pop(_contextkey(jail, chroot, root), None)
+    __context__.pop(_contextkey(jail, chroot, root, prefix='pkg.origin'), None)
+    new = list_pkgs(jail=jail, chroot=chroot, root=root, with_origin=True)
+    ret = salt.utils.data.compare_dicts(old, new)
 
     if errors:
         raise CommandExecutionError(
@@ -1008,9 +1064,9 @@ def remove(name=None,
     return ret
 
 # Support pkg.delete to remove packages, since this is the CLI usage
-delete = salt.utils.alias_function(remove, 'delete')
+delete = salt.utils.functools.alias_function(remove, 'delete')
 # No equivalent to purge packages, use remove instead
-purge = salt.utils.alias_function(remove, 'purge')
+purge = salt.utils.functools.alias_function(remove, 'purge')
 
 
 def upgrade(*names, **kwargs):
@@ -1018,6 +1074,14 @@ def upgrade(*names, **kwargs):
     Upgrade named or all packages (run a ``pkg upgrade``). If <package name> is
     omitted, the operation is executed on all packages.
 
+    Returns a dictionary containing the changes:
+
+    .. code-block:: python
+
+        {'<package>':  {'old': '<old-version>',
+                        'new': '<new-version>'}}
+
+
     CLI Example:
 
     .. code-block:: bash
@@ -1037,6 +1101,10 @@ def upgrade(*names, **kwargs):
         Audit packages within the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Audit packages within the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1077,13 +1145,9 @@ def upgrade(*names, **kwargs):
 
             salt '*' pkg.upgrade <package name> dryrun=True
     '''
-    ret = {'changes': {},
-           'result': True,
-           'comment': '',
-           }
-
     jail = kwargs.pop('jail', None)
     chroot = kwargs.pop('chroot', None)
+    root = kwargs.pop('root', None)
     force = kwargs.pop('force', False)
     local = kwargs.pop('local', False)
     dryrun = kwargs.pop('dryrun', False)
@@ -1096,35 +1160,32 @@ def upgrade(*names, **kwargs):
         opts += 'n'
     if not dryrun:
         opts += 'y'
-    if opts:
-        opts = '-' + opts
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.append('upgrade')
     if opts:
         cmd.append('-' + opts)
     cmd.extend(names)
 
     old = list_pkgs()
-    call = __salt__['cmd.run_all'](cmd,
-                                   output_loglevel='trace',
-                                   python_shell=False,
-                                   redirect_stderr=True)
-
-    if call['retcode'] != 0:
-        ret['result'] = False
-        if call['stdout']:
-            ret['comment'] = call['stdout']
-
-    __context__.pop(_contextkey(jail, chroot), None)
-    __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
+    result = __salt__['cmd.run_all'](cmd,
+                                     output_loglevel='trace',
+                                     python_shell=False)
+    __context__.pop(_contextkey(jail, chroot, root), None)
+    __context__.pop(_contextkey(jail, chroot, root, prefix='pkg.origin'), None)
     new = list_pkgs()
-    ret['changes'] = salt.utils.compare_dicts(old, new)
+    ret = salt.utils.data.compare_dicts(old, new)
+
+    if result['retcode'] != 0:
+        raise CommandExecutionError(
+            'Problem encountered upgrading packages',
+            info={'changes': ret, 'result': result}
+        )
 
     return ret
 
 
-def clean(jail=None, chroot=None):
+def clean(jail=None, chroot=None, root=None):
     '''
     Cleans the local cache of fetched remote packages
 
@@ -1137,13 +1198,13 @@ def clean(jail=None, chroot=None):
         salt '*' pkg.clean chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        _pkg(jail, chroot) + ['clean'],
+        _pkg(jail, chroot, root) + ['clean'],
         output_loglevel='trace',
         python_shell=False
     )
 
 
-def autoremove(jail=None, chroot=None, dryrun=False):
+def autoremove(jail=None, chroot=None, root=None, dryrun=False):
     '''
     Delete packages which were automatically installed as dependencies and are
     not required anymore.
@@ -1167,7 +1228,7 @@ def autoremove(jail=None, chroot=None, dryrun=False):
     else:
         opts += 'y'
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.append('autoremove')
     if opts:
         cmd.append('-' + opts)
@@ -1180,6 +1241,7 @@ def autoremove(jail=None, chroot=None, dryrun=False):
 
 def check(jail=None,
           chroot=None,
+          root=None,
           depends=False,
           recompute=False,
           checksum=False):
@@ -1199,6 +1261,10 @@ def check(jail=None,
         Perform the sanity check in the specified chroot (ignored if ``jail``
         is specified)
 
+    root
+        Perform the sanity check in the specified root (ignored if ``jail``
+        is specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1246,7 +1312,7 @@ def check(jail=None,
     if checksum:
         opts += 's'
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.append('check')
     if opts:
         cmd.append('-' + opts)
@@ -1257,7 +1323,7 @@ def check(jail=None,
     )
 
 
-def which(path, jail=None, chroot=None, origin=False, quiet=False):
+def which(path, jail=None, chroot=None, root=None, origin=False, quiet=False):
     '''
     Displays which package installed a specific file
 
@@ -1280,6 +1346,10 @@ def which(path, jail=None, chroot=None, origin=False, quiet=False):
         Perform the check in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Perform the check in the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1311,7 +1381,7 @@ def which(path, jail=None, chroot=None, origin=False, quiet=False):
     if origin:
         opts += 'o'
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.append('which')
     if opts:
         cmd.append('-' + opts)
@@ -1326,6 +1396,7 @@ def which(path, jail=None, chroot=None, origin=False, quiet=False):
 def search(name,
            jail=None,
            chroot=None,
+           root=None,
            exact=False,
            glob=False,
            regex=False,
@@ -1360,6 +1431,10 @@ def search(name,
         Perform the search using the ``pkg.conf(5)`` from the specified chroot
         (ignored if ``jail`` is specified)
 
+    root
+        Perform the search using the ``pkg.conf(5)`` from the specified root
+        (ignored if ``jail`` is specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1502,7 +1577,7 @@ def search(name,
     if prefix:
         opts += 'p'
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.append('search')
     if opts:
         cmd.append('-' + opts)
@@ -1517,6 +1592,7 @@ def search(name,
 def fetch(name,
           jail=None,
           chroot=None,
+          root=None,
           fetch_all=False,
           quiet=False,
           fromrepo=None,
@@ -1547,6 +1623,10 @@ def fetch(name,
         Fetch package in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Fetch package in the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1643,7 +1723,7 @@ def fetch(name,
     if depends:
         opts += 'd'
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.extend(['fetch', '-y'])
     if fromrepo:
         cmd.extend(['-r', fromrepo])
@@ -1660,6 +1740,7 @@ def fetch(name,
 def updating(name,
              jail=None,
              chroot=None,
+             root=None,
              filedate=None,
              filename=None):
     ''''
@@ -1684,6 +1765,10 @@ def updating(name,
         Perform the action in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Perform the action in the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1715,7 +1800,7 @@ def updating(name,
     if filename:
         opts += 'f {0}'.format(filename)
 
-    cmd = _pkg(jail, chroot)
+    cmd = _pkg(jail, chroot, root)
     cmd.append('updating')
     if opts:
         cmd.append('-' + opts)
