@@ -2,13 +2,13 @@
 '''
 Control a salt cloud system
 '''
-from __future__ import absolute_import
-
 # Import python libs
-import json
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import salt libs
-import salt.utils
+import salt.utils.data
+import salt.utils.json
+
 HAS_CLOUD = False
 try:
     import saltcloud  # pylint: disable=W0611
@@ -42,7 +42,7 @@ def create(name, profile):
     cmd = 'salt-cloud --out json -p {0} {1}'.format(profile, name)
     out = __salt__['cmd.run_stdout'](cmd, python_shell=False)
     try:
-        ret = json.loads(out, object_hook=salt.utils.decode_dict)
+        ret = salt.utils.json.loads(out)
     except ValueError:
         ret = {}
     return ret
