@@ -6,7 +6,7 @@ minion.
 
 :depends:   - esky Python module for update functionality
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import copy
@@ -15,6 +15,8 @@ import logging
 import os
 import signal
 import sys
+import time
+import shutil
 
 # Import 3rd-party libs
 # pylint: disable=import-error
@@ -44,13 +46,16 @@ import salt.payload
 import salt.runner
 import salt.state
 import salt.transport
-import salt.utils
 import salt.utils.args
 import salt.utils.event
 import salt.utils.extmods
+import salt.utils.files
+import salt.utils.functools
 import salt.utils.minion
+import salt.utils.path
 import salt.utils.process
 import salt.utils.url
+import salt.utils.versions
 import salt.wheel
 
 HAS_PSUTIL = True
@@ -76,7 +81,8 @@ def _get_top_file_envs():
         return __context__['saltutil._top_file_envs']
     except KeyError:
         try:
-            st_ = salt.state.HighState(__opts__)
+            st_ = salt.state.HighState(__opts__,
+                                       initial_pillar=__pillar__)
             top = st_.get_top()
             if top:
                 envs = list(st_.top_matches(top).keys()) or 'base'
@@ -90,7 +96,7 @@ def _get_top_file_envs():
         return envs
 
 
-def _sync(form, saltenv=None):
+def _sync(form, saltenv=None, extmod_whitelist=None, extmod_blacklist=None):
     '''
     Sync the given directory in the given environment
     '''
@@ -98,12 +104,13 @@ def _sync(form, saltenv=None):
         saltenv = _get_top_file_envs()
     if isinstance(saltenv, six.string_types):
         saltenv = saltenv.split(',')
-    ret, touched = salt.utils.extmods.sync(__opts__, form, saltenv=saltenv)
+    ret, touched = salt.utils.extmods.sync(__opts__, form, saltenv=saltenv, extmod_whitelist=extmod_whitelist,
+                                           extmod_blacklist=extmod_blacklist)
     # Dest mod_dir is touched? trigger reload if requested
     if touched:
         mod_file = os.path.join(__opts__['cachedir'], 'module_refresh')
-        with salt.utils.fopen(mod_file, 'a+') as ofile:
-            ofile.write('')
+        with salt.utils.files.fopen(mod_file, 'a'):
+            pass
     if form == 'grains' and \
        __opts__.get('grains_cache') and \
        os.path.isfile(os.path.join(__opts__['cachedir'], 'grains.cache.p')):
@@ -180,21 +187,31 @@ def update(version=None):
     return ret
 
 
-def sync_beacons(saltenv=None, refresh=True):
+def sync_beacons(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.5.1
 
     Sync beacons from ``salt://_beacons`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for beacons to sync. If no top files are
+        found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available beacons on the minion. This refresh
         will be performed even if no new beacons are synced. Set to ``False``
         to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Example:
 
     .. code-block:: bash
@@ -203,26 +220,36 @@ def sync_beacons(saltenv=None, refresh=True):
         salt '*' saltutil.sync_beacons saltenv=dev
         salt '*' saltutil.sync_beacons saltenv=base,dev
     '''
-    ret = _sync('beacons', saltenv)
+    ret = _sync('beacons', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_beacons()
     return ret
 
 
-def sync_sdb(saltenv=None):
+def sync_sdb(saltenv=None, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.5.8,2015.8.3
 
     Sync sdb modules from ``salt://_sdb`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for sdb modules to sync. If no top files
+        are found, then the ``base`` environment will be synced.
+
     refresh : False
         This argument has no affect and is included for consistency with the
         other sync functions.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Example:
 
     .. code-block:: bash
@@ -231,20 +258,24 @@ def sync_sdb(saltenv=None):
         salt '*' saltutil.sync_sdb saltenv=dev
         salt '*' saltutil.sync_sdb saltenv=base,dev
     '''
-    ret = _sync('sdb', saltenv)
+    ret = _sync('sdb', saltenv, extmod_whitelist, extmod_blacklist)
     return ret
 
 
-def sync_modules(saltenv=None, refresh=True):
+def sync_modules(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
     Sync execution modules from ``salt://_modules`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for execution modules to sync. If no top
+        files are found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available execution modules on the minion.
         This refresh will be performed even if no new execution modules are
@@ -267,6 +298,12 @@ def sync_modules(saltenv=None, refresh=True):
         See :ref:`here <reloading-modules>` for a more detailed explanation of
         why this is necessary.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Example:
 
     .. code-block:: bash
@@ -275,27 +312,37 @@ def sync_modules(saltenv=None, refresh=True):
         salt '*' saltutil.sync_modules saltenv=dev
         salt '*' saltutil.sync_modules saltenv=base,dev
     '''
-    ret = _sync('modules', saltenv)
+    ret = _sync('modules', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_states(saltenv=None, refresh=True):
+def sync_states(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
     Sync state modules from ``salt://_states`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for state modules to sync. If no top
+        files are found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available states on the minion. This refresh
         will be performed even if no new state modules are synced. Set to
         ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -304,7 +351,7 @@ def sync_states(saltenv=None, refresh=True):
         salt '*' saltutil.sync_states saltenv=dev
         salt '*' saltutil.sync_states saltenv=base,dev
     '''
-    ret = _sync('states', saltenv)
+    ret = _sync('states', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
@@ -312,7 +359,7 @@ def sync_states(saltenv=None, refresh=True):
 
 def refresh_grains(**kwargs):
     '''
-    .. versionadded:: 2016.3.6,2016.11.4,Nitrogen
+    .. versionadded:: 2016.3.6,2016.11.4,2017.7.0
 
     Refresh the minion's grains without syncing custom grains modules from
     ``salt://_grains``.
@@ -330,10 +377,10 @@ def refresh_grains(**kwargs):
 
         salt '*' saltutil.refresh_grains
     '''
-    kwargs = salt.utils.clean_kwargs(**kwargs)
+    kwargs = salt.utils.args.clean_kwargs(**kwargs)
     _refresh_pillar = kwargs.pop('refresh_pillar', True)
     if kwargs:
-        salt.utils.invalid_kwargs(kwargs)
+        salt.utils.args.invalid_kwargs(kwargs)
     # Modules and pillar need to be refreshed in case grains changes affected
     # them, and the module refresh process reloads the grains and assigns the
     # newly-reloaded grains to each execution module's __grains__ dunder.
@@ -343,22 +390,32 @@ def refresh_grains(**kwargs):
     return True
 
 
-def sync_grains(saltenv=None, refresh=True):
+def sync_grains(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
     Sync grains modules from ``salt://_grains`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for grains modules to sync. If no top
+        files are found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available execution modules and recompile
         pillar data for the minion. This refresh will be performed even if no
         new grains modules are synced. Set to ``False`` to prevent this
         refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -367,29 +424,39 @@ def sync_grains(saltenv=None, refresh=True):
         salt '*' saltutil.sync_grains saltenv=dev
         salt '*' saltutil.sync_grains saltenv=base,dev
     '''
-    ret = _sync('grains', saltenv)
+    ret = _sync('grains', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
         refresh_pillar()
     return ret
 
 
-def sync_renderers(saltenv=None, refresh=True):
+def sync_renderers(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
     Sync renderers from ``salt://_renderers`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for renderers to sync. If no top files
+        are found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available execution modules on the minion.
         This refresh will be performed even if no new renderers are synced.
         Set to ``False`` to prevent this refresh. Set to ``False`` to prevent
         this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -398,27 +465,37 @@ def sync_renderers(saltenv=None, refresh=True):
         salt '*' saltutil.sync_renderers saltenv=dev
         salt '*' saltutil.sync_renderers saltenv=base,dev
     '''
-    ret = _sync('renderers', saltenv)
+    ret = _sync('renderers', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_returners(saltenv=None, refresh=True):
+def sync_returners(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
-    Sync beacons from ``salt://_returners`` to the minion
+    Sync returners from ``salt://_returners`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for returners to sync. If no top files
+        are found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available execution modules on the minion.
         This refresh will be performed even if no new returners are synced. Set
         to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -426,27 +503,37 @@ def sync_returners(saltenv=None, refresh=True):
         salt '*' saltutil.sync_returners
         salt '*' saltutil.sync_returners saltenv=dev
     '''
-    ret = _sync('returners', saltenv)
+    ret = _sync('returners', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_proxymodules(saltenv=None, refresh=False):
+def sync_proxymodules(saltenv=None, refresh=False, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.8.2
 
     Sync proxy modules from ``salt://_proxy`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for proxy modules to sync. If no top
+        files are found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available execution modules on the minion.
         This refresh will be performed even if no new proxy modules are synced.
         Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -455,27 +542,37 @@ def sync_proxymodules(saltenv=None, refresh=False):
         salt '*' saltutil.sync_proxymodules saltenv=dev
         salt '*' saltutil.sync_proxymodules saltenv=base,dev
     '''
-    ret = _sync('proxy', saltenv)
+    ret = _sync('proxy', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_engines(saltenv=None, refresh=False):
+def sync_engines(saltenv=None, refresh=False, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2016.3.0
 
     Sync engine modules from ``salt://_engines`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for engines to sync. If no top files are
+        found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available execution modules on the minion.
         This refresh will be performed even if no new engine modules are synced.
         Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -483,25 +580,73 @@ def sync_engines(saltenv=None, refresh=False):
         salt '*' saltutil.sync_engines
         salt '*' saltutil.sync_engines saltenv=base,dev
     '''
-    ret = _sync('engines', saltenv)
+    ret = _sync('engines', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_output(saltenv=None, refresh=True):
+def sync_thorium(saltenv=None, refresh=False, extmod_whitelist=None, extmod_blacklist=None):
+    '''
+    .. versionadded:: 2018.3.0
+
+    Sync Thorium modules from ``salt://_thorium`` to the minion
+
+    saltenv
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for engines to sync. If no top files are
+        found, then the ``base`` environment will be synced.
+
+    refresh: ``True``
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new Thorium modules are synced.
+        Set to ``False`` to prevent this refresh.
+
+    extmod_whitelist
+        comma-seperated list of modules to sync
+
+    extmod_blacklist
+        comma-seperated list of modules to blacklist based on type
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.sync_thorium
+        salt '*' saltutil.sync_thorium saltenv=base,dev
+    '''
+    ret = _sync('thorium', saltenv, extmod_whitelist, extmod_blacklist)
+    if refresh:
+        refresh_modules()
+    return ret
+
+
+def sync_output(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     Sync outputters from ``salt://_output`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for outputters to sync. If no top files
+        are found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available execution modules on the minion.
         This refresh will be performed even if no new outputters are synced.
         Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -510,19 +655,19 @@ def sync_output(saltenv=None, refresh=True):
         salt '*' saltutil.sync_output saltenv=dev
         salt '*' saltutil.sync_output saltenv=base,dev
     '''
-    ret = _sync('output', saltenv)
+    ret = _sync('output', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
-sync_outputters = salt.utils.alias_function(sync_output, 'sync_outputters')
+sync_outputters = salt.utils.functools.alias_function(sync_output, 'sync_outputters')
 
 
-def sync_utils(saltenv=None, refresh=True):
+def sync_clouds(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
-    .. versionadded:: 2014.7.0
+    .. versionadded:: 2017.7.0
 
-    Sync utility modules from ``salt://_utils`` to the minion
+    Sync cloud modules from ``salt://_cloud`` to the minion
 
     saltenv : base
         The fileserver environment from which to sync. To sync from more than
@@ -533,6 +678,51 @@ def sync_utils(saltenv=None, refresh=True):
         This refresh will be performed even if no new utility modules are
         synced. Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.sync_clouds
+        salt '*' saltutil.sync_clouds saltenv=dev
+        salt '*' saltutil.sync_clouds saltenv=base,dev
+    '''
+    ret = _sync('clouds', saltenv, extmod_whitelist, extmod_blacklist)
+    if refresh:
+        refresh_modules()
+    return ret
+
+
+def sync_utils(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
+    '''
+    .. versionadded:: 2014.7.0
+
+    Sync utility modules from ``salt://_utils`` to the minion
+
+    saltenv
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for utility modules to sync. If no top
+        files are found, then the ``base`` environment will be synced.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new utility modules are
+        synced. Set to ``False`` to prevent this refresh.
+
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -541,27 +731,61 @@ def sync_utils(saltenv=None, refresh=True):
         salt '*' saltutil.sync_utils saltenv=dev
         salt '*' saltutil.sync_utils saltenv=base,dev
     '''
-    ret = _sync('utils', saltenv)
+    ret = _sync('utils', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_log_handlers(saltenv=None, refresh=True):
+def list_extmods():
+    '''
+    .. versionadded:: 2017.7.0
+
+    List Salt modules which have been synced externally
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.list_extmods
+    '''
+    ret = {}
+    ext_dir = os.path.join(__opts__['cachedir'], 'extmods')
+    mod_types = os.listdir(ext_dir)
+    for mod_type in mod_types:
+        ret[mod_type] = set()
+        for _, _, files in salt.utils.path.os_walk(os.path.join(ext_dir, mod_type)):
+            for fh_ in files:
+                ret[mod_type].add(fh_.split('.')[0])
+        ret[mod_type] = list(ret[mod_type])
+    return ret
+
+
+def sync_log_handlers(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.8.0
 
     Sync log handlers from ``salt://_log_handlers`` to the minion
 
-    saltenv : base
+    saltenv
         The fileserver environment from which to sync. To sync from more than
         one environment, pass a comma-separated list.
 
+        If not passed, then all environments configured in the :ref:`top files
+        <states-top>` will be checked for log handlers to sync. If no top files
+        are found, then the ``base`` environment will be synced.
+
     refresh : True
         If ``True``, refresh the available execution modules on the minion.
         This refresh will be performed even if no new log handlers are synced.
         Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -570,13 +794,13 @@ def sync_log_handlers(saltenv=None, refresh=True):
         salt '*' saltutil.sync_log_handlers saltenv=dev
         salt '*' saltutil.sync_log_handlers saltenv=base,dev
     '''
-    ret = _sync('log_handlers', saltenv)
+    ret = _sync('log_handlers', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_pillar(saltenv=None, refresh=True):
+def sync_pillar(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.8.11,2016.3.2
 
@@ -589,6 +813,12 @@ def sync_pillar(saltenv=None, refresh=True):
         Also refresh the execution modules available to the minion, and refresh
         pillar data.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     .. note::
         This function will raise an error if executed on a traditional (i.e.
         not masterless) minion
@@ -604,14 +834,14 @@ def sync_pillar(saltenv=None, refresh=True):
         raise CommandExecutionError(
             'Pillar modules can only be synced to masterless minions'
         )
-    ret = _sync('pillar', saltenv)
+    ret = _sync('pillar', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
         refresh_pillar()
     return ret
 
 
-def sync_all(saltenv=None, refresh=True):
+def sync_all(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionchanged:: 2015.8.11,2016.3.2
         On masterless minions, pillar modules are now synced, and refreshed
@@ -643,6 +873,12 @@ def sync_all(saltenv=None, refresh=True):
         See :ref:`here <reloading-modules>` for a more detailed explanation of
         why this is necessary.
 
+    extmod_whitelist : None
+        dictionary of modules to sync based on type
+
+    extmod_blacklist : None
+        dictionary of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -650,23 +886,26 @@ def sync_all(saltenv=None, refresh=True):
         salt '*' saltutil.sync_all
         salt '*' saltutil.sync_all saltenv=dev
         salt '*' saltutil.sync_all saltenv=base,dev
+        salt '*' saltutil.sync_all extmod_whitelist={'modules': ['custom_module']}
     '''
     log.debug('Syncing all')
     ret = {}
-    ret['beacons'] = sync_beacons(saltenv, False)
-    ret['modules'] = sync_modules(saltenv, False)
-    ret['states'] = sync_states(saltenv, False)
-    ret['sdb'] = sync_sdb(saltenv)
-    ret['grains'] = sync_grains(saltenv, False)
-    ret['renderers'] = sync_renderers(saltenv, False)
-    ret['returners'] = sync_returners(saltenv, False)
-    ret['output'] = sync_output(saltenv, False)
-    ret['utils'] = sync_utils(saltenv, False)
-    ret['log_handlers'] = sync_log_handlers(saltenv, False)
-    ret['proxymodules'] = sync_proxymodules(saltenv, False)
-    ret['engines'] = sync_engines(saltenv, False)
+    ret['clouds'] = sync_clouds(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['beacons'] = sync_beacons(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['modules'] = sync_modules(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['states'] = sync_states(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['sdb'] = sync_sdb(saltenv, extmod_whitelist, extmod_blacklist)
+    ret['grains'] = sync_grains(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['renderers'] = sync_renderers(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['returners'] = sync_returners(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['output'] = sync_output(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['utils'] = sync_utils(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['log_handlers'] = sync_log_handlers(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['proxymodules'] = sync_proxymodules(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['engines'] = sync_engines(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['thorium'] = sync_thorium(saltenv, False, extmod_whitelist, extmod_blacklist)
     if __opts__['file_client'] == 'local':
-        ret['pillar'] = sync_pillar(saltenv, False)
+        ret['pillar'] = sync_pillar(saltenv, False, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
         refresh_pillar()
@@ -708,7 +947,7 @@ def refresh_pillar():
         ret = False  # Effectively a no-op, since we can't really return without an event system
     return ret
 
-pillar_refresh = salt.utils.alias_function(refresh_pillar, 'pillar_refresh')
+pillar_refresh = salt.utils.functools.alias_function(refresh_pillar, 'pillar_refresh')
 
 
 def refresh_modules(async=True):
@@ -789,12 +1028,45 @@ def clear_cache():
 
         salt '*' saltutil.clear_cache
     '''
-    for root, dirs, files in salt.utils.safe_walk(__opts__['cachedir'], followlinks=False):
+    for root, dirs, files in salt.utils.files.safe_walk(__opts__['cachedir'], followlinks=False):
         for name in files:
             try:
                 os.remove(os.path.join(root, name))
             except OSError as exc:
-                log.error('Attempt to clear cache with saltutil.clear_cache FAILED with: {0}'.format(exc))
+                log.error(
+                    'Attempt to clear cache with saltutil.clear_cache '
+                    'FAILED with: %s', exc
+                )
+                return False
+    return True
+
+
+def clear_job_cache(hours=24):
+    '''
+    Forcibly removes job cache folders and files on a minion.
+
+    .. versionadded:: 2018.3.0
+
+    WARNING: The safest way to clear a minion cache is by first stopping
+    the minion and then deleting the cache files before restarting it.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.clear_job_cache hours=12
+    '''
+    threshold = time.time() - hours * 3600
+    for root, dirs, files in salt.utils.files.safe_walk(os.path.join(__opts__['cachedir'], 'minion_jobs'),
+                                                  followlinks=False):
+        for name in dirs:
+            try:
+                directory = os.path.join(root, name)
+                mtime = os.path.getmtime(directory)
+                if mtime < threshold:
+                    shutil.rmtree(directory)
+            except OSError as exc:
+                log.error('Attempt to clear cache with saltutil.clear_job_cache FAILED with: %s', exc)
                 return False
     return True
 
@@ -864,7 +1136,7 @@ def find_cached_job(jid):
     '''
     serial = salt.payload.Serial(__opts__)
     proc_dir = os.path.join(__opts__['cachedir'], 'minion_jobs')
-    job_dir = os.path.join(proc_dir, str(jid))
+    job_dir = os.path.join(proc_dir, six.text_type(jid))
     if not os.path.isdir(job_dir):
         if not __opts__.get('cache_jobs'):
             return ('Local jobs cache directory not found; you may need to'
@@ -872,21 +1144,19 @@ def find_cached_job(jid):
         else:
             return 'Local jobs cache directory {0} not found'.format(job_dir)
     path = os.path.join(job_dir, 'return.p')
-    with salt.utils.fopen(path, 'rb') as fp_:
+    with salt.utils.files.fopen(path, 'rb') as fp_:
         buf = fp_.read()
-        fp_.close()
-        if buf:
-            try:
-                data = serial.loads(buf)
-            except NameError:
-                # msgpack error in salt-ssh
-                return
+    if buf:
+        try:
+            data = serial.loads(buf)
+        except NameError:
+            # msgpack error in salt-ssh
+            pass
         else:
-            return
-    if not isinstance(data, dict):
-        # Invalid serial object
-        return
-    return data
+            if isinstance(data, dict):
+                # if not a dict, this was an invalid serialized object
+                return data
+    return None
 
 
 def signal_job(jid, sig):
@@ -919,7 +1189,7 @@ def signal_job(jid, sig):
                         data['pid']
                         )
             except OSError:
-                path = os.path.join(__opts__['cachedir'], 'proc', str(jid))
+                path = os.path.join(__opts__['cachedir'], 'proc', six.text_type(jid))
                 if os.path.isfile(path):
                     os.remove(path)
                 return ('Job {0} was not running and job data has been '
@@ -1035,7 +1305,7 @@ def revoke_auth(preserve_minion_cache=False):
 
     for master in masters:
         channel = salt.transport.Channel.factory(__opts__, master_uri=master)
-        tok = channel.auth.gen_token('salt')
+        tok = channel.auth.gen_token(b'salt')
         load = {'cmd': 'revoke_auth',
                 'id': __opts__['id'],
                 'tok': tok,
@@ -1055,28 +1325,44 @@ def _get_ssh_or_api_client(cfgfile, ssh=False):
     return client
 
 
-def _exec(client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs):
+def _exec(client, tgt, fun, arg, timeout, tgt_type, ret, kwarg, **kwargs):
+    if 'expr_form' in kwargs:
+        salt.utils.versions.warn_until(
+            'Fluorine',
+            'The target type should be passed using the \'tgt_type\' '
+            'argument instead of \'expr_form\'. Support for using '
+            '\'expr_form\' will be removed in Salt Fluorine.'
+        )
+        tgt_type = kwargs.pop('expr_form')
+
     fcn_ret = {}
     seen = 0
     if 'batch' in kwargs:
         _cmd = client.cmd_batch
         cmd_kwargs = {
-            'tgt': tgt, 'fun': fun, 'arg': arg, 'expr_form': expr_form,
+            'tgt': tgt, 'fun': fun, 'arg': arg, 'tgt_type': tgt_type,
             'ret': ret, 'kwarg': kwarg, 'batch': kwargs['batch'],
         }
         del kwargs['batch']
+    elif 'subset' in kwargs:
+        _cmd = client.cmd_subset
+        cmd_kwargs = {
+            'tgt': tgt, 'fun': fun, 'arg': arg, 'tgt_type': tgt_type,
+            'ret': ret, 'kwarg': kwarg, 'sub': kwargs['subset'],
+        }
+        del kwargs['subset']
     else:
         _cmd = client.cmd_iter
         cmd_kwargs = {
             'tgt': tgt, 'fun': fun, 'arg': arg, 'timeout': timeout,
-            'expr_form': expr_form, 'ret': ret, 'kwarg': kwarg,
+            'tgt_type': tgt_type, 'ret': ret, 'kwarg': kwarg,
         }
     cmd_kwargs.update(kwargs)
     for ret_comp in _cmd(**cmd_kwargs):
         fcn_ret.update(ret_comp)
         seen += 1
         # fcn_ret can be empty, so we cannot len the whole return dict
-        if expr_form == 'list' and len(tgt) == seen:
+        if tgt_type == 'list' and len(tgt) == seen:
             # do not wait for timeout when explicit list matching
             # and all results are there
             break
@@ -1087,12 +1373,16 @@ def cmd(tgt,
         fun,
         arg=(),
         timeout=None,
-        expr_form='glob',
+        tgt_type='glob',
         ret='',
         kwarg=None,
         ssh=False,
         **kwargs):
     '''
+    .. versionchanged:: 2017.7.0
+        The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
+        releases must use ``expr_form``.
+
     Assuming this minion is a master, execute a salt command
 
     CLI Example:
@@ -1104,7 +1394,7 @@ def cmd(tgt,
     cfgfile = __opts__['conf_file']
     client = _get_ssh_or_api_client(cfgfile, ssh)
     fcn_ret = _exec(
-        client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs)
+        client, tgt, fun, arg, timeout, tgt_type, ret, kwarg, **kwargs)
     # if return is empty, we may have not used the right conf,
     # try with the 'minion relative master configuration counter part
     # if available
@@ -1116,7 +1406,7 @@ def cmd(tgt,
     ):
         client = _get_ssh_or_api_client(master_cfgfile, ssh)
         fcn_ret = _exec(
-            client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs)
+            client, tgt, fun, arg, timeout, tgt_type, ret, kwarg, **kwargs)
 
     if 'batch' in kwargs:
         old_ret, fcn_ret = fcn_ret, {}
@@ -1133,12 +1423,16 @@ def cmd_iter(tgt,
              fun,
              arg=(),
              timeout=None,
-             expr_form='glob',
+             tgt_type='glob',
              ret='',
              kwarg=None,
              ssh=False,
              **kwargs):
     '''
+    .. versionchanged:: 2017.7.0
+        The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
+        releases must use ``expr_form``.
+
     Assuming this minion is a master, execute a salt command
 
     CLI Example:
@@ -1156,14 +1450,14 @@ def cmd_iter(tgt,
             fun,
             arg,
             timeout,
-            expr_form,
+            tgt_type,
             ret,
             kwarg,
             **kwargs):
         yield ret
 
 
-def runner(name, **kwargs):
+def runner(name, arg=None, kwarg=None, full_return=False, saltenv='base', jid=None, **kwargs):
     '''
     Execute a runner function. This function must be run on the master,
     either by targeting a minion running on a master or by using
@@ -1185,11 +1479,17 @@ def runner(name, **kwargs):
     .. code-block:: bash
 
         salt master_minion saltutil.runner jobs.list_jobs
+        salt master_minion saltutil.runner test.arg arg="['baz']" kwarg="{'foo': 'bar'}"
     '''
-    jid = kwargs.pop('__orchestration_jid__', None)
-    saltenv = kwargs.pop('__env__', 'base')
-    full_return = kwargs.pop('full_return', False)
-    kwargs = salt.utils.clean_kwargs(**kwargs)
+    if arg is None:
+        arg = []
+    if kwarg is None:
+        kwarg = {}
+    jid = kwargs.pop('__orchestration_jid__', jid)
+    saltenv = kwargs.pop('__env__', saltenv)
+    kwargs = salt.utils.args.clean_kwargs(**kwargs)
+    if kwargs:
+        kwarg.update(kwargs)
 
     if 'master_job_cache' not in __opts__:
         master_config = os.path.join(os.path.dirname(__opts__['conf_file']),
@@ -1202,18 +1502,19 @@ def runner(name, **kwargs):
     if name in rclient.functions:
         aspec = salt.utils.args.get_function_argspec(rclient.functions[name])
         if 'saltenv' in aspec.args:
-            kwargs['saltenv'] = saltenv
+            kwarg['saltenv'] = saltenv
 
     if jid:
         salt.utils.event.fire_args(
             __opts__,
             jid,
-            {'type': 'runner', 'name': name, 'args': kwargs},
+            {'type': 'runner', 'name': name, 'args': arg, 'kwargs': kwarg},
             prefix='run'
         )
 
     return rclient.cmd(name,
-                       kwarg=kwargs,
+                       arg=arg,
+                       kwarg=kwarg,
                        print_event=False,
                        full_return=full_return)
 
