@@ -12,17 +12,19 @@ Execute calls on selinux
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 import os
 import re
 
 # Import salt libs
-import salt.utils
+import salt.utils.files
+import salt.utils.path
+import salt.utils.stringutils
 import salt.utils.decorators as decorators
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 
 
 _SELINUX_FILETYPES = {
@@ -46,7 +48,7 @@ def __virtual__():
     # Iterate over all of the commands this module uses and make sure
     # each of them are available in the standard PATH to prevent breakage
     for cmd in required_cmds:
-        if not salt.utils.which(cmd):
+        if not salt.utils.path.which(cmd):
             return (False, cmd + ' is not in the path')
     # SELinux only makes sense on Linux *obviously*
     if __grains__['kernel'] == 'Linux':
@@ -94,8 +96,8 @@ def getenforce():
         return 'Disabled'
     try:
         enforce = os.path.join(_selinux_fs_path, 'enforce')
-        with salt.utils.fopen(enforce, 'r') as _fp:
-            if _fp.readline().strip() == '0':
+        with salt.utils.files.fopen(enforce, 'r') as _fp:
+            if salt.utils.stringutils.to_unicode(_fp.readline()).strip() == '0':
                 return 'Permissive'
             else:
                 return 'Enforcing'
@@ -115,8 +117,9 @@ def getconfig():
     '''
     try:
         config = '/etc/selinux/config'
-        with salt.utils.fopen(config, 'r') as _fp:
+        with salt.utils.files.fopen(config, 'r') as _fp:
             for line in _fp:
+                line = salt.utils.stringutils.to_unicode(line)
                 if line.strip().startswith('SELINUX='):
                     return line.split('=')[1].capitalize().strip()
     except (IOError, OSError, AttributeError):
@@ -158,26 +161,26 @@ def setenforce(mode):
     if getenforce() != 'Disabled':
         enforce = os.path.join(selinux_fs_path(), 'enforce')
         try:
-            with salt.utils.fopen(enforce, 'w') as _fp:
-                _fp.write(mode)
+            with salt.utils.files.fopen(enforce, 'w') as _fp:
+                _fp.write(salt.utils.stringutils.to_str(mode))
         except (IOError, OSError) as exc:
             msg = 'Could not write SELinux enforce file: {0}'
-            raise CommandExecutionError(msg.format(str(exc)))
+            raise CommandExecutionError(msg.format(exc))
 
     config = '/etc/selinux/config'
     try:
-        with salt.utils.fopen(config, 'r') as _cf:
+        with salt.utils.files.fopen(config, 'r') as _cf:
             conf = _cf.read()
         try:
-            with salt.utils.fopen(config, 'w') as _cf:
+            with salt.utils.files.fopen(config, 'w') as _cf:
                 conf = re.sub(r"\nSELINUX=.*\n", "\nSELINUX=" + modestring + "\n", conf)
-                _cf.write(conf)
+                _cf.write(salt.utils.stringutils.to_str(conf))
         except (IOError, OSError) as exc:
             msg = 'Could not write SELinux config file: {0}'
-            raise CommandExecutionError(msg.format(str(exc)))
+            raise CommandExecutionError(msg.format(exc))
     except (IOError, OSError) as exc:
         msg = 'Could not read SELinux config file: {0}'
-        raise CommandExecutionError(msg.format(str(exc)))
+        raise CommandExecutionError(msg.format(exc))
 
     return getenforce()
 
@@ -374,8 +377,10 @@ def list_semod():
 
 def _validate_filetype(filetype):
     '''
-    Checks if the given filetype is a valid SELinux filetype specification.
-    Throws an SaltInvocationError if it isn't.
+    .. versionadded:: 2017.7.0
+
+    Checks if the given filetype is a valid SELinux filetype
+    specification. Throws an SaltInvocationError if it isn't.
     '''
     if filetype not in _SELINUX_FILETYPES.keys():
         raise SaltInvocationError('Invalid filetype given: {0}'.format(filetype))
@@ -384,6 +389,8 @@ def _validate_filetype(filetype):
 
 def _context_dict_to_string(context):
     '''
+    .. versionadded:: 2017.7.0
+
     Converts an SELinux file context from a dict to a string.
     '''
     return '{sel_user}:{sel_role}:{sel_type}:{sel_level}'.format(**context)
@@ -391,6 +398,8 @@ def _context_dict_to_string(context):
 
 def _context_string_to_dict(context):
     '''
+    .. versionadded:: 2017.7.0
+
     Converts an SELinux file context from string to dict.
     '''
     if not re.match('[^:]+:[^:]+:[^:]+:[^:]+$', context):
@@ -405,8 +414,11 @@ def _context_string_to_dict(context):
 
 def filetype_id_to_string(filetype='a'):
     '''
-    Translates SELinux filetype single-letter representation
-    to a more human-readable version (which is also used in `semanage fcontext -l`).
+    .. versionadded:: 2017.7.0
+
+    Translates SELinux filetype single-letter representation to a more
+    human-readable version (which is also used in `semanage fcontext
+    -l`).
     '''
     _validate_filetype(filetype)
     return _SELINUX_FILETYPES.get(filetype, 'error')
@@ -414,20 +426,27 @@ def filetype_id_to_string(filetype='a'):
 
 def fcontext_get_policy(name, filetype=None, sel_type=None, sel_user=None, sel_level=None):
     '''
-    Returns the current entry in the SELinux policy list as a dictionary.
-    Returns None if no exact match was found
+    .. versionadded:: 2017.7.0
+
+    Returns the current entry in the SELinux policy list as a
+    dictionary. Returns None if no exact match was found.
+
     Returned keys are:
-    - filespec (the name supplied and matched)
-    - filetype (the descriptive name of the filetype supplied)
-    - sel_user, sel_role, sel_type, sel_level (the selinux context)
+
+    * filespec (the name supplied and matched)
+    * filetype (the descriptive name of the filetype supplied)
+    * sel_user, sel_role, sel_type, sel_level (the selinux context)
+
     For a more in-depth explanation of the selinux context, go to
     https://access.redhat.com/documentation/en-US/Red_Hat_Enterprise_Linux/6/html/Security-Enhanced_Linux/chap-Security-Enhanced_Linux-SELinux_Contexts.html
 
-    name: filespec of the file or directory. Regex syntax is allowed.
-    filetype: The SELinux filetype specification.
-              Use one of [a, f, d, c, b, s, l, p].
-              See also `man semanage-fcontext`.
-              Defaults to 'a' (all files)
+    name
+        filespec of the file or directory. Regex syntax is allowed.
+
+    filetype
+        The SELinux filetype specification. Use one of [a, f, d, c, b,
+        s, l, p]. See also `man semanage-fcontext`. Defaults to 'a'
+        (all files).
 
     CLI Example:
 
@@ -447,7 +466,7 @@ def fcontext_get_policy(name, filetype=None, sel_type=None, sel_user=None, sel_l
     cmd_kwargs['filetype'] = '[[:alpha:] ]+' if filetype is None else filetype_id_to_string(filetype)
     cmd = 'semanage fcontext -l | egrep ' + \
           "'^{filespec}{spacer}{filetype}{spacer}{sel_user}:{sel_role}:{sel_type}:{sel_level}$'".format(**cmd_kwargs)
-    current_entry_text = __salt__['cmd.shell'](cmd)
+    current_entry_text = __salt__['cmd.shell'](cmd, ignore_retcode=True)
     if current_entry_text == '':
         return None
     ret = {}
@@ -460,20 +479,34 @@ def fcontext_get_policy(name, filetype=None, sel_type=None, sel_user=None, sel_l
 
 def fcontext_add_or_delete_policy(action, name, filetype=None, sel_type=None, sel_user=None, sel_level=None):
     '''
-    Sets or deletes the SELinux policy for a given filespec and other optional parameters.
-    Returns the result of the call to semanage.
-    Note that you don't have to remove an entry before setting a new one for a given
-    filespec and filetype, as adding one with semanage automatically overwrites a
-    previously configured SELinux context.
+    .. versionadded:: 2017.7.0
 
-    name: filespec of the file or directory. Regex syntax is allowed.
-    file_type: The SELinux filetype specification.
-              Use one of [a, f, d, c, b, s, l, p].
-              See also ``man semanage-fcontext``.
-              Defaults to 'a' (all files)
-    sel_type: SELinux context type. There are many.
-    sel_user: SELinux user. Use ``semanage login -l`` to determine which ones are available to you
-    sel_level: The MLS range of the SELinux context.
+    Sets or deletes the SELinux policy for a given filespec and other
+    optional parameters.
+
+    Returns the result of the call to semanage.
+
+    Note that you don't have to remove an entry before setting a new
+    one for a given filespec and filetype, as adding one with semanage
+    automatically overwrites a previously configured SELinux context.
+
+    name
+        filespec of the file or directory. Regex syntax is allowed.
+
+    file_type
+        The SELinux filetype specification. Use one of [a, f, d, c, b,
+        s, l, p]. See also ``man semanage-fcontext``. Defaults to 'a'
+        (all files).
+
+    sel_type
+        SELinux context type. There are many.
+
+    sel_user
+        SELinux user. Use ``semanage login -l`` to determine which ones
+        are available to you.
+
+    sel_level
+        The MLS range of the SELinux context.
 
     CLI Example:
 
@@ -499,10 +532,14 @@ def fcontext_add_or_delete_policy(action, name, filetype=None, sel_type=None, se
 
 def fcontext_policy_is_applied(name, recursive=False):
     '''
-    Returns an empty string if the SELinux policy for a given filespec is applied,
-    returns string with differences in policy and actual situation otherwise.
+    .. versionadded:: 2017.7.0
 
-    name: filespec of the file or directory. Regex syntax is allowed.
+    Returns an empty string if the SELinux policy for a given filespec
+    is applied, returns string with differences in policy and actual
+    situation otherwise.
+
+    name
+        filespec of the file or directory. Regex syntax is allowed.
 
     CLI Example:
 
@@ -519,11 +556,17 @@ def fcontext_policy_is_applied(name, recursive=False):
 
 def fcontext_apply_policy(name, recursive=False):
     '''
-    Applies SElinux policies to filespec using `restorecon [-R] filespec`.
-    Returns dict with changes if succesful, the output of the restorecon command otherwise.
+    .. versionadded:: 2017.7.0
 
-    name: filespec of the file or directory. Regex syntax is allowed.
-    recursive: Recursively apply SELinux policies.
+    Applies SElinux policies to filespec using `restorecon [-R]
+    filespec`. Returns dict with changes if succesful, the output of
+    the restorecon command otherwise.
+
+    name
+        filespec of the file or directory. Regex syntax is allowed.
+
+    recursive
+        Recursively apply SELinux policies.
 
     CLI Example:
 
