@@ -2,11 +2,10 @@
 '''
 Module for running vmadm command on SmartOS
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import Python libs
 import logging
-import json
 import os
 try:
     from shlex import quote as _quote_args  # pylint: disable=E0611
@@ -14,11 +13,18 @@ except ImportError:
     from pipes import quote as _quote_args
 
 # Import Salt libs
-import salt.ext.six as six
-import salt.utils
+import salt.utils.args
 import salt.utils.decorators as decorators
+import salt.utils.files
+import salt.utils.json
+import salt.utils.path
+import salt.utils.platform
+import salt.utils.stringutils
 from salt.utils.odict import OrderedDict
 
+# Import 3rd-party libs
+from salt.ext import six
+
 log = logging.getLogger(__name__)
 
 # Function aliases
@@ -35,21 +41,21 @@ def _check_vmadm():
     '''
     Looks to see if vmadm is present on the system
     '''
-    return salt.utils.which('vmadm')
+    return salt.utils.path.which('vmadm')
 
 
 def _check_zfs():
     '''
     Looks to see if zfs is present on the system
     '''
-    return salt.utils.which('zfs')
+    return salt.utils.path.which('zfs')
 
 
 def __virtual__():
     '''
     Provides vmadm on SmartOS
     '''
-    if salt.utils.is_smartos_globalzone() and _check_vmadm():
+    if salt.utils.platform.is_smartos_globalzone() and _check_vmadm():
         return __virtualname__
     return (
         False,
@@ -91,7 +97,7 @@ def _create_update_from_file(mode='create', uuid=None, path=None):
         ret['Error'] = _exit_status(retcode)
         if 'stderr' in res:
             if res['stderr'][0] == '{':
-                ret['Error'] = json.loads(res['stderr'])
+                ret['Error'] = salt.utils.json.loads(res['stderr'])
             else:
                 ret['Error'] = res['stderr']
         return ret
@@ -108,7 +114,7 @@ def _create_update_from_file(mode='create', uuid=None, path=None):
         ret['Error'] = _exit_status(retcode)
         if 'stderr' in res:
             if res['stderr'][0] == '{':
-                ret['Error'] = json.loads(res['stderr'])
+                ret['Error'] = salt.utils.json.loads(res['stderr'])
             else:
                 ret['Error'] = res['stderr']
         return ret
@@ -127,8 +133,8 @@ def _create_update_from_cfg(mode='create', uuid=None, vmcfg=None):
 
     # write json file
     vmadm_json_file = __salt__['temp.file'](prefix='vmadm-')
-    with salt.utils.fopen(vmadm_json_file, 'w') as vmadm_json:
-        vmadm_json.write(json.dumps(vmcfg))
+    with salt.utils.files.fopen(vmadm_json_file, 'w') as vmadm_json:
+        salt.utils.json.dump(vmcfg, vmadm_json)
 
     # vmadm validate create|update [-f <filename>]
     cmd = '{vmadm} validate {mode} {brand} -f {vmadm_json_file}'.format(
@@ -143,7 +149,7 @@ def _create_update_from_cfg(mode='create', uuid=None, vmcfg=None):
         ret['Error'] = _exit_status(retcode)
         if 'stderr' in res:
             if res['stderr'][0] == '{':
-                ret['Error'] = json.loads(res['stderr'])
+                ret['Error'] = salt.utils.json.loads(res['stderr'])
             else:
                 ret['Error'] = res['stderr']
         return ret
@@ -160,13 +166,13 @@ def _create_update_from_cfg(mode='create', uuid=None, vmcfg=None):
         ret['Error'] = _exit_status(retcode)
         if 'stderr' in res:
             if res['stderr'][0] == '{':
-                ret['Error'] = json.loads(res['stderr'])
+                ret['Error'] = salt.utils.json.loads(res['stderr'])
             else:
                 ret['Error'] = res['stderr']
         return ret
     else:
         # cleanup json file (only when succesful to help troubleshooting)
-        salt.utils.safe_rm(vmadm_json_file)
+        salt.utils.files.safe_rm(vmadm_json_file)
 
         # return uuid
         if res['stderr'].startswith('Successfully created VM'):
@@ -399,7 +405,7 @@ def lookup(search=None, order=None, one=False):
     if one:
         result = res['stdout']
     else:
-        for vm in json.loads(res['stdout']):
+        for vm in salt.utils.json.loads(res['stdout']):
             result.append(vm)
 
     return result
@@ -520,7 +526,7 @@ def get(vm, key='uuid'):
     if retcode != 0:
         ret['Error'] = res['stderr'] if 'stderr' in res else _exit_status(retcode)
         return ret
-    return json.loads(res['stdout'])
+    return salt.utils.json.loads(res['stdout'])
 
 
 def info(vm, info_type='all', key='uuid'):
@@ -565,7 +571,7 @@ def info(vm, info_type='all', key='uuid'):
     if retcode != 0:
         ret['Error'] = res['stderr'] if 'stderr' in res else _exit_status(retcode)
         return ret
-    return json.loads(res['stdout'])
+    return salt.utils.json.loads(res['stdout'])
 
 
 def create_snapshot(vm, name, key='uuid'):
@@ -751,10 +757,10 @@ def reprovision(vm, image, key='uuid'):
         ret['Error'] = 'Image ({0}) is not present on this host'.format(image)
         return ret
     # vmadm reprovision <uuid> [-f <filename>]
-    cmd = 'echo {image} | {vmadm} reprovision {uuid}'.format(
-        vmadm=vmadm,
-        uuid=vm,
-        image=_quote_args(json.dumps({'image_uuid': image}))
+    cmd = six.text_type('echo {image} | {vmadm} reprovision {uuid}').format(
+        vmadm=salt.utils.stringutils.to_unicode(vmadm),
+        uuid=salt.utils.stringutils.to_unicode(vm),
+        image=_quote_args(salt.utils.json.dumps({'image_uuid': image}))
     )
     res = __salt__['cmd.run_all'](cmd, python_shell=True)
     retcode = res['retcode']
@@ -783,7 +789,7 @@ def create(from_file=None, **kwargs):
     ret = {}
     # prepare vmcfg
     vmcfg = {}
-    kwargs = salt.utils.clean_kwargs(**kwargs)
+    kwargs = salt.utils.args.clean_kwargs(**kwargs)
     for k, v in six.iteritems(kwargs):
         vmcfg[k] = v
 
@@ -818,7 +824,7 @@ def update(vm, from_file=None, key='uuid', **kwargs):
     vmadm = _check_vmadm()
     # prepare vmcfg
     vmcfg = {}
-    kwargs = salt.utils.clean_kwargs(**kwargs)
+    kwargs = salt.utils.args.clean_kwargs(**kwargs)
     for k, v in six.iteritems(kwargs):
         vmcfg[k] = v
 
