@@ -5,32 +5,51 @@ Salt interface to ZFS commands
 :codeauthor: Nitin Madhok <nmadhok@clemson.edu>
 
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import Python libs
+import re
+import math
 import logging
 
 # Import Salt libs
-import salt.utils
+import salt.utils.args
+import salt.utils.path
 import salt.modules.cmdmod
 import salt.utils.decorators as decorators
 from salt.utils.odict import OrderedDict
+from salt.utils.stringutils import to_num as str_to_num
+from salt.ext import six
 
+__virtualname__ = 'zfs'
 log = logging.getLogger(__name__)
 
+# Precompiled regex
+re_zfs_size = re.compile(r'^(\d+|\d+(?=\d*)\.\d+)([KkMmGgTtPpEeZz][Bb]?)$')
+
 # Function alias to set mapping.
 __func_alias__ = {
     'list_': 'list',
 }
 
 
+def __virtual__():
+    '''
+    Only load when the platform has zfs support
+    '''
+    if __grains__['zfs_support']:
+        return __virtualname__
+    else:
+        return (False, "The zfs module cannot be loaded: zfs not supported")
+
+
 @decorators.memoize
 def _check_zfs():
     '''
     Looks to see if zfs is present on the system.
     '''
     # Get the path to the zfs binary.
-    return salt.utils.which('zfs')
+    return salt.utils.path.which('zfs')
 
 
 @decorators.memoize
@@ -39,7 +58,7 @@ def _check_features():
     Looks to see if zpool-features is available
     '''
     # get man location
-    man = salt.utils.which('man')
+    man = salt.utils.path.which('man')
     if not man:
         return False
 
@@ -50,32 +69,36 @@ def _check_features():
     return res['retcode'] == 0
 
 
-def __virtual__():
+def _conform_value(value, convert_size=False):
     '''
-    Makes sure that ZFS kernel module is loaded.
+    Ensure value always conform to what zfs expects
     '''
-    on_freebsd = __grains__['kernel'] == 'FreeBSD'
-    on_linux = __grains__['kernel'] == 'Linux'
-    on_solaris = __grains__['kernel'] == 'SunOS' and __grains__['kernelrelease'] == '5.11'
+    # NOTE: salt breaks the on/off/yes/no properties
+    if isinstance(value, bool):
+        return 'on' if value else 'off'
 
-    cmd = ''
-    if on_freebsd:
-        cmd = 'kldstat -q -m zfs'
-    elif on_linux:
-        modinfo = salt.utils.which('modinfo')
-        if modinfo:
-            cmd = '{0} zfs'.format(modinfo)
-        else:
-            cmd = 'ls /sys/module/zfs'
-    elif on_solaris:
-        # not using salt.utils.which('zfs') to keep compatible with others
-        cmd = 'which zfs'
+    if isinstance(value, six.text_type) or isinstance(value, str):
+        # NOTE: handle whitespaces
+        if ' ' in value:
+            # NOTE: quoting the string may be better
+            #       but it is hard to know if we already quoted it before
+            #       this can be improved in the future
+            return "'{0}'".format(value.strip("'"))
 
-    if cmd and salt.modules.cmdmod.retcode(
-        cmd, output_loglevel='quiet', ignore_retcode=True
-    ) == 0:
-        return 'zfs'
-    return (False, "The zfs module cannot be loaded: zfs not found")
+        # NOTE: handle ZFS size conversion
+        match_size = re_zfs_size.match(value)
+        if convert_size and match_size:
+            v_size = float(match_size.group(1))
+            v_unit = match_size.group(2).upper()[0]
+            v_power = math.pow(1024, ['K', 'M', 'G', 'T', 'P', 'E', 'Z'].index(v_unit) + 1)
+            value = v_size * v_power
+            return int(value) if int(value) == value else value
+
+        # NOTE: convert to numeric if needed
+        return str_to_num(value)
+
+    # NOTE: passthrough
+    return value
 
 
 def exists(name, **kwargs):
@@ -161,14 +184,10 @@ def create(name, **kwargs):
     # if zpool properties specified, then
     # create "-o property=value" pairs
     if properties:
-        optlist = []
+        proplist = []
         for prop in properties:
-            if isinstance(properties[prop], bool):  # salt breaks the on/off/yes/no properties :(
-                properties[prop] = 'on' if properties[prop] else 'off'
-
-            optlist.append('-o {0}={1}'.format(prop, properties[prop]))
-        opts = ' '.join(optlist)
-        cmd = '{0} {1}'.format(cmd, opts)
+            proplist.append('-o {0}={1}'.format(prop, _conform_value(properties[prop])))
+        cmd = '{0} {1}'.format(cmd, ' '.join(proplist))
 
     if volume_size:
         cmd = '{0} -V {1}'.format(cmd, volume_size)
@@ -308,7 +327,7 @@ def rename(name, new_name, **kwargs):
 def list_(name=None, **kwargs):
     '''
     .. versionadded:: 2015.5.0
-    .. versionchanged:: 2016.3.0
+    .. versionchanged:: 2018.3.0
 
     Return a list of all datasets or a specified dataset on the system and the
     values of their used, available, referenced, and mountpoint properties.
@@ -328,6 +347,9 @@ def list_(name=None, **kwargs):
         property to sort on (default = name)
     order : string [ascending|descending]
         sort order (default = ascending)
+    parsable : boolean
+        display numbers in parsable (exact) values
+        .. versionadded:: 2018.3.0
 
     CLI Example:
 
@@ -345,8 +367,13 @@ def list_(name=None, **kwargs):
     sort = kwargs.get('sort', None)
     ltype = kwargs.get('type', None)
     order = kwargs.get('order', 'ascending')
+    parsable = kwargs.get('parsable', False)
     cmd = '{0} list -H'.format(zfs)
 
+    # parsable output
+    if parsable:
+        cmd = '{0} -p'.format(cmd)
+
     # filter on type
     if ltype:
         cmd = '{0} -t {1}'.format(cmd, ltype)
@@ -383,7 +410,7 @@ def list_(name=None, **kwargs):
             ds_data = {}
 
             for prop in properties:
-                ds_data[prop] = ds[properties.index(prop)]
+                ds_data[prop] = _conform_value(ds[properties.index(prop)])
 
             ret[ds_data['name']] = ds_data
             del ret[ds_data['name']]['name']
@@ -684,12 +711,10 @@ def clone(name_a, name_b, **kwargs):
     # if zpool properties specified, then
     # create "-o property=value" pairs
     if properties:
-        optlist = []
+        proplist = []
         for prop in properties:
-            if isinstance(properties[prop], bool):  # salt breaks the on/off/yes/no properties :(
-                properties[prop] = 'on' if properties[prop] else 'off'
-            optlist.append('-o {0}={1}'.format(prop, properties[prop]))
-        properties = ' '.join(optlist)
+            proplist.append('-o {0}={1}'.format(prop, properties[prop]))
+        properties = ' '.join(proplist)
 
     res = __salt__['cmd.run_all']('{zfs} clone {create_parent}{properties}{name_a} {name_b}'.format(
         zfs=zfs,
@@ -1062,12 +1087,10 @@ def snapshot(*snapshot, **kwargs):
     # if zpool properties specified, then
     # create "-o property=value" pairs
     if properties:
-        optlist = []
+        proplist = []
         for prop in properties:
-            if isinstance(properties[prop], bool):  # salt breaks the on/off/yes/no properties :(
-                properties[prop] = 'on' if properties[prop] else 'off'
-            optlist.append('-o {0}={1}'.format(prop, properties[prop]))
-        properties = ' '.join(optlist)
+            proplist.append('-o {0}={1}'.format(prop, _conform_value((properties[prop]))))
+        properties = ' '.join(proplist)
 
     for csnap in snapshot:
         if '@' not in csnap:
@@ -1142,7 +1165,7 @@ def set(*dataset, **kwargs):
         ret['error'] = 'one or more snapshots must be specified'
 
     # clean kwargs
-    properties = salt.utils.clean_kwargs(**kwargs)
+    properties = salt.utils.args.clean_kwargs(**kwargs)
     if len(properties) < 1:
         ret['error'] = '{0}one or more properties must be specified'.format(
             '{0},\n'.format(ret['error']) if 'error' in ret else ''
@@ -1154,14 +1177,10 @@ def set(*dataset, **kwargs):
     # for better error handling we don't do one big set command
     for ds in dataset:
         for prop in properties:
-
-            if isinstance(properties[prop], bool):  # salt breaks the on/off/yes/no properties :(
-                properties[prop] = 'on' if properties[prop] else 'off'
-
             res = __salt__['cmd.run_all']('{zfs} set {prop}={value} {dataset}'.format(
                 zfs=zfs,
                 prop=prop,
-                value=properties[prop],
+                value=_conform_value(properties[prop]),
                 dataset=ds
             ))
             if ds not in ret:
@@ -1180,6 +1199,7 @@ def set(*dataset, **kwargs):
 def get(*dataset, **kwargs):
     '''
     .. versionadded:: 2016.3.0
+    .. versionchanged:: 2018.3.0
 
     Displays properties for the given datasets.
 
@@ -1199,6 +1219,9 @@ def get(*dataset, **kwargs):
     source : string
         comma-separated list of sources to display. Must be one of the following:
         local, default, inherited, temporary, and none. The default value is all sources.
+    parsable : boolean
+        display numbers in parsable (exact) values
+        .. versionadded:: 2018.3.0
 
     .. note::
 
@@ -1222,8 +1245,13 @@ def get(*dataset, **kwargs):
     fields = kwargs.get('fields', 'value,source')
     ltype = kwargs.get('type', None)
     source = kwargs.get('source', None)
+    parsable = kwargs.get('parsable', False)
     cmd = '{0} get -H'.format(zfs)
 
+    # parsable output
+    if parsable:
+        cmd = '{0} -p'.format(cmd)
+
     # recursively get
     if depth:
         cmd = '{0} -d {1}'.format(cmd, depth)
@@ -1262,7 +1290,7 @@ def get(*dataset, **kwargs):
             ds_data = {}
 
             for field in fields:
-                ds_data[field] = ds[fields.index(field)]
+                ds_data[field] = _conform_value(ds[fields.index(field)])
 
             ds_name = ds_data['name']
             ds_prop = ds_data['property']
