@@ -4,14 +4,16 @@ Render the pillar data
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import copy
+import fnmatch
 import os
 import collections
 import logging
 import tornado.gen
 import sys
 import traceback
+import inspect
 
 # Import salt libs
 import salt.loader
@@ -19,23 +21,29 @@ import salt.fileclient
 import salt.minion
 import salt.crypt
 import salt.transport
-import salt.utils.url
+import salt.utils.args
 import salt.utils.cache
 import salt.utils.crypt
+import salt.utils.data
+import salt.utils.dictupdate
+import salt.utils.url
 from salt.exceptions import SaltClientError
 from salt.template import compile_template
-from salt.utils.dictupdate import merge
 from salt.utils.odict import OrderedDict
 from salt.version import __version__
+# Even though dictupdate is imported, invoking salt.utils.dictupdate.merge here
+# causes an UnboundLocalError. This should be investigated and fixed, but until
+# then, leave the import directly below this comment intact.
+from salt.utils.dictupdate import merge
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
 
 def get_pillar(opts, grains, minion_id, saltenv=None, ext=None, funcs=None,
-               pillar_override=None, pillarenv=None):
+               pillar_override=None, pillarenv=None, extra_minion_data=None):
     '''
     Return the correct pillar driver based on the file_client option
     '''
@@ -50,16 +58,18 @@ def get_pillar(opts, grains, minion_id, saltenv=None, ext=None, funcs=None,
     log.debug('Determining pillar cache')
     if opts['pillar_cache']:
         log.info('Compiling pillar from cache')
-        log.debug('get_pillar using pillar cache with ext: {0}'.format(ext))
+        log.debug('get_pillar using pillar cache with ext: %s', ext)
         return PillarCache(opts, grains, minion_id, saltenv, ext=ext, functions=funcs,
                 pillar_override=pillar_override, pillarenv=pillarenv)
     return ptype(opts, grains, minion_id, saltenv, ext, functions=funcs,
-                 pillar_override=pillar_override, pillarenv=pillarenv)
+                 pillar_override=pillar_override, pillarenv=pillarenv,
+                 extra_minion_data=extra_minion_data)
 
 
 # TODO: migrate everyone to this one!
 def get_async_pillar(opts, grains, minion_id, saltenv=None, ext=None, funcs=None,
-               pillar_override=None, pillarenv=None):
+                     pillar_override=None, pillarenv=None,
+                     extra_minion_data=None):
     '''
     Return the correct pillar driver based on the file_client option
     '''
@@ -71,17 +81,64 @@ def get_async_pillar(opts, grains, minion_id, saltenv=None, ext=None, funcs=None
         'local': AsyncPillar,
     }.get(file_client, AsyncPillar)
     return ptype(opts, grains, minion_id, saltenv, ext, functions=funcs,
-                 pillar_override=pillar_override, pillarenv=pillarenv)
+                 pillar_override=pillar_override, pillarenv=pillarenv,
+                 extra_minion_data=extra_minion_data)
 
 
-class AsyncRemotePillar(object):
+class RemotePillarMixin(object):
+    '''
+    Common remote pillar functionality
+    '''
+    def get_ext_pillar_extra_minion_data(self, opts):
+        '''
+        Returns the extra data from the minion's opts dict (the config file).
+
+        This data will be passed to external pillar functions.
+        '''
+        def get_subconfig(opts_key):
+            '''
+            Returns a dict containing the opts key subtree, while maintaining
+            the opts structure
+            '''
+            ret_dict = aux_dict = {}
+            config_val = opts
+            subkeys = opts_key.split(':')
+            # Build an empty dict with the opts path
+            for subkey in subkeys[:-1]:
+                aux_dict[subkey] = {}
+                aux_dict = aux_dict[subkey]
+                if not config_val.get(subkey):
+                    # The subkey is not in the config
+                    return {}
+                config_val = config_val[subkey]
+            if subkeys[-1] not in config_val:
+                return {}
+            aux_dict[subkeys[-1]] = config_val[subkeys[-1]]
+            return ret_dict
+
+        extra_data = {}
+        if 'pass_to_ext_pillars' in opts:
+            if not isinstance(opts['pass_to_ext_pillars'], list):
+                log.exception('\'pass_to_ext_pillars\' config is malformed.')
+                raise SaltClientError('\'pass_to_ext_pillars\' config is '
+                                      'malformed.')
+            for key in opts['pass_to_ext_pillars']:
+                salt.utils.dictupdate.update(extra_data,
+                                             get_subconfig(key),
+                                             recursive_update=True,
+                                             merge_lists=True)
+        log.trace('ext_pillar_extra_data = %s', extra_data)
+        return extra_data
+
+
+class AsyncRemotePillar(RemotePillarMixin):
     '''
     Get the pillar from the master
     '''
     def __init__(self, opts, grains, minion_id, saltenv, ext=None, functions=None,
-                 pillar_override=None, pillarenv=None):
+                 pillar_override=None, pillarenv=None, extra_minion_data=None):
         self.opts = opts
-        self.opts['environment'] = saltenv
+        self.opts['saltenv'] = saltenv
         self.ext = ext
         self.grains = grains
         self.minion_id = minion_id
@@ -92,6 +149,14 @@ class AsyncRemotePillar(object):
         if not isinstance(self.pillar_override, dict):
             self.pillar_override = {}
             log.error('Pillar data must be a dictionary')
+        self.extra_minion_data = extra_minion_data or {}
+        if not isinstance(self.extra_minion_data, dict):
+            self.extra_minion_data = {}
+            log.error('Extra minion data must be a dictionary')
+        salt.utils.dictupdate.update(self.extra_minion_data,
+                                     self.get_ext_pillar_extra_minion_data(opts),
+                                     recursive_update=True,
+                                     merge_lists=True)
 
     @tornado.gen.coroutine
     def compile_pillar(self):
@@ -100,9 +165,10 @@ class AsyncRemotePillar(object):
         '''
         load = {'id': self.minion_id,
                 'grains': self.grains,
-                'saltenv': self.opts['environment'],
+                'saltenv': self.opts['saltenv'],
                 'pillarenv': self.opts['pillarenv'],
                 'pillar_override': self.pillar_override,
+                'extra_minion_data': self.extra_minion_data,
                 'ver': '2',
                 'cmd': '_pillar'}
         if self.ext:
@@ -125,14 +191,14 @@ class AsyncRemotePillar(object):
         raise tornado.gen.Return(ret_pillar)
 
 
-class RemotePillar(object):
+class RemotePillar(RemotePillarMixin):
     '''
     Get the pillar from the master
     '''
     def __init__(self, opts, grains, minion_id, saltenv, ext=None, functions=None,
-                 pillar_override=None, pillarenv=None):
+                 pillar_override=None, pillarenv=None, extra_minion_data=None):
         self.opts = opts
-        self.opts['environment'] = saltenv
+        self.opts['saltenv'] = saltenv
         self.ext = ext
         self.grains = grains
         self.minion_id = minion_id
@@ -143,6 +209,14 @@ class RemotePillar(object):
         if not isinstance(self.pillar_override, dict):
             self.pillar_override = {}
             log.error('Pillar data must be a dictionary')
+        self.extra_minion_data = extra_minion_data or {}
+        if not isinstance(self.extra_minion_data, dict):
+            self.extra_minion_data = {}
+            log.error('Extra minion data must be a dictionary')
+        salt.utils.dictupdate.update(self.extra_minion_data,
+                                     self.get_ext_pillar_extra_minion_data(opts),
+                                     recursive_update=True,
+                                     merge_lists=True)
 
     def compile_pillar(self):
         '''
@@ -150,9 +224,10 @@ class RemotePillar(object):
         '''
         load = {'id': self.minion_id,
                 'grains': self.grains,
-                'saltenv': self.opts['environment'],
+                'saltenv': self.opts['saltenv'],
                 'pillarenv': self.opts['pillarenv'],
                 'pillar_override': self.pillar_override,
+                'extra_minion_data': self.extra_minion_data,
                 'ver': '2',
                 'cmd': '_pillar'}
         if self.ext:
@@ -163,8 +238,8 @@ class RemotePillar(object):
 
         if not isinstance(ret_pillar, dict):
             log.error(
-                'Got a bad pillar from master, type {0}, expecting dict: '
-                '{1}'.format(type(ret_pillar).__name__, ret_pillar)
+                'Got a bad pillar from master, type %s, expecting dict: %s',
+                type(ret_pillar).__name__, ret_pillar
             )
             return {}
         return ret_pillar
@@ -186,7 +261,7 @@ class PillarCache(object):
     '''
     # TODO ABC?
     def __init__(self, opts, grains, minion_id, saltenv, ext=None, functions=None,
-            pillar_override=None, pillarenv=None):
+                 pillar_override=None, pillarenv=None, extra_minion_data=None):
         # Yes, we need all of these because we need to route to the Pillar object
         # if we have no cache. This is another refactor target.
 
@@ -223,7 +298,7 @@ class PillarCache(object):
         In the event of a cache miss, we need to incur the overhead of caching
         a new pillar.
         '''
-        log.debug('Pillar cache getting external pillar with ext: {0}'.format(self.ext))
+        log.debug('Pillar cache getting external pillar with ext: %s', self.ext)
         fresh_pillar = Pillar(self.opts,
                               self.grains,
                               self.minion_id,
@@ -232,30 +307,30 @@ class PillarCache(object):
                               functions=self.functions,
                               pillar_override=self.pillar_override,
                               pillarenv=self.pillarenv)
-        return fresh_pillar.compile_pillar()  # FIXME We are not yet passing pillar_dirs in here
+        return fresh_pillar.compile_pillar()
 
     def compile_pillar(self, *args, **kwargs):  # Will likely just be pillar_dirs
-        log.debug('Scanning pillar cache for information about minion {0} and saltenv {1}'.format(self.minion_id, self.saltenv))
-        log.debug('Scanning cache: {0}'.format(self.cache._dict))
+        log.debug('Scanning pillar cache for information about minion %s and pillarenv %s', self.minion_id, self.pillarenv)
+        log.debug('Scanning cache: %s', self.cache._dict)
         # Check the cache!
         if self.minion_id in self.cache:  # Keyed by minion_id
             # TODO Compare grains, etc?
-            if self.saltenv in self.cache[self.minion_id]:
+            if self.pillarenv in self.cache[self.minion_id]:
                 # We have a cache hit! Send it back.
-                log.debug('Pillar cache hit for minion {0} and saltenv {1}'.format(self.minion_id, self.saltenv))
-                return self.cache[self.minion_id][self.saltenv]
+                log.debug('Pillar cache hit for minion %s and pillarenv %s', self.minion_id, self.pillarenv)
+                return self.cache[self.minion_id][self.pillarenv]
             else:
                 # We found the minion but not the env. Store it.
                 fresh_pillar = self.fetch_pillar()
-                self.cache[self.minion_id][self.saltenv] = fresh_pillar
-                log.debug('Pillar cache miss for saltenv {0} for minion {1}'.format(self.saltenv, self.minion_id))
+                self.cache[self.minion_id][self.pillarenv] = fresh_pillar
+                log.debug('Pillar cache miss for pillarenv %s for minion %s', self.pillarenv, self.minion_id)
                 return fresh_pillar
         else:
             # We haven't seen this minion yet in the cache. Store it.
             fresh_pillar = self.fetch_pillar()
-            self.cache[self.minion_id] = {self.saltenv: fresh_pillar}
-            log.debug('Pillar cache miss for minion {0}'.format(self.minion_id))
-            log.debug('Current pillar cache: {0}'.format(self.cache._dict))  # FIXME hack!
+            self.cache[self.minion_id] = {self.pillarenv: fresh_pillar}
+            log.debug('Pillar cache miss for minion %s', self.minion_id)
+            log.debug('Current pillar cache: %s', self.cache._dict)  # FIXME hack!
             return fresh_pillar
 
 
@@ -264,7 +339,7 @@ class Pillar(object):
     Read over the pillar top files and render the pillar data
     '''
     def __init__(self, opts, grains, minion_id, saltenv, ext=None, functions=None,
-                 pillar_override=None, pillarenv=None):
+                 pillar_override=None, pillarenv=None, extra_minion_data=None):
         self.minion_id = minion_id
         self.ext = ext
         if pillarenv is None:
@@ -276,6 +351,7 @@ class Pillar(object):
         self.opts = self.__gen_opts(opts, grains, saltenv=saltenv, pillarenv=pillarenv)
         self.saltenv = saltenv
         self.client = salt.fileclient.get_file_client(self.opts, True)
+        self.avail = self.__gather_avail()
 
         if opts.get('file_client', '') == 'local':
             opts['grains'] = grains
@@ -309,6 +385,10 @@ class Pillar(object):
         if not isinstance(self.pillar_override, dict):
             self.pillar_override = {}
             log.error('Pillar data must be a dictionary')
+        self.extra_minion_data = extra_minion_data or {}
+        if not isinstance(self.extra_minion_data, dict):
+            self.extra_minion_data = {}
+            log.error('Extra minion data must be a dictionary')
 
     def __valid_on_demand_ext_pillar(self, opts):
         '''
@@ -344,6 +424,15 @@ class Pillar(object):
             return False
         return True
 
+    def __gather_avail(self):
+        '''
+        Gather the lists of available sls data from the master
+        '''
+        avail = {}
+        for saltenv in self._get_envs():
+            avail[saltenv] = self.client.list_states(saltenv)
+        return avail
+
     def __gen_opts(self, opts_in, grains, saltenv=None, ext=None, pillarenv=None):
         '''
         The options need to be altered to conform to the file client
@@ -356,9 +445,9 @@ class Pillar(object):
         else:
             opts['grains'] = grains
         # Allow minion/CLI saltenv/pillarenv to take precedence over master
-        opts['environment'] = saltenv \
+        opts['saltenv'] = saltenv \
             if saltenv is not None \
-            else opts.get('environment')
+            else opts.get('saltenv')
         opts['pillarenv'] = pillarenv \
             if pillarenv is not None \
             else opts.get('pillarenv')
@@ -445,8 +534,7 @@ class Pillar(object):
             errors.append(
                     ('Rendering Primary Top file failed, render error:\n{0}'
                         .format(exc)))
-            log.error('Pillar rendering failed for minion {0}: '.format(self.minion_id),
-                    exc_info=True)
+            log.exception('Pillar rendering failed for minion %s', self.minion_id)
 
         # Search initial top files for includes
         for saltenv, ctops in six.iteritems(tops):
@@ -555,7 +643,7 @@ class Pillar(object):
             merged_tops = self.merge_tops(tops)
         except TypeError as err:
             merged_tops = OrderedDict()
-            errors.append('Error encountered while render pillar top file.')
+            errors.append('Error encountered while rendering pillar top file.')
         return merged_tops, errors
 
     def top_matches(self, top):
@@ -597,8 +685,8 @@ class Pillar(object):
         fn_ = self.client.get_state(sls, saltenv).get('dest', False)
         if not fn_:
             if sls in self.ignored_pillars.get(saltenv, []):
-                log.debug('Skipping ignored and missing SLS \'{0}\' in'
-                          ' environment \'{1}\''.format(sls, saltenv))
+                log.debug('Skipping ignored and missing SLS \'%s\' in '
+                          'environment \'%s\'', sls, saltenv)
                 return None, mods, errors
             elif self.opts['pillar_roots'].get(saltenv):
                 msg = ('Specified SLS \'{0}\' in environment \'{1}\' is not'
@@ -664,6 +752,8 @@ class Pillar(object):
                         log.error(msg)
                         errors.append(msg)
                     else:
+                        # render included state(s)
+                        include_states = []
                         for sub_sls in state.pop('include'):
                             if isinstance(sub_sls, dict):
                                 sub_sls, v = next(six.iteritems(sub_sls))
@@ -685,16 +775,23 @@ class Pillar(object):
                                             nstate = {
                                                 key_fragment: nstate
                                             }
-
+                                    include_states.append(nstate)
+                                if err:
+                                    errors += err
+                        if include_states:
+                            # merge included state(s) with the current state merged last
+                            include_states.append(state)
+                            state = None
+                            for s in include_states:
+                                if state is None:
+                                    state = s
+                                else:
                                     state = merge(
                                         state,
-                                        nstate,
+                                        s,
                                         self.merge_strategy,
                                         self.opts.get('renderer', 'yaml'),
                                         self.opts.get('pillar_merge_lists', False))
-
-                                if err:
-                                    errors += err
         return state, mods, errors
 
     def render_pillar(self, matches, errors=None):
@@ -706,8 +803,23 @@ class Pillar(object):
         if errors is None:
             errors = []
         for saltenv, pstates in six.iteritems(matches):
+            pstatefiles = []
             mods = set()
-            for sls in pstates:
+            for sls_match in pstates:
+                matched_pstates = []
+                try:
+                    matched_pstates = fnmatch.filter(self.avail[saltenv], sls_match)
+                except KeyError:
+                    errors.extend(
+                        ['No matching pillar environment for environment '
+                         '\'{0}\' found'.format(saltenv)]
+                    )
+                if matched_pstates:
+                    pstatefiles.extend(matched_pstates)
+                else:
+                    pstatefiles.append(sls_match)
+
+            for sls in pstatefiles:
                 pstate, mods, err = self.render_pstate(sls, saltenv, mods)
 
                 if err:
@@ -716,15 +828,12 @@ class Pillar(object):
                 if pstate is not None:
                     if not isinstance(pstate, dict):
                         log.error(
-                            'The rendered pillar sls file, \'{0}\' state did '
+                            'The rendered pillar sls file, \'%s\' state did '
                             'not return the expected data format. This is '
                             'a sign of a malformed pillar sls file. Returned '
-                            'errors: {1}'.format(
-                                sls,
-                                ', '.join(
-                                    ['\'{0}\''.format(e) for e in errors]
-                                )
-                            )
+                            'errors: %s',
+                            sls,
+                            ', '.join(["'{0}'".format(e) for e in errors])
                         )
                         continue
                     pillar = merge(
@@ -736,35 +845,43 @@ class Pillar(object):
 
         return pillar, errors
 
-    def _external_pillar_data(self, pillar, val, pillar_dirs, key):
+    def _external_pillar_data(self, pillar, val, key):
         '''
         Builds actual pillar data structure and updates the ``pillar`` variable
         '''
         ext = None
+        args = salt.utils.args.get_function_argspec(self.ext_pillars[key]).args
 
         if isinstance(val, dict):
-            ext = self.ext_pillars[key](self.minion_id, pillar, **val)
+            if ('extra_minion_data' in args) and self.extra_minion_data:
+                ext = self.ext_pillars[key](
+                    self.minion_id, pillar,
+                    extra_minion_data=self.extra_minion_data, **val)
+            else:
+                ext = self.ext_pillars[key](self.minion_id, pillar, **val)
         elif isinstance(val, list):
-            if key == 'git':
-                ext = self.ext_pillars[key](self.minion_id,
-                                            val,
-                                            pillar_dirs)
+            if ('extra_minion_data' in args) and self.extra_minion_data:
+                ext = self.ext_pillars[key](
+                    self.minion_id, pillar, *val,
+                    extra_minion_data=self.extra_minion_data)
             else:
                 ext = self.ext_pillars[key](self.minion_id,
                                             pillar,
                                             *val)
         else:
-            if key == 'git':
-                ext = self.ext_pillars[key](self.minion_id,
-                                            val,
-                                            pillar_dirs)
+            if ('extra_minion_data' in args) and self.extra_minion_data:
+                ext = self.ext_pillars[key](
+                    self.minion_id,
+                    pillar,
+                    val,
+                    extra_minion_data=self.extra_minion_data)
             else:
                 ext = self.ext_pillars[key](self.minion_id,
                                             pillar,
                                             val)
         return ext
 
-    def ext_pillar(self, pillar, pillar_dirs, errors=None):
+    def ext_pillar(self, pillar, errors=None):
         '''
         Render the external pillar data
         '''
@@ -778,9 +895,12 @@ class Pillar(object):
                     and self.opts.get('__role') != 'minion':
                 # Avoid circular import
                 import salt.utils.gitfs
-                from salt.pillar.git_pillar import PER_REMOTE_OVERRIDES
-                git_pillar = salt.utils.gitfs.GitPillar(self.opts)
-                git_pillar.init_remotes(self.ext['git'], PER_REMOTE_OVERRIDES)
+                import salt.pillar.git_pillar
+                git_pillar = salt.utils.gitfs.GitPillar(
+                    self.opts,
+                    self.ext['git'],
+                    per_remote_overrides=salt.pillar.git_pillar.PER_REMOTE_OVERRIDES,
+                    per_remote_only=salt.pillar.git_pillar.PER_REMOTE_ONLY)
                 git_pillar.fetch_remotes()
         except TypeError:
             # Handle malformed ext_pillar
@@ -794,11 +914,12 @@ class Pillar(object):
         ext = None
         # Bring in CLI pillar data
         if self.pillar_override:
-            pillar = merge(pillar,
-                           self.pillar_override,
-                           self.merge_strategy,
-                           self.opts.get('renderer', 'yaml'),
-                           self.opts.get('pillar_merge_lists', False))
+            pillar = merge(
+                pillar,
+                self.pillar_override,
+                self.merge_strategy,
+                self.opts.get('renderer', 'yaml'),
+                self.opts.get('pillar_merge_lists', False))
 
         for run in self.opts['ext_pillar']:
             if not isinstance(run, dict):
@@ -810,15 +931,14 @@ class Pillar(object):
             for key, val in six.iteritems(run):
                 if key not in self.ext_pillars:
                     log.critical(
-                        'Specified ext_pillar interface {0} is '
-                        'unavailable'.format(key)
+                        'Specified ext_pillar interface %s is unavailable',
+                        key
                     )
                     continue
                 try:
                     ext = self._external_pillar_data(pillar,
-                                                        val,
-                                                        pillar_dirs,
-                                                        key)
+                                                     val,
+                                                     key)
                 except Exception as exc:
                     errors.append(
                         'Failed to load ext_pillar {0}: {1}'.format(
@@ -840,29 +960,27 @@ class Pillar(object):
                 ext = None
         return pillar, errors
 
-    def compile_pillar(self, ext=True, pillar_dirs=None):
+    def compile_pillar(self, ext=True):
         '''
         Render the pillar data and return
         '''
         top, top_errors = self.get_top()
         if ext:
             if self.opts.get('ext_pillar_first', False):
-                self.opts['pillar'], errors = self.ext_pillar(
-                    self.pillar_override,
-                    pillar_dirs)
+                self.opts['pillar'], errors = self.ext_pillar(self.pillar_override)
                 self.rend = salt.loader.render(self.opts, self.functions)
                 matches = self.top_matches(top)
                 pillar, errors = self.render_pillar(matches, errors=errors)
-                pillar = merge(self.opts['pillar'],
-                               pillar,
-                               self.merge_strategy,
-                               self.opts.get('renderer', 'yaml'),
-                               self.opts.get('pillar_merge_lists', False))
+                pillar = merge(
+                    self.opts['pillar'],
+                    pillar,
+                    self.merge_strategy,
+                    self.opts.get('renderer', 'yaml'),
+                    self.opts.get('pillar_merge_lists', False))
             else:
                 matches = self.top_matches(top)
                 pillar, errors = self.render_pillar(matches)
-                pillar, errors = self.ext_pillar(
-                    pillar, pillar_dirs, errors=errors)
+                pillar, errors = self.ext_pillar(pillar, errors=errors)
         else:
             matches = self.top_matches(top)
             pillar, errors = self.render_pillar(matches)
@@ -877,15 +995,16 @@ class Pillar(object):
             pillar['master'] = mopts
         if errors:
             for error in errors:
-                log.critical('Pillar render error: {0}'.format(error))
+                log.critical('Pillar render error: %s', error)
             pillar['_errors'] = errors
 
         if self.pillar_override:
-            pillar = merge(pillar,
-                           self.pillar_override,
-                           self.merge_strategy,
-                           self.opts.get('renderer', 'yaml'),
-                           self.opts.get('pillar_merge_lists', False))
+            pillar = merge(
+                pillar,
+                self.pillar_override,
+                self.merge_strategy,
+                self.opts.get('renderer', 'yaml'),
+                self.opts.get('pillar_merge_lists', False))
 
         decrypt_errors = self.decrypt_pillar(pillar)
         if decrypt_errors:
@@ -901,11 +1020,11 @@ class Pillar(object):
             decrypt_pillar = self.opts['decrypt_pillar']
             if not isinstance(decrypt_pillar, dict):
                 decrypt_pillar = \
-                    salt.utils.repack_dictlist(self.opts['decrypt_pillar'])
+                    salt.utils.data.repack_dictlist(self.opts['decrypt_pillar'])
             if not decrypt_pillar:
                 errors.append('decrypt_pillar config option is malformed')
             for key, rend in six.iteritems(decrypt_pillar):
-                ptr = salt.utils.traverse_dict(
+                ptr = salt.utils.data.traverse_dict(
                     pillar,
                     key,
                     default=None,
@@ -937,7 +1056,7 @@ class Pillar(object):
                             # parent is the pillar dict itself.
                             ptr = pillar
                         else:
-                            ptr = salt.utils.traverse_dict(
+                            ptr = salt.utils.data.traverse_dict(
                                 pillar,
                                 parent,
                                 default=None,
@@ -957,6 +1076,6 @@ class Pillar(object):
 # ext_pillar etc.
 class AsyncPillar(Pillar):
     @tornado.gen.coroutine
-    def compile_pillar(self, ext=True, pillar_dirs=None):
-        ret = super(AsyncPillar, self).compile_pillar(ext=ext, pillar_dirs=pillar_dirs)
+    def compile_pillar(self, ext=True):
+        ret = super(AsyncPillar, self).compile_pillar(ext=ext)
         raise tornado.gen.Return(ret)
