@@ -375,20 +375,20 @@ You can also select a custom merging strategy using a ``__`` object in a list:
 '''
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
+import functools
+import glob
 import os
 import posixpath
 import logging
-from functools import partial
-from glob import glob
 
-import yaml
 from jinja2 import FileSystemLoader, Environment
 
 # Import Salt libs
-import salt.ext.six as six
-import salt.utils
-
+from salt.ext import six
+import salt.utils.data
+import salt.utils.jinja
+import salt.utils.yaml
 
 log = logging.getLogger(__name__)
 strategies = ('overwrite', 'merge-first', 'merge-last', 'remove')
@@ -398,9 +398,9 @@ def ext_pillar(minion_id, pillar, *args, **kwargs):
     stack = {}
     stack_config_files = list(args)
     traverse = {
-        'pillar': partial(salt.utils.traverse_dict_and_list, pillar),
-        'grains': partial(salt.utils.traverse_dict_and_list, __grains__),
-        'opts': partial(salt.utils.traverse_dict_and_list, __opts__),
+        'pillar': functools.partial(salt.utils.data.traverse_dict_and_list, pillar),
+        'grains': functools.partial(salt.utils.data.traverse_dict_and_list, __grains__),
+        'opts': functools.partial(salt.utils.data.traverse_dict_and_list, __opts__),
         }
     for matcher, matchs in six.iteritems(kwargs):
         t, matcher = matcher.split(':', 1)
@@ -413,8 +413,8 @@ def ext_pillar(minion_id, pillar, *args, **kwargs):
         stack_config_files += cfgs
     for cfg in stack_config_files:
         if not os.path.isfile(cfg):
-            log.warning('Ignoring pillar stack cfg "{0}": '
-                     'file does not exist'.format(cfg))
+            log.warning(
+                'Ignoring pillar stack cfg "%s": file does not exist', cfg)
             continue
         stack = _process_stack_cfg(cfg, stack, minion_id, pillar)
     return stack
@@ -429,16 +429,15 @@ def _construct_unicode(loader, node):
 
 
 def _process_stack_cfg(cfg, stack, minion_id, pillar):
-    log.debug('Config: {0}'.format(cfg))
+    log.debug('Config: %s', cfg)
     basedir, filename = os.path.split(cfg)
-    yaml.SafeLoader.add_constructor("tag:yaml.org,2002:python/unicode", _construct_unicode)
     jenv = Environment(loader=FileSystemLoader(basedir), extensions=['jinja2.ext.do', salt.utils.jinja.SerializerExtension])
     jenv.globals.update({
         "__opts__": __opts__,
         "__salt__": __salt__,
         "__grains__": __grains__,
         "__stack__": {
-            'traverse': salt.utils.traverse_dict_and_list
+            'traverse': salt.utils.data.traverse_dict_and_list
             },
         "minion_id": minion_id,
         "pillar": pillar,
@@ -447,19 +446,21 @@ def _process_stack_cfg(cfg, stack, minion_id, pillar):
             jenv.get_template(filename).render(stack=stack)):
         if not item.strip():
             continue  # silently ignore whitespace or empty lines
-        paths = glob(os.path.join(basedir, item))
+        paths = glob.glob(os.path.join(basedir, item))
         if not paths:
-            log.warning('Ignoring pillar stack template "{0}": can\'t find from '
-                     'root dir "{1}"'.format(item, basedir))
+            log.warning(
+                'Ignoring pillar stack template "%s": can\'t find from root '
+                'dir "%s"', item, basedir
+            )
             continue
         for path in sorted(paths):
-            log.debug('YAML: basedir={0}, path={1}'.format(basedir, path))
+            log.debug('YAML: basedir=%s, path=%s', basedir, path)
             # FileSystemLoader always expects unix-style paths
             unix_path = _to_unix_slashes(os.path.relpath(path, basedir))
-            obj = yaml.safe_load(jenv.get_template(unix_path).render(stack=stack))
+            obj = salt.utils.yaml.safe_load(jenv.get_template(unix_path).render(stack=stack))
             if not isinstance(obj, dict):
-                log.info('Ignoring pillar stack template "{0}": Can\'t parse '
-                         'as a valid yaml dictionary'.format(path))
+                log.info('Ignoring pillar stack template "%s": Can\'t parse '
+                         'as a valid yaml dictionary', path)
                 continue
             stack = _merge_dict(stack, obj)
     return stack
@@ -497,8 +498,7 @@ def _merge_dict(stack, obj):
                     stack[k] = _cleanup(v)
                     v = stack_k
                 if type(stack[k]) != type(v):
-                    log.debug('Force overwrite, types differ: '
-                              '\'{0}\' != \'{1}\''.format(stack[k], v))
+                    log.debug('Force overwrite, types differ: \'%s\' != \'%s\'', stack[k], v)
                     stack[k] = _cleanup(v)
                 elif isinstance(v, dict):
                     stack[k] = _merge_dict(stack[k], v)
@@ -534,7 +534,7 @@ def _parse_stack_cfg(content):
     Allow top level cfg to be YAML
     '''
     try:
-        obj = yaml.safe_load(content)
+        obj = salt.utils.yaml.safe_load(content)
         if isinstance(obj, list):
             return obj
     except Exception as e:
