@@ -47,19 +47,19 @@ config:
 '''
 
 # Import Python Libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
+import hashlib
 import logging
 import os
-import os.path
-import hashlib
 import re
-import json
-import yaml
 
 # Import Salt Libs
-import salt.ext.six as six
-import salt.utils
-from salt.utils.yamlloader import SaltYamlSafeLoader
+import salt.utils.files
+import salt.utils.json
+import salt.utils.yaml
+
+# Import 3rd-party libs
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -75,7 +75,7 @@ def present(name, api_name, swagger_file, stage_name, api_key_required,
             lambda_integration_role, lambda_region=None, stage_variables=None,
             region=None, key=None, keyid=None, profile=None,
             lambda_funcname_format='{stage}_{api}_{resource}_{method}',
-            authorization_type='NONE'):
+            authorization_type='NONE', error_response_template=None, response_template=None):
     '''
     Ensure the spcified api_name with the corresponding swaggerfile is deployed to the
     given stage_name in AWS ApiGateway.
@@ -125,7 +125,9 @@ def present(name, api_name, swagger_file, stage_name, api_key_required,
     with the following schema.  The lambda functions should throw exceptions for any non successful responses.
     An optional pattern field can be specified in errorMessage field to aid the response mapping from Lambda
     to the proper error return status codes.
+
         .. code-block:: yaml
+
             Error:
               type: object
               properties:
@@ -204,6 +206,35 @@ def present(name, api_name, swagger_file, stage_name, api_key_required,
     authorization_type
         This field can be either 'NONE', or 'AWS_IAM'.  This will be applied to all methods in the given
         swagger spec file.  Default is set to 'NONE'
+
+    error_response_template
+        String value that defines the response template mapping that should be applied in cases error occurs.
+        Refer to AWS documentation for details:
+            http://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html
+        If set to None, the following default value is used:
+            '#set($inputRoot = $input.path(\'$\'))\n'
+            '{\n'
+            '  "errorMessage" : "$inputRoot.errorMessage",\n'
+            '  "errorType" : "$inputRoot.errorType",\n'
+            '  "stackTrace" : [\n'
+            '#foreach($stackTrace in $inputRoot.stackTrace)\n'
+            '    [\n'
+            '#foreach($elem in $stackTrace)\n'
+            '      "$elem"\n'
+            '#if($foreach.hasNext),#end\n'
+            '#end\n'
+            '    ]\n'
+            '#if($foreach.hasNext),#end\n'
+            '#end\n'
+            '  ]\n'
+
+        .. versionadded:: 2017.7.0
+
+    response_template
+        String value that defines the response template mapping applied in case of success (including OPTIONS method)
+        If set to None, empty ({}) template is assumed, which will transfer response from the lambda function as is.
+
+        .. versionadded:: 2017.7.0
     '''
     ret = {'name': name,
            'result': True,
@@ -220,7 +251,9 @@ def present(name, api_name, swagger_file, stage_name, api_key_required,
         # try to open the swagger file and basic validation
         swagger = _Swagger(api_name, stage_name,
                            lambda_funcname_format,
-                           swagger_file, common_args)
+                           swagger_file,
+                           error_response_template, response_template,
+                           common_args)
 
         # retrieve stage variables
         stage_vars = _get_stage_variables(stage_variables)
@@ -363,7 +396,7 @@ def absent(name, api_name, stage_name, nuke_api=False, region=None, key=None, ke
                             ('keyid', keyid),
                             ('profile', profile)])
 
-        swagger = _Swagger(api_name, stage_name, '', None, common_args)
+        swagger = _Swagger(api_name, stage_name, '', None, None, None, common_args)
 
         if not swagger.restApiId:
             ret['comment'] = '[Rest API: {0}] does not exist.'.format(api_name)
@@ -395,14 +428,19 @@ def absent(name, api_name, stage_name, nuke_api=False, region=None, key=None, ke
 
 
 # Helper Swagger Class for swagger version 2.0 API specification
-def _gen_md5_filehash(fname):
+def _gen_md5_filehash(fname, *args):
     '''
     helper function to generate a md5 hash of the swagger definition file
+    any extra argument passed to the function is converted to a string
+    and participates in the hash calculation
     '''
     _hash = hashlib.md5()
-    with salt.utils.fopen(fname, 'rb') as f:
+    with salt.utils.files.fopen(fname, 'rb') as f:
         for chunk in iter(lambda: f.read(4096), b''):
             _hash.update(chunk)
+
+    for extra_arg in args:
+        _hash.update(six.b(str(extra_arg)))
     return _hash.hexdigest()
 
 
@@ -410,7 +448,7 @@ def _dict_to_json_pretty(d, sort_keys=True):
     '''
     helper function to generate pretty printed json output
     '''
-    return json.dumps(d, indent=4, separators=(',', ': '), sort_keys=sort_keys)
+    return salt.utils.json.dumps(d, indent=4, separators=(',', ': '), sort_keys=sort_keys)
 
 
 # Heuristic on whether or not the property name loosely matches given set of 'interesting' factors
@@ -422,7 +460,7 @@ def _name_matches(name, matches):
     for m in matches:
         if name.endswith(m):
             return True
-        if name.lower().endswith('_'+m.lower()):
+        if name.lower().endswith('_' + m.lower()):
             return True
         if name.lower() == m.lower():
             return True
@@ -503,79 +541,77 @@ class _Swagger(object):
     JSON_SCHEMA_DRAFT_4 = 'http://json-schema.org/draft-04/schema#'
 
     # AWS integration templates for normal and options methods
-    REQUEST_TEMPLATE = {'application/json':
-                        '#set($inputRoot = $input.path(\'$\'))\n'
-                        '{\n'
-                        '"header_params" : {\n'
-                        '#set ($map = $input.params().header)\n'
-                        '#foreach( $param in $map.entrySet() )\n'
-                        '"$param.key" : "$param.value" #if( $foreach.hasNext ), #end\n'
-                        '#end\n'
-                        '},\n'
-                        '"query_params" : {\n'
-                        '#set ($map = $input.params().querystring)\n'
-                        '#foreach( $param in $map.entrySet() )\n'
-                        '"$param.key" : "$param.value" #if( $foreach.hasNext ), #end\n'
-                        '#end\n'
-                        '},\n'
-                        '"path_params" : {\n'
-                        '#set ($map = $input.params().path)\n'
-                        '#foreach( $param in $map.entrySet() )\n'
-                        '"$param.key" : "$param.value" #if( $foreach.hasNext ), #end\n'
-                        '#end\n'
-                        '},\n'
-                        '"apigw_context" : {\n'
-                        '"apiId": "$context.apiId",\n'
-                        '"httpMethod": "$context.httpMethod",\n'
-                        '"requestId": "$context.requestId",\n'
-                        '"resourceId": "$context.resourceId",\n'
-                        '"resourcePath": "$context.resourcePath",\n'
-                        '"stage": "$context.stage",\n'
-                        '"identity": {\n'
-                        '  "user":"$context.identity.user",\n'
-                        '  "userArn":"$context.identity.userArn",\n'
-                        '  "userAgent":"$context.identity.userAgent",\n'
-                        '  "sourceIp":"$context.identity.sourceIp",\n'
-                        '  "cognitoIdentityId":"$context.identity.cognitoIdentityId",\n'
-                        '  "cognitoIdentityPoolId":"$context.identity.cognitoIdentityPoolId",\n'
-                        '  "cognitoAuthenticationType":"$context.identity.cognitoAuthenticationType",\n'
-                        '  "cognitoAuthenticationProvider":["$util.escapeJavaScript($context.identity.cognitoAuthenticationProvider)"],\n'
-                        '  "caller":"$context.identity.caller",\n'
-                        '  "apiKey":"$context.identity.apiKey",\n'
-                        '  "accountId":"$context.identity.accountId"\n'
-                        '}\n'
-                        '},\n'
-                        '"body_params" : $input.json(\'$\'),\n'
-                        '"stage_variables": {\n'
-                        '#foreach($variable in $stageVariables.keySet())\n'
-                        '"$variable": "$util.escapeJavaScript($stageVariables.get($variable))"\n'
-                        '#if($foreach.hasNext), #end\n'
-                        '#end\n'
-                        '}\n'
-                        '}'}
+    REQUEST_TEMPLATE = {'application/json': '#set($inputRoot = $input.path(\'$\'))\n'
+                                            '{\n'
+                                            '"header_params" : {\n'
+                                            '#set ($map = $input.params().header)\n'
+                                            '#foreach( $param in $map.entrySet() )\n'
+                                            '"$param.key" : "$param.value" #if( $foreach.hasNext ), #end\n'
+                                            '#end\n'
+                                            '},\n'
+                                            '"query_params" : {\n'
+                                            '#set ($map = $input.params().querystring)\n'
+                                            '#foreach( $param in $map.entrySet() )\n'
+                                            '"$param.key" : "$param.value" #if( $foreach.hasNext ), #end\n'
+                                            '#end\n'
+                                            '},\n'
+                                            '"path_params" : {\n'
+                                            '#set ($map = $input.params().path)\n'
+                                            '#foreach( $param in $map.entrySet() )\n'
+                                            '"$param.key" : "$param.value" #if( $foreach.hasNext ), #end\n'
+                                            '#end\n'
+                                            '},\n'
+                                            '"apigw_context" : {\n'
+                                            '"apiId": "$context.apiId",\n'
+                                            '"httpMethod": "$context.httpMethod",\n'
+                                            '"requestId": "$context.requestId",\n'
+                                            '"resourceId": "$context.resourceId",\n'
+                                            '"resourcePath": "$context.resourcePath",\n'
+                                            '"stage": "$context.stage",\n'
+                                            '"identity": {\n'
+                                            '  "user":"$context.identity.user",\n'
+                                            '  "userArn":"$context.identity.userArn",\n'
+                                            '  "userAgent":"$context.identity.userAgent",\n'
+                                            '  "sourceIp":"$context.identity.sourceIp",\n'
+                                            '  "cognitoIdentityId":"$context.identity.cognitoIdentityId",\n'
+                                            '  "cognitoIdentityPoolId":"$context.identity.cognitoIdentityPoolId",\n'
+                                            '  "cognitoAuthenticationType":"$context.identity.cognitoAuthenticationType",\n'
+                                            '  "cognitoAuthenticationProvider":["$util.escapeJavaScript($context.identity.cognitoAuthenticationProvider)"],\n'
+                                            '  "caller":"$context.identity.caller",\n'
+                                            '  "apiKey":"$context.identity.apiKey",\n'
+                                            '  "accountId":"$context.identity.accountId"\n'
+                                            '}\n'
+                                            '},\n'
+                                            '"body_params" : $input.json(\'$\'),\n'
+                                            '"stage_variables": {\n'
+                                            '#foreach($variable in $stageVariables.keySet())\n'
+                                            '"$variable": "$util.escapeJavaScript($stageVariables.get($variable))"\n'
+                                            '#if($foreach.hasNext), #end\n'
+                                            '#end\n'
+                                            '}\n'
+                                            '}'}
     REQUEST_OPTION_TEMPLATE = {'application/json': '{"statusCode": 200}'}
 
     # AWS integration response template mapping to convert stackTrace part or the error
     # to a uniform format containing strings only. Swagger does not seem to allow defining
     # an array of non-uniform types, to it is not possible to create error model to match
     # exactly what comes out of lambda functions in case of error.
-    RESPONSE_TEMPLATE = {'application/json':
-                         '#set($inputRoot = $input.path(\'$\'))\n'
-                         '{\n'
-                         '  "errorMessage" : "$inputRoot.errorMessage",\n'
-                         '  "errorType" : "$inputRoot.errorType",\n'
-                         '  "stackTrace" : [\n'
-                         '#foreach($stackTrace in $inputRoot.stackTrace)\n'
-                         '    [\n'
-                         '#foreach($elem in $stackTrace)\n'
-                         '      "$elem"\n'
-                         '#if($foreach.hasNext),#end\n'
-                         '#end\n'
-                         '    ]\n'
-                         '#if($foreach.hasNext),#end\n'
-                         '#end\n'
-                         '  ]\n'
-                         '}'}
+    RESPONSE_TEMPLATE = {'application/json': '#set($inputRoot = $input.path(\'$\'))\n'
+                                             '{\n'
+                                             '  "errorMessage" : "$inputRoot.errorMessage",\n'
+                                             '  "errorType" : "$inputRoot.errorType",\n'
+                                             '  "stackTrace" : [\n'
+                                             '#foreach($stackTrace in $inputRoot.stackTrace)\n'
+                                             '    [\n'
+                                             '#foreach($elem in $stackTrace)\n'
+                                             '      "$elem"\n'
+                                             '#if($foreach.hasNext),#end\n'
+                                             '#end\n'
+                                             '    ]\n'
+                                             '#if($foreach.hasNext),#end\n'
+                                             '#end\n'
+                                             '  ]\n'
+                                             '}'}
     RESPONSE_OPTION_TEMPLATE = {}
 
     # This string should not be modified, every API created by this state will carry the description
@@ -638,6 +674,7 @@ class _Swagger(object):
         '''
         Helper class for Swagger Method Response Object
         '''
+
         def __init__(self, r):
             self._r = r
 
@@ -663,23 +700,24 @@ class _Swagger(object):
             return _headers
 
     def __init__(self, api_name, stage_name, lambda_funcname_format,
-                 swagger_file_path, common_aws_args):
+                 swagger_file_path, error_response_template, response_template, common_aws_args):
         self._api_name = api_name
         self._stage_name = stage_name
         self._lambda_funcname_format = lambda_funcname_format
         self._common_aws_args = common_aws_args
         self._restApiId = ''
         self._deploymentId = ''
+        self._error_response_template = error_response_template
+        self._response_template = response_template
 
         if swagger_file_path is not None:
             if os.path.exists(swagger_file_path) and os.path.isfile(swagger_file_path):
                 self._swagger_file = swagger_file_path
-                self._md5_filehash = _gen_md5_filehash(self._swagger_file)
-                with salt.utils.fopen(self._swagger_file, 'rb') as sf:
-                    self._cfg = yaml.load(
-                        sf,
-                        Loader=SaltYamlSafeLoader
-                    )
+                self._md5_filehash = _gen_md5_filehash(self._swagger_file,
+                                                       error_response_template,
+                                                       response_template)
+                with salt.utils.files.fopen(self._swagger_file, 'rb') as sf:
+                    self._cfg = salt.utils.yaml.safe_load(sf)
                 self._swagger_version = ''
             else:
                 raise IOError('Invalid swagger file path, {0}'.format(swagger_file_path))
@@ -710,7 +748,7 @@ class _Swagger(object):
                 if 'responses' not in opobj:
                     raise ValueError('missing mandatory responses field in path item object')
                 for rescode, resobj in six.iteritems(opobj.get('responses')):
-                    if not self._is_http_error_rescode(str(rescode)):
+                    if not self._is_http_error_rescode(str(rescode)):  # future lint: disable=blacklisted-function
                         continue
 
                     # only check for response code from 400-599
@@ -742,6 +780,10 @@ class _Swagger(object):
                                          'be used'.format(modelname))
 
     def _validate_lambda_funcname_format(self):
+        '''
+        Checks if the lambda function name format contains only known elements
+        :return: True on success, ValueError raised on error
+        '''
         try:
             if self._lambda_funcname_format:
                 known_kwargs = dict(stage='',
@@ -1440,6 +1482,15 @@ class _Swagger(object):
         patterns = self._find_patterns(model)
         return patterns[0] if patterns else defaultPattern
 
+    def _get_response_template(self, method_name, http_status):
+        if method_name == 'options' or not self._is_http_error_rescode(http_status):
+            response_templates = {'application/json': self._response_template} \
+                if self._response_template else self.RESPONSE_OPTION_TEMPLATE
+        else:
+            response_templates = {'application/json': self._error_response_template} \
+                if self._error_response_template else self.RESPONSE_TEMPLATE
+        return response_templates
+
     def _parse_method_response(self, method_name, method_response, httpStatus):
         '''
         Helper function to construct the method response params, models, and integration_params
@@ -1460,10 +1511,7 @@ class _Swagger(object):
             method_integration_response_params[response_header] = (
                 "'{0}'".format(header_data.get('default')) if 'default' in header_data else "'*'")
 
-        if method_name == 'options' or not self._is_http_error_rescode(httpStatus):
-            response_templates = _Swagger.RESPONSE_OPTION_TEMPLATE
-        else:
-            response_templates = _Swagger.RESPONSE_TEMPLATE
+        response_templates = self._get_response_template(method_name, httpStatus)
 
         return {'params': method_response_params,
                 'models': method_response_models,
@@ -1550,32 +1598,32 @@ class _Swagger(object):
 
         if 'responses' in method_data:
             for response, response_data in six.iteritems(method_data['responses']):
-                httpStatus = str(response)
+                httpStatus = str(response)  # future lint: disable=blacklisted-function
                 method_response = self._parse_method_response(method_name.lower(),
                                                               _Swagger.SwaggerMethodResponse(response_data), httpStatus)
 
                 mr = __salt__['boto_apigateway.create_api_method_response'](
-                                                                restApiId=self.restApiId,
-                                                                resourcePath=resource_path,
-                                                                httpMethod=method_name.upper(),
-                                                                statusCode=httpStatus,
-                                                                responseParameters=method_response.get('params'),
-                                                                responseModels=method_response.get('models'),
-                                                                **self._common_aws_args)
+                    restApiId=self.restApiId,
+                    resourcePath=resource_path,
+                    httpMethod=method_name.upper(),
+                    statusCode=httpStatus,
+                    responseParameters=method_response.get('params'),
+                    responseModels=method_response.get('models'),
+                    **self._common_aws_args)
                 if not mr.get('created'):
                     ret = _log_error_and_abort(ret, mr)
                     return ret
                 ret = _log_changes(ret, '_deploy_method.create_api_method_response', mr)
 
                 mir = __salt__['boto_apigateway.create_api_integration_response'](
-                                                restApiId=self.restApiId,
-                                                resourcePath=resource_path,
-                                                httpMethod=method_name.upper(),
-                                                statusCode=httpStatus,
-                                                selectionPattern=method_response.get('pattern'),
-                                                responseParameters=method_response.get('integration_params'),
-                                                responseTemplates=method_response.get('response_templates'),
-                                                **self._common_aws_args)
+                    restApiId=self.restApiId,
+                    resourcePath=resource_path,
+                    httpMethod=method_name.upper(),
+                    statusCode=httpStatus,
+                    selectionPattern=method_response.get('pattern'),
+                    responseParameters=method_response.get('integration_params'),
+                    responseTemplates=method_response.get('response_templates'),
+                    **self._common_aws_args)
                 if not mir.get('created'):
                     ret = _log_error_and_abort(ret, mir)
                     return ret
@@ -1619,3 +1667,402 @@ class _Swagger(object):
                     ret = self._deploy_method(ret, path, method, method_data, api_key_required,
                                               lambda_integration_role, lambda_region, authorization_type)
         return ret
+
+
+def usage_plan_present(name, plan_name, description=None, throttle=None, quota=None, region=None, key=None, keyid=None,
+                       profile=None):
+    '''
+    Ensure the spcifieda usage plan with the corresponding metrics is deployed
+
+    .. versionadded:: 2017.7.0
+
+    name
+        name of the state
+
+    plan_name
+        [Required] name of the usage plan
+
+    throttle
+        [Optional] throttling parameters expressed as a dictionary.
+        If provided, at least one of the throttling parameters must be present
+
+        rateLimit
+            rate per second at which capacity bucket is populated
+
+        burstLimit
+            maximum rate allowed
+
+    quota
+        [Optional] quota on the number of api calls permitted by the plan.
+        If provided, limit and period must be present
+
+        limit
+            [Required] number of calls permitted per quota period
+
+        offset
+            [Optional] number of calls to be subtracted from the limit at the beginning of the period
+
+        period
+            [Required] period to which quota applies. Must be DAY, WEEK or MONTH
+
+    .. code-block:: yaml
+
+        UsagePlanPresent:
+          boto_apigateway.usage_plan_present:
+            - plan_name: my_usage_plan
+            - throttle:
+                rateLimit: 70
+                burstLimit: 100
+            - quota:
+                limit: 1000
+                offset: 0
+                period: DAY
+            - profile: my_profile
+
+    '''
+    func_params = locals()
+
+    ret = {'name': name,
+           'result': True,
+           'comment': '',
+           'changes': {}
+           }
+
+    try:
+        common_args = dict([('region', region),
+                            ('key', key),
+                            ('keyid', keyid),
+                            ('profile', profile)])
+
+        existing = __salt__['boto_apigateway.describe_usage_plans'](name=plan_name, **common_args)
+        if 'error' in existing:
+            ret['result'] = False
+            ret['comment'] = 'Failed to describe existing usage plans'
+            return ret
+
+        if not existing['plans']:
+            # plan does not exist, we need to create it
+            if __opts__['test']:
+                ret['comment'] = 'a new usage plan {0} would be created'.format(plan_name)
+                ret['result'] = None
+                return ret
+
+            result = __salt__['boto_apigateway.create_usage_plan'](name=plan_name,
+                                                                   description=description,
+                                                                   throttle=throttle,
+                                                                   quota=quota,
+                                                                   **common_args)
+            if 'error' in result:
+                ret['result'] = False
+                ret['comment'] = 'Failed to create a usage plan {0}, {1}'.format(plan_name, result['error'])
+                return ret
+
+            ret['changes']['old'] = {'plan': None}
+            ret['comment'] = 'A new usage plan {0} has been created'.format(plan_name)
+
+        else:
+            # need an existing plan modified to match given value
+            plan = existing['plans'][0]
+            needs_updating = False
+
+            modifiable_params = (('throttle', ('rateLimit', 'burstLimit')), ('quota', ('limit', 'offset', 'period')))
+            for p, fields in modifiable_params:
+                for f in fields:
+                    actual_param = {} if func_params.get(p) is None else func_params.get(p)
+                    if plan.get(p, {}).get(f, None) != actual_param.get(f, None):
+                        needs_updating = True
+                        break
+
+            if not needs_updating:
+                ret['comment'] = 'usage plan {0} is already in a correct state'.format(plan_name)
+                ret['result'] = True
+                return ret
+
+            if __opts__['test']:
+                ret['comment'] = 'a new usage plan {0} would be updated'.format(plan_name)
+                ret['result'] = None
+                return ret
+
+            result = __salt__['boto_apigateway.update_usage_plan'](plan['id'],
+                                                                   throttle=throttle,
+                                                                   quota=quota,
+                                                                   **common_args)
+            if 'error' in result:
+                ret['result'] = False
+                ret['comment'] = 'Failed to update a usage plan {0}, {1}'.format(plan_name, result['error'])
+                return ret
+
+            ret['changes']['old'] = {'plan': plan}
+            ret['comment'] = 'usage plan {0} has been updated'.format(plan_name)
+
+        newstate = __salt__['boto_apigateway.describe_usage_plans'](name=plan_name, **common_args)
+        if 'error' in existing:
+            ret['result'] = False
+            ret['comment'] = 'Failed to describe existing usage plans after updates'
+            return ret
+
+        ret['changes']['new'] = {'plan': newstate['plans'][0]}
+
+    except (ValueError, IOError) as e:
+        ret['result'] = False
+        ret['comment'] = '{0}'.format(e.args)
+
+    return ret
+
+
+def usage_plan_absent(name, plan_name, region=None, key=None, keyid=None, profile=None):
+    '''
+    Ensures usage plan identified by name is no longer present
+
+    .. versionadded:: 2017.7.0
+
+    name
+        name of the state
+
+    plan_name
+        name of the plan to remove
+
+    .. code-block:: yaml
+
+        usage plan absent:
+          boto_apigateway.usage_plan_absent:
+            - plan_name: my_usage_plan
+            - profile: my_profile
+
+    '''
+    ret = {'name': name,
+           'result': True,
+           'comment': '',
+           'changes': {}
+           }
+
+    try:
+        common_args = dict([('region', region),
+                            ('key', key),
+                            ('keyid', keyid),
+                            ('profile', profile)])
+
+        existing = __salt__['boto_apigateway.describe_usage_plans'](name=plan_name, **common_args)
+        if 'error' in existing:
+            ret['result'] = False
+            ret['comment'] = 'Failed to describe existing usage plans'
+            return ret
+
+        if not existing['plans']:
+            ret['comment'] = 'Usage plan {0} does not exist already'.format(plan_name)
+            return ret
+
+        if __opts__['test']:
+            ret['comment'] = 'Usage plan {0} exists and would be deleted'.format(plan_name)
+            ret['result'] = None
+            return ret
+
+        plan_id = existing['plans'][0]['id']
+        result = __salt__['boto_apigateway.delete_usage_plan'](plan_id, **common_args)
+
+        if 'error' in result:
+            ret['result'] = False
+            ret['comment'] = 'Failed to delete usage plan {0}, {1}'.format(plan_name, result)
+            return ret
+
+        ret['comment'] = 'Usage plan {0} has been deleted'.format(plan_name)
+        ret['changes']['old'] = {'plan': existing['plans'][0]}
+        ret['changes']['new'] = {'plan': None}
+
+    except (ValueError, IOError) as e:
+        ret['result'] = False
+        ret['comment'] = '{0}'.format(e.args)
+
+    return ret
+
+
+def usage_plan_association_present(name, plan_name, api_stages, region=None, key=None, keyid=None, profile=None):
+    '''
+    Ensures usage plan identified by name is added to provided api_stages
+
+    .. versionadded:: 2017.7.0
+
+    name
+        name of the state
+
+    plan_name
+        name of the plan to use
+
+    api_stages
+        list of dictionaries, where each dictionary consists of the following keys:
+
+        apiId
+            apiId of the api to attach usage plan to
+
+        stage
+            stage name of the api to attach usage plan to
+
+    .. code-block:: yaml
+
+        UsagePlanAssociationPresent:
+          boto_apigateway.usage_plan_association_present:
+            - plan_name: my_plan
+            - api_stages:
+              - apiId: 9kb0404ec0
+                stage: my_stage
+              - apiId: l9v7o2aj90
+                stage: my_stage
+            - profile: my_profile
+
+    '''
+    ret = {'name': name,
+           'result': True,
+           'comment': '',
+           'changes': {}
+           }
+    try:
+        common_args = dict([('region', region),
+                            ('key', key),
+                            ('keyid', keyid),
+                            ('profile', profile)])
+
+        existing = __salt__['boto_apigateway.describe_usage_plans'](name=plan_name, **common_args)
+        if 'error' in existing:
+            ret['result'] = False
+            ret['comment'] = 'Failed to describe existing usage plans'
+            return ret
+
+        if not existing['plans']:
+            ret['comment'] = 'Usage plan {0} does not exist'.format(plan_name)
+            ret['result'] = False
+            return ret
+
+        if len(existing['plans']) != 1:
+            ret['comment'] = 'There are multiple usage plans with the same name - it is not supported'
+            ret['result'] = False
+            return ret
+
+        plan = existing['plans'][0]
+        plan_id = plan['id']
+        plan_stages = plan.get('apiStages', [])
+
+        stages_to_add = []
+        for api in api_stages:
+            if api not in plan_stages:
+                stages_to_add.append(api)
+
+        if not stages_to_add:
+            ret['comment'] = 'Usage plan is already asssociated to all api stages'
+            return ret
+
+        result = __salt__['boto_apigateway.attach_usage_plan_to_apis'](plan_id, stages_to_add, **common_args)
+        if 'error' in result:
+            ret['comment'] = 'Failed to associate a usage plan {0} to the apis {1}, {2}'.format(plan_name,
+                                                                                                stages_to_add,
+                                                                                                result['error'])
+            ret['result'] = False
+            return ret
+
+        ret['comment'] = 'successfully associated usage plan to apis'
+        ret['changes']['old'] = plan_stages
+        ret['changes']['new'] = result.get('result', {}).get('apiStages', [])
+
+    except (ValueError, IOError) as e:
+        ret['result'] = False
+        ret['comment'] = '{0}'.format(e.args)
+
+    return ret
+
+
+def usage_plan_association_absent(name, plan_name, api_stages, region=None, key=None, keyid=None, profile=None):
+    '''
+    Ensures usage plan identified by name is removed from provided api_stages
+    If a plan is associated to stages not listed in api_stages parameter,
+    those associations remain intact.
+
+    .. versionadded:: 2017.7.0
+
+    name
+        name of the state
+
+    plan_name
+        name of the plan to use
+
+    api_stages
+        list of dictionaries, where each dictionary consists of the following keys:
+
+        apiId
+            apiId of the api to detach usage plan from
+
+        stage
+            stage name of the api to detach usage plan from
+
+    .. code-block:: yaml
+
+        UsagePlanAssociationAbsent:
+          boto_apigateway.usage_plan_association_absent:
+            - plan_name: my_plan
+            - api_stages:
+              - apiId: 9kb0404ec0
+                stage: my_stage
+              - apiId: l9v7o2aj90
+                stage: my_stage
+            - profile: my_profile
+
+    '''
+    ret = {'name': name,
+           'result': True,
+           'comment': '',
+           'changes': {}
+           }
+    try:
+        common_args = dict([('region', region),
+                            ('key', key),
+                            ('keyid', keyid),
+                            ('profile', profile)])
+
+        existing = __salt__['boto_apigateway.describe_usage_plans'](name=plan_name, **common_args)
+        if 'error' in existing:
+            ret['result'] = False
+            ret['comment'] = 'Failed to describe existing usage plans'
+            return ret
+
+        if not existing['plans']:
+            ret['comment'] = 'Usage plan {0} does not exist'.format(plan_name)
+            ret['result'] = False
+            return ret
+
+        if len(existing['plans']) != 1:
+            ret['comment'] = 'There are multiple usage plans with the same name - it is not supported'
+            ret['result'] = False
+            return ret
+
+        plan = existing['plans'][0]
+        plan_id = plan['id']
+        plan_stages = plan.get('apiStages', [])
+
+        if not plan_stages:
+            ret['comment'] = 'Usage plan {0} has no associated stages already'.format(plan_name)
+            return ret
+
+        stages_to_remove = []
+        for api in api_stages:
+            if api in plan_stages:
+                stages_to_remove.append(api)
+
+        if not stages_to_remove:
+            ret['comment'] = 'Usage plan is already not asssociated to any api stages'
+            return ret
+
+        result = __salt__['boto_apigateway.detach_usage_plan_from_apis'](plan_id, stages_to_remove, **common_args)
+        if 'error' in result:
+            ret['comment'] = 'Failed to disassociate a usage plan {0} from the apis {1}, {2}'.format(plan_name,
+                                                                                                     stages_to_remove,
+                                                                                                     result['error'])
+            ret['result'] = False
+            return ret
+
+        ret['comment'] = 'successfully disassociated usage plan from apis'
+        ret['changes']['old'] = plan_stages
+        ret['changes']['new'] = result.get('result', {}).get('apiStages', [])
+
+    except (ValueError, IOError) as e:
+        ret['result'] = False
+        ret['comment'] = '{0}'.format(e.args)
+
+    return ret
