@@ -50,12 +50,7 @@ config:
         - keyid: GKTADJGHEIQSXMKKRBJ08H
         - key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
         - tags:
-          -
-            - key1
-            - value1
-          -
-            - key2
-            - value2
+            key: value
 
 .. code-block:: yaml
 
@@ -76,13 +71,14 @@ config:
 '''
 
 # Import Python Libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 import os
 
 # Import Salt Libs
+from salt.ext import six
 from salt.exceptions import SaltInvocationError
-import salt.utils
+import salt.utils.data
 
 log = logging.getLogger(__name__)
 
@@ -106,6 +102,7 @@ def present(name,
             storage_type=None,
             db_security_groups=None,
             vpc_security_group_ids=None,
+            vpc_security_groups=None,
             availability_zone=None,
             db_subnet_group_name=None,
             preferred_maintenance_window=None,
@@ -179,7 +176,10 @@ def present(name,
         A list of DB security groups to associate with this DB instance.
 
     vpc_security_group_ids
-        A list of EC2 VPC security groups to associate with this DB instance.
+        A list of EC2 VPC security group IDs to associate with this DB instance.
+
+    vpc_security_groups
+        A list of EC2 VPC security groups (IDs or Name tags) to associate with this DB instance.
 
     availability_zone
         The EC2 Availability Zone that the database instance will be created
@@ -261,7 +261,7 @@ def present(name,
         the state. Available states: available, modifying, backing-up
 
     tags
-        A list of tags.
+        A dict of tags.
 
     copy_tags_to_snapshot
         Specifies whether tags are copied from the DB instance to snapshots of
@@ -301,7 +301,7 @@ def present(name,
         A dict with region, key and keyid, or a pillar key (string) that
         contains a dict with region, key and keyid.
 
-    .. _create_dbinstance: https://boto3.readthedocs.io/en/latest/reference/services/rds.html#RDS.Client.create_db_instance
+    .. _create_db_instance: https://boto3.readthedocs.io/en/latest/reference/services/rds.html#RDS.Client.create_db_instance
     '''
     ret = {'name': name,
            'result': True,
@@ -313,7 +313,7 @@ def present(name,
 
     if not r.get('exists'):
         if __opts__['test']:
-            ret['comment'] = 'RDS instance {0} is set to be created.'.format(name)
+            ret['comment'] = 'RDS instance {0} would be created.'.format(name)
             ret['result'] = None
             return ret
 
@@ -323,6 +323,7 @@ def present(name,
                                         master_user_password,
                                         db_name, db_security_groups,
                                         vpc_security_group_ids,
+                                        vpc_security_groups,
                                         availability_zone,
                                         db_subnet_group_name,
                                         preferred_maintenance_window,
@@ -350,9 +351,11 @@ def present(name,
             ret['comment'] = 'Failed to create RDS instance {0}.'.format(r['error']['message'])
             return ret
 
-        _describe = __salt__['boto_rds.describe'](name, tags, region, key, keyid, profile)
         ret['changes']['old'] = {'instance': None}
-        ret['changes']['new'] = _describe
+        ret['changes']['new'] = {
+              'instance': __salt__['boto_rds.describe_db_instances'](
+              name=name, jmespath='DBInstances[0]', region=region, key=key,
+              keyid=keyid, profile=profile)}
         ret['comment'] = 'RDS instance {0} created.'.format(name)
     else:
         ret['comment'] = 'RDS instance {0} exists.'.format(name)
@@ -376,13 +379,13 @@ def replica_present(name, source, db_instance_class=None,
             - source: mydb
     '''
     ret = {'name': name,
-           'result': None,
+           'result': True,
            'comment': '',
            'changes': {}
            }
     replica_exists = __salt__['boto_rds.exists'](name, tags, region, key,
                                                  keyid, profile)
-    if not replica_exists:
+    if not replica_exists.get('exists'):
         if __opts__['test']:
             ret['comment'] = 'RDS read replica {0} is set to be created '.format(name)
             ret['result'] = None
@@ -396,25 +399,30 @@ def replica_present(name, source, db_instance_class=None,
                                                            tags, region, key,
                                                            keyid, profile)
         if created:
-            config = __salt__['boto_rds.describe'](name, tags, region,
-                                                   key, keyid, profile)
-            ret['result'] = True
             ret['comment'] = 'RDS replica {0} created.'.format(name)
-            ret['changes']['old'] = None
-            ret['changes']['new'] = config
+            ret['changes']['old'] = {'instance': None}
+            ret['changes']['new'] = {
+                  'instance': __salt__['boto_rds.describe_db_instances'](
+                  name=name, jmespath='DBInstances[0]', region=region,
+                  key=key, keyid=keyid, profile=profile)}
         else:
             ret['result'] = False
             ret['comment'] = 'Failed to create RDS replica {0}.'.format(name)
     else:
-        _describe = __salt__['boto_rds.describe'](name, tags, region, key,
-                                                  keyid, profile)
-        if db_parameter_group_name is not None and _describe['db_parameter_groups'][0]['DBParameterGroupName'] != db_parameter_group_name:
-            modified = __salt__['boto_rds.modify_db_instance'](name, db_parameter_group_name=db_parameter_group_name, region=region,
-                                                               key=key, keyid=keyid, profile=profile)
+        jmespath = 'DBInstances[0].DBParameterGroups[0].DBParameterGroupName'
+        pmg_name = __salt__['boto_rds.describe_db_instances'](name=name,
+              jmespath=jmespath, region=region, key=key, keyid=keyid,
+              profile=profile)
+        pmg_name = pmg_name[0] if len(pmg_name) else None
+        if pmg_name != db_parameter_group_name:
+            modified = __salt__['boto_rds.modify_db_instance'](
+                  name=name, db_parameter_group_name=db_parameter_group_name,
+                  region=region, key=key, keyid=keyid, profile=profile)
             if not modified:
                 ret['result'] = False
-                ret['comment'] = 'Failed to update parameter group of {0} RDS instance.'.format(name)
-            ret['changes']['old'] = _describe['db_parameter_groups'][0]['DBParameterGroupName']
+                ret['comment'] = ('Failed to update parameter group of {0} RDS '
+                                  'instance.'.format(name))
+            ret['changes']['old'] = pmg_name
             ret['changes']['new'] = db_parameter_group_name
         ret['result'] = True
         ret['comment'] = 'RDS replica {0} exists.'.format(name)
@@ -442,7 +450,7 @@ def subnet_group_present(name, description, subnet_ids=None, subnet_names=None,
         Subnet group description.
 
     tags
-        A list of tags.
+        A dict of tags.
 
     region
         Region to connect to.
@@ -457,7 +465,7 @@ def subnet_group_present(name, description, subnet_ids=None, subnet_names=None,
         A dict with region, key and keyid, or a pillar key (string) that
         contains a dict with region, key and keyid.
     '''
-    if not salt.utils.exactly_one((subnet_ids, subnet_names)):
+    if not salt.utils.data.exactly_one((subnet_ids, subnet_names)):
         raise SaltInvocationError('One (but not both) of subnet_ids or '
                                   'subnet_names must be provided.')
 
@@ -490,21 +498,17 @@ def subnet_group_present(name, description, subnet_ids=None, subnet_names=None,
 
     exists = __salt__['boto_rds.subnet_group_exists'](name=name, tags=tags, region=region, key=key,
                                                       keyid=keyid, profile=profile)
-    if not exists:
+    if not exists.get('exists'):
         if __opts__['test']:
             ret['comment'] = 'Subnet group {0} is set to be created.'.format(name)
             ret['result'] = None
             return ret
-        if not r.get('created'):
-            ret['result'] = False
-            ret['comment'] = 'Failed to create {0} subnet group.'.format(r['error']['message'])
-            return ret
         created = __salt__['boto_rds.create_subnet_group'](name=name,
-                                                     description=description,
-                                                     subnet_ids=subnet_ids,
-                                                     tags=tags, region=region,
-                                                     key=key, keyid=keyid,
-                                                     profile=profile)
+                                                           description=description,
+                                                           subnet_ids=subnet_ids,
+                                                           tags=tags, region=region,
+                                                           key=key, keyid=keyid,
+                                                           profile=profile)
 
         if not created:
             ret['result'] = False
@@ -513,6 +517,7 @@ def subnet_group_present(name, description, subnet_ids=None, subnet_names=None,
         ret['changes']['old'] = None
         ret['changes']['new'] = name
         ret['comment'] = 'Subnet {0} created.'.format(name)
+        return ret
     else:
         ret['comment'] = 'Subnet {0} present.'.format(name)
 
@@ -520,8 +525,8 @@ def subnet_group_present(name, description, subnet_ids=None, subnet_names=None,
 
 
 def absent(name, skip_final_snapshot=None, final_db_snapshot_identifier=None,
-           tags=None, region=None, key=None, keyid=None, profile=None,
-           wait_for_deletion=True, timeout=180):
+           tags=None, wait_for_deletion=True, timeout=180,
+           region=None, key=None, keyid=None, profile=None):
     '''
     Ensure RDS instance is absent.
 
@@ -538,7 +543,14 @@ def absent(name, skip_final_snapshot=None, final_db_snapshot_identifier=None,
         snapshot.
 
     tags
-        A list of tags.
+        A dict of tags.
+
+    wait_for_deletion (bool)
+        Wait for the RDS instance to be deleted completely before finishing
+        the state.
+
+    timeout (in seconds)
+        The amount of time that can pass before raising an Exception.
 
     region
         Region to connect to.
@@ -552,15 +564,6 @@ def absent(name, skip_final_snapshot=None, final_db_snapshot_identifier=None,
     profile
         A dict with region, key and keyid, or a pillar key (string) that
         contains a dict with region, key and keyid.
-
-    .. _create_db_instance: https://boto3.readthedocs.io/en/latest/reference/services/rds.html#RDS.Client.create_db_instance
-
-    wait_for_deletion (bool)
-        Wait for the RDS instance to be deleted completely before finishing
-        the state.
-
-    timeout (in seconds)
-        The amount of time that can pass before raising an Exception.
     '''
     ret = {'name': name,
            'result': True,
@@ -568,27 +571,27 @@ def absent(name, skip_final_snapshot=None, final_db_snapshot_identifier=None,
            'changes': {}
            }
 
-    exists = __salt__['boto_rds.exists'](name, tags, region, key, keyid,
-                                         profile)
-    if not exists:
+    current = __salt__['boto_rds.describe_db_instances'](
+            name=name, region=region, key=key, keyid=keyid, profile=profile)
+    if not len(current):
         ret['result'] = True
-        ret['comment'] = '{0} RDS does not exist.'.format(name)
+        ret['comment'] = '{0} RDS already absent.'.format(name)
         return ret
 
     if __opts__['test']:
-        ret['comment'] = 'RDS {0} is set to be removed.'.format(name)
+        ret['comment'] = 'RDS {0} would be removed.'.format(name)
         ret['result'] = None
         return ret
     deleted = __salt__['boto_rds.delete'](name, skip_final_snapshot,
                                           final_db_snapshot_identifier,
-                                          region, key, keyid, profile,
+                                          region, key, keyid, profile, tags,
                                           wait_for_deletion, timeout)
     if not deleted:
         ret['result'] = False
         ret['comment'] = 'Failed to delete {0} RDS.'.format(name)
         return ret
-    ret['changes']['old'] = name
-    ret['changes']['new'] = None
+    ret['changes']['old'] = {'instance': current[0]}
+    ret['changes']['new'] = {'instance': None}
     ret['comment'] = 'RDS {0} deleted.'.format(name)
     return ret
 
@@ -652,7 +655,7 @@ def parameter_present(name, db_parameter_group_family, description, parameters=N
         parameters.
 
     tags
-        A list of tags.
+        A dict of tags.
 
     region
         Region to connect to.
@@ -673,7 +676,7 @@ def parameter_present(name, db_parameter_group_family, description, parameters=N
            'changes': {}
            }
     res = __salt__['boto_rds.parameter_group_exists'](name=name, tags=tags, region=region, key=key,
-                                                         keyid=keyid, profile=profile)
+                                                      keyid=keyid, profile=profile)
     if not res.get('exists'):
         if __opts__['test']:
             ret['comment'] = 'Parameter group {0} is set to be created.'.format(name)
@@ -695,16 +698,24 @@ def parameter_present(name, db_parameter_group_family, description, parameters=N
         changed = {}
         for items in parameters:
             for k, value in items.items():
-                params[k] = value
-        logging.debug('Parameters from user are : {0}.'.format(params))
+                if type(value) is bool:
+                    params[k] = 'on' if value else 'off'
+                else:
+                    params[k] = six.text_type(value)
+        log.debug('Parameters from user are : %s.', params)
         options = __salt__['boto_rds.describe_parameters'](name=name, region=region, key=key, keyid=keyid, profile=profile)
         if not options.get('result'):
             ret['result'] = False
             ret['comment'] = os.linesep.join([ret['comment'], 'Faled to get parameters for group  {0}.'.format(name)])
             return ret
         for parameter in options['parameters'].values():
-            if parameter['ParameterName'] in params and str(params.get(parameter['ParameterName'])) != str(parameter['ParameterValue']):
-                logging.debug('Values that are being compared are {0}:{1} .'.format(params.get(parameter['ParameterName']), parameter['ParameterValue']))
+            if parameter['ParameterName'] in params and params.get(parameter['ParameterName']) != six.text_type(parameter['ParameterValue']):
+                log.debug(
+                    'Values that are being compared for %s are %s:%s.',
+                    parameter['ParameterName'],
+                    params.get(parameter['ParameterName']),
+                    parameter['ParameterValue']
+                )
                 changed[parameter['ParameterName']] = params.get(parameter['ParameterName'])
         if len(changed) > 0:
             if __opts__['test']:
@@ -713,9 +724,9 @@ def parameter_present(name, db_parameter_group_family, description, parameters=N
                 return ret
             update = __salt__['boto_rds.update_parameter_group'](name, parameters=changed, apply_method=apply_method, tags=tags, region=region,
                                                                  key=key, keyid=keyid, profile=profile)
-            if not update:
+            if 'error' in update:
                 ret['result'] = False
-                ret['comment'] = os.linesep.join([ret['comment'], 'Failed to change parameters {0} for group {1}.'.format(changed, name)])
+                ret['comment'] = os.linesep.join([ret['comment'], 'Failed to change parameters {0} for group {1}:'.format(changed, name), update['error']['message']])
                 return ret
             ret['changes']['Parameters'] = changed
             ret['comment'] = os.linesep.join([ret['comment'], 'Parameters {0} for group {1} are changed.'.format(changed, name)])
