@@ -30,11 +30,11 @@ Management of Docker volumes
 These states were moved from the :mod:`docker <salt.states.docker>` state
 module (formerly called **dockerng**) in the 2017.7.0 release.
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 
 # Import salt libs
-import salt.utils
+import salt.utils.data
 
 # Enable proper logging
 log = logging.getLogger(__name__)  # pylint: disable=invalid-name
@@ -134,8 +134,8 @@ def present(name, driver=None, driver_opts=None, force=False):
            'changes': {},
            'result': False,
            'comment': ''}
-    if salt.utils.is_dictlist(driver_opts):
-        driver_opts = salt.utils.repack_dictlist(driver_opts)
+    if salt.utils.data.is_dictlist(driver_opts):
+        driver_opts = salt.utils.data.repack_dictlist(driver_opts)
     volume = _find_volume(name)
     if not volume:
         if __opts__['test']:
