@@ -24,8 +24,18 @@ Mount any type of mountable filesystem with the mounted function:
         - pass_num: 2
         - persist: True
         - mkmnt: True
+
+    /var/lib/bigdata:
+      mount.mounted:
+        - device: /srv/bigdata
+        - fstype: none
+        - opts: bind
+        - dump: 0
+        - pass_num: 0
+        - persist: True
+        - mkmnt: True
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import os.path
@@ -35,10 +45,19 @@ import re
 from salt.ext.six import string_types
 
 import logging
-import salt.ext.six as six
+from salt.ext import six
 log = logging.getLogger(__name__)
 
 
+def _size_convert(_re_size):
+    converted_size = int(_re_size.group('size_value'))
+    if _re_size.group('size_unit') == 'm':
+        converted_size = int(converted_size) * 1024
+    if _re_size.group('size_unit') == 'g':
+        converted_size = int(converted_size) * 1024 * 1024
+    return converted_size
+
+
 def mounted(name,
             device,
             fstype,
@@ -56,7 +75,8 @@ def mounted(name,
             extra_mount_invisible_keys=None,
             extra_mount_ignore_fs_keys=None,
             extra_mount_translate_options=None,
-            hidden_opts=None):
+            hidden_opts=None,
+            **kwargs):
     '''
     Verify that a device is mounted
 
@@ -69,7 +89,7 @@ def mounted(name,
 
     fstype
         The filesystem type, this will be ``xfs``, ``ext2/3/4`` in the case of classic
-        filesystems, and ``fuse`` in the case of fuse mounts
+        filesystems, ``fuse`` in the case of fuse mounts, and ``nfs`` in the case of nfs mounts
 
     mkmnt
         If the mount point is not present then the state will fail, set ``mkmnt: True``
@@ -94,7 +114,7 @@ def mounted(name,
         Set if the mount should be mounted immediately, Default is ``True``
 
     user
-        The user to own the mount; this defaults to the user salt is
+        The account used to execute the mount; this defaults to the user salt is
         running as on the minion
 
     match_on
@@ -177,6 +197,8 @@ def mounted(name,
            'result': True,
            'comment': ''}
 
+    update_mount_cache = False
+
     if device_name_regex is None:
         device_name_regex = []
 
@@ -355,11 +377,7 @@ def mounted(name,
 
                     size_match = re.match(r'size=(?P<size_value>[0-9]+)(?P<size_unit>k|m|g)', opt)
                     if size_match:
-                        converted_size = int(size_match.group('size_value'))
-                        if size_match.group('size_unit') == 'm':
-                            converted_size = int(size_match.group('size_value')) * 1024
-                        if size_match.group('size_unit') == 'g':
-                            converted_size = int(size_match.group('size_value')) * 1024 * 1024
+                        converted_size = _size_convert(size_match)
                         opt = "size={0}k".format(converted_size)
                     # make cifs option user synonym for option username which is reported by /proc/mounts
                     if fstype in ['cifs'] and opt.split('=')[0] == 'user':
@@ -379,10 +397,20 @@ def mounted(name,
                             _info = __salt__[name_id_opts[_param]](_givenid)
                             if _info and _param in _info:
                                 _id = _info[_param]
-                        opt = _param + '=' + str(_id)
+                        opt = _param + '=' + six.text_type(_id)
+
+                    _active_superopts = active[real_name].get('superopts', [])
+                    for _active_opt in _active_superopts:
+                        size_match = re.match(r'size=(?P<size_value>[0-9]+)(?P<size_unit>k|m|g)', _active_opt)
+                        if size_match:
+                            converted_size = _size_convert(size_match)
+                            opt = "size={0}k".format(converted_size)
+                            _active_superopts.remove(_active_opt)
+                            _active_opt = "size={0}k".format(converted_size)
+                            _active_superopts.append(_active_opt)
 
                     if opt not in active[real_name]['opts'] \
-                    and opt not in active[real_name].get('superopts', []) \
+                    and opt not in _active_superopts \
                     and opt not in mount_invisible_options \
                     and opt not in mount_ignore_fs_keys.get(fstype, []) \
                     and opt not in mount_invisible_keys:
@@ -413,6 +441,50 @@ def mounted(name,
                                 # don't write remount into fstab
                                 if 'remount' in opts:
                                     opts.remove('remount')
+
+                            # Update the cache
+                            update_mount_cache = True
+
+                mount_cache = __salt__['mount.read_mount_cache'](real_name)
+                if 'opts' in mount_cache:
+                    _missing = [opt for opt in mount_cache['opts']
+                                if opt not in opts]
+
+                    if _missing:
+                        if __opts__['test']:
+                            ret['result'] = None
+                            ret['comment'] = ('Remount would be forced because'
+                                              ' options ({0})'
+                                              'changed'.format(','.join(_missing)))
+                            return ret
+                        else:
+                            # Some file systems require umounting and mounting if options change
+                            # add others to list that require similiar functionality
+                            if fstype in ['nfs', 'cvfs'] or fstype.startswith('fuse'):
+                                ret['changes']['umount'] = "Forced unmount and mount because " \
+                                                            + "options ({0}) changed".format(opt)
+                                unmount_result = __salt__['mount.umount'](real_name)
+                                if unmount_result is True:
+                                    mount_result = __salt__['mount.mount'](real_name, device, mkmnt=mkmnt, fstype=fstype, opts=opts)
+                                    ret['result'] = mount_result
+                                else:
+                                    ret['result'] = False
+                                    ret['comment'] = 'Unable to unmount {0}: {1}.'.format(real_name, unmount_result)
+                                    return ret
+                            else:
+                                ret['changes']['umount'] = "Forced remount because " \
+                                                            + "options ({0}) changed".format(opt)
+                                remount_result = __salt__['mount.remount'](real_name, device, mkmnt=mkmnt, fstype=fstype, opts=opts)
+                                ret['result'] = remount_result
+                                # Cleanup after the remount, so we
+                                # don't write remount into fstab
+                                if 'remount' in opts:
+                                    opts.remove('remount')
+
+                        update_mount_cache = True
+                else:
+                    update_mount_cache = True
+
             if real_device not in device_list:
                 # name matches but device doesn't - need to umount
                 _device_mismatch_is_ignored = None
@@ -443,6 +515,7 @@ def mounted(name,
                         ret['comment'] = "Unable to unmount"
                         ret['result'] = None
                         return ret
+                    update_mount_cache = True
             else:
                 ret['comment'] = 'Target was already mounted'
     # using a duplicate check so I can catch the results of a umount
@@ -466,6 +539,7 @@ def mounted(name,
 
             out = __salt__['mount.mount'](name, device, mkmnt, fstype, opts, user=user)
             active = __salt__['mount.active'](extended=True)
+            update_mount_cache = True
             if isinstance(out, string_types):
                 # Failed to (re)mount, the state has failed!
                 ret['comment'] = out
@@ -565,6 +639,13 @@ def mounted(name,
                                                   config,
                                                   match_on=match_on)
 
+        if update_mount_cache:
+            cache_result = __salt__['mount.write_mount_cache'](real_name,
+                                                               device,
+                                                               mkmnt=mkmnt,
+                                                               fstype=fstype,
+                                                               mount_opts=opts)
+
         if out == 'present':
             ret['comment'] += '. Entry already exists in the fstab.'
             return ret
@@ -673,7 +754,8 @@ def unmounted(name,
               device=None,
               config='/etc/fstab',
               persist=False,
-              user=None):
+              user=None,
+              **kwargs):
     '''
     .. versionadded:: 0.17.0
 
@@ -703,6 +785,8 @@ def unmounted(name,
            'result': True,
            'comment': ''}
 
+    update_mount_cache = False
+
     # Get the active data
     active = __salt__['mount.active'](extended=True)
     if name not in active:
@@ -717,8 +801,10 @@ def unmounted(name,
             return ret
         if device:
             out = __salt__['mount.umount'](name, device, user=user)
+            update_mount_cache = True
         else:
             out = __salt__['mount.umount'](name, user=user)
+            update_mount_cache = True
         if isinstance(out, string_types):
             # Failed to umount, the state has failed!
             ret['comment'] = out
@@ -731,6 +817,9 @@ def unmounted(name,
             ret['comment'] = 'Execute set to False, Target was not unmounted'
             ret['result'] = True
 
+    if update_mount_cache:
+        cache_result = __salt__['mount.delete_mount_cache'](name)
+
     if persist:
         # Override default for Mac OS
         if __grains__['os'] in ['MacOS', 'Darwin'] and config == '/etc/fstab':
