@@ -21,25 +21,26 @@ Please check Installation_ for complete details.
 .. _NAPALM: https://napalm.readthedocs.io
 .. _Installation: https://napalm.readthedocs.io/en/latest/installation.html
 '''
-from __future__ import absolute_import
-
-import json
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
+
 log = logging.getLogger(__file__)
 
 # Import third party libs
 try:
-    import yaml
-    # pylint: disable=W0611
+    # pylint: disable=unused-import
     import napalm_yang
     HAS_NAPALM_YANG = True
-    # pylint: enable=W0611
+    # pylint: enable=unused-import
 except ImportError:
     HAS_NAPALM_YANG = False
 
 # Import salt modules
-from salt.utils import fopen
+import salt.utils.files
+import salt.utils.json
 import salt.utils.napalm
+import salt.utils.stringutils
+import salt.utils.yaml
 
 # ------------------------------------------------------------------------------
 # state properties
@@ -76,7 +77,7 @@ def __virtual__():
 
 def managed(name,
             data,
-            models,
+            *models,
             **kwargs):
     '''
     Manage the device configuration given the input data strucuted
@@ -142,6 +143,8 @@ def managed(name,
                 config:
                   description: "description example"
     '''
+    if isinstance(models, tuple) and isinstance(models[0], list):
+        models = models[0]
     ret = salt.utils.napalm.default_ret(name)
     test = kwargs.get('test', False) or __opts__.get('test', False)
     debug = kwargs.get('debug', False) or __opts__.get('debug', False)
@@ -150,19 +153,23 @@ def managed(name,
     return_compliance_report = kwargs.get('compliance_report', False) or __opts__.get('compliance_report', False)
     profiles = kwargs.get('profiles', [])
     temp_file = __salt__['temp.file']()
-    log.debug('Creating temp file: {0}'.format(temp_file))
+    log.debug('Creating temp file: %s', temp_file)
     if 'to_dict' not in data:
         data = {'to_dict': data}
     data = [data]
-    with fopen(temp_file, 'w') as file_handle:
-        yaml.safe_dump(json.loads(json.dumps(data)), file_handle, encoding='utf-8', allow_unicode=True)
-    device_config = __salt__['napalm_yang.parse'](models,
+    with salt.utils.files.fopen(temp_file, 'w') as file_handle:
+        salt.utils.yaml.safe_dump(
+            salt.utils.json.loads(salt.utils.json.dumps(data)),
+            file_handle,
+            encoding='utf-8'
+        )
+    device_config = __salt__['napalm_yang.parse'](*models,
                                                   config=True,
                                                   profiles=profiles)
     log.debug('Parsed the config from the device:')
     log.debug(device_config)
     compliance_report = __salt__['napalm_yang.compliance_report'](device_config,
-                                                                  models,
+                                                                  *models,
                                                                   filepath=temp_file)
     log.debug('Compliance report:')
     log.debug(compliance_report)
@@ -179,7 +186,7 @@ def managed(name,
     if '_kwargs' in data:
         data.pop('_kwargs')
     loaded_changes = __salt__['napalm_yang.load_config'](data,
-                                                         models,
+                                                         *models,
                                                          profiles=profiles,
                                                          test=test,
                                                          debug=debug,
@@ -199,7 +206,7 @@ def managed(name,
 
 def configured(name,
                data,
-               models,
+               *models,
                **kwargs):
     '''
     Configure the network device, given the input data strucuted
@@ -269,6 +276,8 @@ def configured(name,
                 config:
                   description: "description example"
     '''
+    if isinstance(models, tuple) and isinstance(models[0], list):
+        models = models[0]
     ret = salt.utils.napalm.default_ret(name)
     test = kwargs.get('test', False) or __opts__.get('test', False)
     debug = kwargs.get('debug', False) or __opts__.get('debug', False)
@@ -278,7 +287,7 @@ def configured(name,
     if '_kwargs' in data:
         data.pop('_kwargs')
     loaded_changes = __salt__['napalm_yang.load_config'](data,
-                                                         models,
+                                                         *models,
                                                          profiles=profiles,
                                                          test=test,
                                                          debug=debug,
