@@ -82,17 +82,19 @@ Management of SmartOS Standalone Compute Nodes
     be manually reset to their default value.
     The same behavior as when using 'vmadm update'.
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import Python libs
 import logging
 import os
 
 # Import Salt libs
-import salt.utils
-import salt.utils.files
 import salt.utils.atomicfile
-from salt.utils.odict import OrderedDict
+import salt.utils.data
+import salt.utils.files
+
+# Import 3rd party libs
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -122,15 +124,16 @@ def _load_config():
     config = {}
 
     if os.path.isfile('/usbkey/config'):
-        with salt.utils.fopen('/usbkey/config', 'r') as config_file:
+        with salt.utils.files.fopen('/usbkey/config', 'r') as config_file:
             for optval in config_file:
+                optval = salt.utils.stringutils.to_unicode(optval)
                 if optval[0] == '#':
                     continue
                 if '=' not in optval:
                     continue
                 optval = optval.split('=')
                 config[optval[0].lower()] = optval[1].strip().strip('"')
-    log.debug('smartos.config - read /usbkey/config: {0}'.format(config))
+    log.debug('smartos.config - read /usbkey/config: %s', config)
     return config
 
 
@@ -141,12 +144,16 @@ def _write_config(config):
     try:
         with salt.utils.atomicfile.atomic_open('/usbkey/config', 'w') as config_file:
             config_file.write("#\n# This file was generated by salt\n#\n")
-            for prop in OrderedDict(sorted(config.items())):
-                if ' ' in str(config[prop]):
+            for prop in salt.utils.odict.OrderedDict(sorted(config.items())):
+                if ' ' in six.text_type(config[prop]):
                     if not config[prop].startswith('"') or not config[prop].endswith('"'):
                         config[prop] = '"{0}"'.format(config[prop])
-                config_file.write("{0}={1}\n".format(prop, config[prop]))
-        log.debug('smartos.config - wrote /usbkey/config: {0}'.format(config))
+                config_file.write(
+                    salt.utils.stringutils.to_str(
+                        "{0}={1}\n".format(prop, config[prop])
+                    )
+                )
+        log.debug('smartos.config - wrote /usbkey/config: %s', config)
     except IOError:
         return False
 
@@ -160,7 +167,7 @@ def _parse_vmconfig(config, instances):
     vmconfig = None
 
     if isinstance(config, (salt.utils.odict.OrderedDict)):
-        vmconfig = OrderedDict()
+        vmconfig = salt.utils.odict.OrderedDict()
         for prop in config:
             if prop not in instances:
                 vmconfig[prop] = config[prop]
@@ -171,6 +178,9 @@ def _parse_vmconfig(config, instances):
                 for instance in config[prop]:
                     instance_config = config[prop][instance]
                     instance_config[instances[prop]] = instance
+                    ## some property are lowercase
+                    if 'mac' in instance_config:
+                        instance_config['mac'] = instance_config['mac'].lower()
                     vmconfig[prop].append(instance_config)
     else:
         log.error('smartos.vm_present::parse_vmconfig - failed to parse')
@@ -187,8 +197,8 @@ def _get_instance_changes(current, state):
     state_keys = set(state.keys())
 
     # compare configs
-    changed = salt.utils.compare_dicts(current, state)
-    for change in changed:
+    changed = salt.utils.data.compare_dicts(current, state)
+    for change in salt.utils.data.compare_dicts(current, state):
         if change in changed and changed[change]['old'] == "":
             del changed[change]
         if change in changed and changed[change]['new'] == "":
@@ -223,7 +233,7 @@ def config_present(name, value):
         value = ""
 
     if name in config:
-        if str(config[name]) == str(value):
+        if six.text_type(config[name]) == six.text_type(value):
             # we're good
             ret['result'] = True
             ret['comment'] = 'property {0} already has value "{1}"'.format(name, value)
@@ -462,7 +472,7 @@ def vm_present(name, vmconfig, config=None):
         'reprovision': False,
     }
     config.update(state_config)
-    log.debug('smartos.vm_present::{0}::config - {1}'.format(name, config))
+    log.debug('smartos.vm_present::%s::config - %s', name, config)
 
     # map special vmconfig parameters
     #  collections have set/remove handlers
@@ -486,7 +496,7 @@ def vm_present(name, vmconfig, config=None):
 
     # parse vmconfig
     vmconfig = _parse_vmconfig(vmconfig, vmconfig_type['instance'])
-    log.debug('smartos.vm_present::{0}::vmconfig - {1}'.format(name, vmconfig))
+    log.debug('smartos.vm_present::%s::vmconfig - %s', name, vmconfig)
 
     # set hostname if needed
     if 'hostname' not in vmconfig:
@@ -557,7 +567,7 @@ def vm_present(name, vmconfig, config=None):
             enforce = True
             if 'enforce_{0}'.format(collection) in config:
                 enforce = config['enforce_{0}'.format(collection)]
-            log.debug('smartos.vm_present::enforce_{0} = {1}'.format(collection, enforce))
+            log.debug('smartos.vm_present::enforce_%s = %s', collection, enforce)
 
             # process add and update for collection
             if collection in vmconfig['state'] and vmconfig['state'][collection] is not None:
@@ -742,9 +752,10 @@ def vm_present(name, vmconfig, config=None):
                             ret['changes'][vmconfig['state']['hostname']] = {}
                         ret['changes'][vmconfig['state']['hostname']]['image_uuid'] = vmconfig['reprovision_uuid']
                 else:
-                    log.warning('smartos.vm_present::{0}::reprovision - image_uuid in state does not match current, reprovision not allowed'.format(
-                        name
-                    ))
+                    log.warning('smartos.vm_present::%s::reprovision - '
+                                'image_uuid in state does not match current, '
+                                'reprovision not allowed',
+                                name)
         else:
             ret['comment'] = 'vm {0} failed to be updated'.format(vmconfig['state']['hostname'])
             if not isinstance(rret, (bool)) and 'Error' in rret:
