@@ -23,14 +23,15 @@ as either absent or present
     testuser:
       user.absent
 '''
-
-# Import python libs
-from __future__ import absolute_import
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import logging
 
-# Import salt libs
-import salt.utils
+# Import Salt libs
+import salt.utils.dateutils
+import salt.utils.platform
+import salt.utils.user
 from salt.utils.locales import sdecode, sdecode_if_string
 
 # Import 3rd-party libs
@@ -67,7 +68,7 @@ def _changes(name,
              workphone='',
              homephone='',
              loginclass=None,
-             date=0,
+             date=None,
              mindays=0,
              maxdays=999999,
              inactdays=0,
@@ -134,7 +135,7 @@ def _changes(name,
                     change['passwd'] = password
         if empty_password and lshad['passwd'] != '':
             change['empty_password'] = True
-        if date and date is not 0 and lshad['lstchg'] != date:
+        if date is not None and lshad['lstchg'] != date:
             change['date'] = date
         if mindays and mindays is not 0 and lshad['min'] != mindays:
             change['mindays'] = mindays
@@ -146,8 +147,8 @@ def _changes(name,
             change['warndays'] = warndays
         if expire and lshad['expire'] != expire:
             change['expire'] = expire
-    elif 'shadow.info' in __salt__ and salt.utils.is_windows():
-        if expire and expire is not -1 and salt.utils.date_format(lshad['expire']) != salt.utils.date_format(expire):
+    elif 'shadow.info' in __salt__ and salt.utils.platform.is_windows():
+        if expire and expire is not -1 and salt.utils.dateutils.strftime(lshad['expire']) != salt.utils.dateutils.strftime(expire):
             change['expire'] = expire
 
     # GECOS fields
@@ -240,7 +241,8 @@ def present(name,
 
     gid_from_name
         If True, the default group id will be set to the id of the group with
-        the same name as the user, Default is ``False``.
+        the same name as the user. If the group does not exist the state will
+        fail. Default is ``False``.
 
     groups
         A list of groups to assign the user to, pass a list object. If a group
@@ -441,8 +443,10 @@ def present(name,
                              if __salt__['group.info'](x)]
         for missing_optgroup in [x for x in optional_groups
                                  if x not in present_optgroups]:
-            log.debug('Optional group "{0}" for user "{1}" is not '
-                      'present'.format(missing_optgroup, name))
+            log.debug(
+                'Optional group "%s" for user "%s" is not present',
+                missing_optgroup, name
+            )
     else:
         present_optgroups = None
 
@@ -450,11 +454,17 @@ def present(name,
     # "optional_groups" lists.
     if groups and optional_groups:
         for isected in set(groups).intersection(optional_groups):
-            log.warning('Group "{0}" specified in both groups and '
-                        'optional_groups for user {1}'.format(isected, name))
+            log.warning(
+                'Group "%s" specified in both groups and optional_groups '
+                'for user %s', isected, name
+            )
 
     if gid_from_name:
         gid = __salt__['file.group_to_gid'](name)
+        if gid == '':
+            ret['comment'] = 'Default group with name "{0}" is not present'.format(name)
+            ret['result'] = False
+            return ret
 
     changes = _changes(name,
                        uid,
@@ -494,7 +504,7 @@ def present(name,
                     val = 'XXX-REDACTED-XXX'
                 elif key == 'group' and not remove_groups:
                     key = 'ensure groups'
-                ret['comment'] += u'{0}: {1}\n'.format(key, val)
+                ret['comment'] += '{0}: {1}\n'.format(key, val)
             return ret
         # The user is present
         if 'shadow.info' in __salt__:
@@ -633,7 +643,7 @@ def present(name,
 
         # Setup params specific to Linux and Windows to be passed to the
         # add.user function
-        if not salt.utils.is_windows():
+        if not salt.utils.platform.is_windows():
             params = {'name': name,
                       'uid': uid,
                       'gid': gid,
@@ -667,8 +677,8 @@ def present(name,
                 # pwd incorrectly reports presence of home
                 ret['changes']['home'] = ''
             if 'shadow.info' in __salt__ \
-                and not salt.utils.is_windows()\
-                and not salt.utils.is_darwin():
+                and not salt.utils.platform.is_windows() \
+                and not salt.utils.platform.is_darwin():
                 if password and not empty_password:
                     __salt__['shadow.set_password'](name, password)
                     spost = __salt__['shadow.info'](name)
@@ -686,7 +696,7 @@ def present(name,
                                          'empty password'.format(name)
                         ret['result'] = False
                     ret['changes']['password'] = ''
-                if date:
+                if date is not None:
                     __salt__['shadow.set_date'](name, date)
                     spost = __salt__['shadow.info'](name)
                     if spost['lstchg'] != date:
@@ -740,7 +750,7 @@ def present(name,
                                          ' {1}'.format(name, expire)
                         ret['result'] = False
                     ret['changes']['expire'] = expire
-            elif salt.utils.is_windows():
+            elif salt.utils.platform.is_windows():
                 if password and not empty_password:
                     if not __salt__['user.setpassword'](name, password):
                         ret['comment'] = 'User {0} created but failed to set' \
@@ -751,13 +761,13 @@ def present(name,
                 if expire:
                     __salt__['shadow.set_expire'](name, expire)
                     spost = __salt__['shadow.info'](name)
-                    if salt.utils.date_format(spost['expire']) != salt.utils.date_format(expire):
+                    if salt.utils.dateutils.strftime(spost['expire']) != salt.utils.dateutils.strftime(expire):
                         ret['comment'] = 'User {0} created but failed to set' \
                                          ' expire days to' \
                                          ' {1}'.format(name, expire)
                         ret['result'] = False
                     ret['changes']['expiration_date'] = spost['expire']
-            elif salt.utils.is_darwin() and password and not empty_password:
+            elif salt.utils.platform.is_darwin() and password and not empty_password:
                 if not __salt__['shadow.set_password'](name, password):
                     ret['comment'] = 'User {0} created but failed to set' \
                                      ' password to' \
@@ -799,7 +809,7 @@ def absent(name, purge=False, force=False):
             ret['result'] = None
             ret['comment'] = 'User {0} set for removal'.format(name)
             return ret
-        beforegroups = set(salt.utils.get_group_list(name))
+        beforegroups = set(salt.utils.user.get_group_list(name))
         ret['result'] = __salt__['user.delete'](name, purge, force)
         aftergroups = set([g for g in beforegroups if __salt__['group.info'](g)])
         if ret['result']:
