@@ -4,19 +4,22 @@ Setup of Python virtualenv sandboxes.
 
 .. versionadded:: 0.17.0
 '''
-from __future__ import absolute_import
 
-# Import python libs
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 import os
 
-# Import salt libs
+# Import Salt libs
 import salt.version
-import salt.utils
-
+import salt.utils.functools
+import salt.utils.platform
+import salt.utils.versions
 from salt.exceptions import CommandExecutionError, CommandNotFoundError
 
+# Import 3rd-party libs
 from salt.ext import six
+
 log = logging.getLogger(__name__)
 
 # Define the module's virtual name
@@ -56,13 +59,18 @@ def managed(name,
             pip_upgrade=False,
             pip_pkgs=None,
             pip_no_cache_dir=False,
-            pip_cache_dir=None):
+            pip_cache_dir=None,
+            process_dependency_links=False):
     '''
     Create a virtualenv and optionally manage it with pip
 
     name
         Path to the virtualenv.
 
+    venv_bin: virtualenv
+        The name (and optionally path) of the virtualenv command. This can also
+        be set globally in the minion config file as ``virtualenv.venv_bin``.
+
     requirements: None
         Path to a pip requirements file. If the path begins with ``salt://``
         the file will be transferred from the master file server.
@@ -109,6 +117,11 @@ def managed(name,
         As an alternative to `requirements`, pass a list of pip packages that
         should be installed.
 
+    process_dependency_links: False
+        Run pip install with the --process_dependency_links flag.
+
+        .. versionadded:: 2017.7.0
+
     Also accepts any kwargs that the virtualenv module will. However, some
     kwargs, such as the ``pip`` option, require ``- distribute: True``.
 
@@ -118,6 +131,8 @@ def managed(name,
           virtualenv.managed:
             - system_site_packages: False
             - requirements: salt://REQUIREMENTS.txt
+            - env_vars:
+                PATH_VAR: '/usr/local/bin/'
     '''
     ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
 
@@ -126,7 +141,7 @@ def managed(name,
         ret['comment'] = 'Virtualenv was not detected on this system'
         return ret
 
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         venv_py = os.path.join(name, 'Scripts', 'python.exe')
     else:
         venv_py = os.path.join(name, 'bin', 'python')
@@ -179,7 +194,7 @@ def managed(name,
 
     if not venv_exists or (venv_exists and clear):
         try:
-            _ret = __salt__['virtualenv.create'](
+            venv_ret = __salt__['virtualenv.create'](
                 name,
                 venv_bin=venv_bin,
                 system_site_packages=system_site_packages,
@@ -197,9 +212,9 @@ def managed(name,
             ret['comment'] = 'Failed to create virtualenv: {0}'.format(err)
             return ret
 
-        if _ret['retcode'] != 0:
+        if venv_ret['retcode'] != 0:
             ret['result'] = False
-            ret['comment'] = _ret['stdout'] + _ret['stderr']
+            ret['comment'] = venv_ret['stdout'] + venv_ret['stderr']
             return ret
 
         ret['result'] = True
@@ -217,7 +232,7 @@ def managed(name,
     if use_wheel:
         min_version = '1.4'
         cur_version = __salt__['pip.version'](bin_env=name)
-        if not salt.utils.compare_versions(ver1=cur_version, oper='>=',
+        if not salt.utils.versions.compare(ver1=cur_version, oper='>=',
                                            ver2=min_version):
             ret['result'] = False
             ret['comment'] = ('The \'use_wheel\' option is only supported in '
@@ -228,7 +243,7 @@ def managed(name,
     if no_use_wheel:
         min_version = '1.4'
         cur_version = __salt__['pip.version'](bin_env=name)
-        if not salt.utils.compare_versions(ver1=cur_version, oper='>=',
+        if not salt.utils.versions.compare(ver1=cur_version, oper='>=',
                                            ver2=min_version):
             ret['result'] = False
             ret['comment'] = ('The \'no_use_wheel\' option is only supported '
@@ -260,9 +275,10 @@ def managed(name,
             if req_canary != os.path.abspath(req_canary):
                 cwd = os.path.dirname(os.path.abspath(req_canary))
 
-        _ret = __salt__['pip.install'](
+        pip_ret = __salt__['pip.install'](
             pkgs=pip_pkgs,
             requirements=requirements,
+            process_dependency_links=process_dependency_links,
             bin_env=name,
             use_wheel=use_wheel,
             no_use_wheel=no_use_wheel,
@@ -284,11 +300,11 @@ def managed(name,
             no_cache_dir=pip_no_cache_dir,
             cache_dir=pip_cache_dir
         )
-        ret['result'] &= _ret['retcode'] == 0
-        if _ret['retcode'] > 0:
+        ret['result'] &= pip_ret['retcode'] == 0
+        if pip_ret['retcode'] > 0:
             ret['comment'] = '{0}\n{1}\n{2}'.format(ret['comment'],
-                                                    _ret['stdout'],
-                                                    _ret['stderr'])
+                                                    pip_ret['stdout'],
+                                                    pip_ret['stderr'])
 
         after = set(__salt__['pip.freeze'](bin_env=name))
 
@@ -301,4 +317,4 @@ def managed(name,
                 'old': old if old else ''}
     return ret
 
-manage = salt.utils.alias_function(managed, 'manage')
+manage = salt.utils.functools.alias_function(managed, 'manage')
