@@ -29,7 +29,7 @@ Management zfs datasets
       zfs.snapshot_present
 
     moka_origin:
-      zfs.hold_present
+      zfs.hold_present:
         - snapshot: test/shares/yuki@frozen
 
     test/shares/moka:
@@ -40,11 +40,14 @@ Management zfs datasets
       zfs.snapshot_absent
 
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import Python libs
 import logging
-from time import strftime, strptime, gmtime
+from time import strftime, strptime, localtime
+
+# Import Salt libs
+from salt.modules.zfs import _conform_value
 
 log = logging.getLogger(__name__)
 
@@ -61,7 +64,7 @@ def __virtual__():
     else:
         return (
             False,
-            '{0} state module can only be loaded on illumos, Solaris, SmartOS, FreeBSD, ...'.format(
+            '{0} state module can only be loaded on illumos, Solaris, SmartOS, FreeBSD, Linux, ...'.format(
                 __virtualname__
             )
         )
@@ -87,8 +90,10 @@ def _absent(name, dataset_type, force=False, recursive=False):
            'result': True,
            'comment': ''}
 
-    log.debug('zfs.{0}_absent::{1}::config::force = {2}'.format(dataset_type, name, force))
-    log.debug('zfs.{0}_absent::{1}::config::recursive = {2}'.format(dataset_type, name, recursive))
+    log.debug('zfs.%s_absent::%s::config::force = %s',
+              dataset_type, name, force)
+    log.debug('zfs.%s_absent::%s::config::recursive = %s',
+              dataset_type, name, recursive)
 
     # check name and type
     if dataset_type not in ['filesystem', 'volume', 'snapshot', 'bookmark']:
@@ -110,8 +115,7 @@ def _absent(name, dataset_type, force=False, recursive=False):
 
     # check if dataset exists
     if ret['result']:
-        dataset = name if '#' not in name else None  # work around bookmark oddities
-        if name in __salt__['zfs.list'](dataset, **{'type': dataset_type}):  # we need to destroy it
+        if __salt__['zfs.exists'](name, **{'type': dataset_type}):  # we need to destroy it
             result = {name: 'destroyed'}
             if not __opts__['test']:
                 result = __salt__['zfs.destroy'](name, **{'force': force, 'recursive': recursive})
@@ -220,42 +224,45 @@ def hold_absent(name, snapshot, recursive=False):
            'result': True,
            'comment': ''}
 
-    log.debug('zfs.hold_absent::{0}::config::snapshot = {1}'.format(name, snapshot))
-    log.debug('zfs.hold_absent::{0}::config::recursive = {1}'.format(name, recursive))
+    log.debug('zfs.hold_absent::%s::config::snapshot = %s',
+              name, snapshot)
+    log.debug('zfs.hold_absent::%s::config::recursive = %s',
+              name, recursive)
 
     # check name and type
     if '@' not in snapshot:
         ret['result'] = False
         ret['comment'] = 'invalid snapshot name: {0}'.format(snapshot)
+        return ret
 
     if '@' in name or '#' in name:
         ret['result'] = False
         ret['comment'] = 'invalid tag name: {0}'.format(name)
+        return ret
 
-    if ret['result']:
-        result = __salt__['zfs.holds'](snapshot)
-        if snapshot not in result:
+    result = __salt__['zfs.holds'](snapshot)
+    if snapshot not in result:
+        ret['result'] = False
+        ret['comment'] = '{0} is probably not a snapshot'.format(snapshot)
+    else:
+        if snapshot in result[snapshot]:
             ret['result'] = False
-            ret['comment'] = '{0} is probably not a snapshot'.format(snapshot)
+            ret['comment'] = result[snapshot]
+        elif result[snapshot] == 'no holds' or name not in result[snapshot]:
+            ret['comment'] = 'hold {0} not present'.format(name)
         else:
-            if snapshot in result[snapshot]:
-                ret['result'] = False
-                ret['comment'] = result[snapshot]
-            elif result[snapshot] == 'no holds' or name not in result[snapshot]:
-                ret['comment'] = 'hold {0} not present'.format(name)
-            else:
-                result = {snapshot: {name: 'released'}}
-                if not __opts__['test']:
-                    result = __salt__['zfs.release'](name, snapshot, **{'recursive': recursive})
+            result = {snapshot: {name: 'released'}}
+            if not __opts__['test']:
+                result = __salt__['zfs.release'](name, snapshot, **{'recursive': recursive})
 
-                ret['result'] = snapshot in result and name in result[snapshot]
-                if ret['result']:
-                    ret['changes'] = result[snapshot]
-                    ret['comment'] = 'hold {0} released'.format(name)
-                else:
-                    ret['comment'] = 'failed to release {0}'.format(name)
-                    if snapshot in result:
-                        ret['comment'] = result[snapshot]
+            ret['result'] = snapshot in result and name in result[snapshot]
+            if ret['result']:
+                ret['changes'] = result[snapshot]
+                ret['comment'] = 'hold {0} released'.format(name)
+            else:
+                ret['comment'] = 'failed to release {0}'.format(name)
+                if snapshot in result:
+                    ret['comment'] = result[snapshot]
 
     return ret
 
@@ -276,42 +283,45 @@ def hold_present(name, snapshot, recursive=False):
            'result': True,
            'comment': ''}
 
-    log.debug('zfs.hold_present::{0}::config::snapshot = {1}'.format(name, snapshot))
-    log.debug('zfs.hold_present::{0}::config::recursive = {1}'.format(name, recursive))
+    log.debug('zfs.hold_present::%s::config::snapshot = %s',
+              name, snapshot)
+    log.debug('zfs.hold_present::%s::config::recursive = %s',
+              name, recursive)
 
     # check name and type
     if '@' not in snapshot:
         ret['result'] = False
         ret['comment'] = 'invalid snapshot name: {0}'.format(snapshot)
+        return ret
 
     if '@' in name or '#' in name:
         ret['result'] = False
         ret['comment'] = 'invalid tag name: {0}'.format(name)
+        return ret
 
-    if ret['result']:
-        result = __salt__['zfs.holds'](snapshot)
-        if snapshot not in result:
+    result = __salt__['zfs.holds'](snapshot)
+    if snapshot not in result:
+        ret['result'] = False
+        ret['comment'] = '{0} is probably not a snapshot'.format(snapshot)
+    else:
+        if snapshot in result[snapshot]:
             ret['result'] = False
-            ret['comment'] = '{0} is probably not a snapshot'.format(snapshot)
-        else:
-            if snapshot in result[snapshot]:
-                ret['result'] = False
-                ret['comment'] = result[snapshot]
-            elif result[snapshot] == 'no holds' or name not in result[snapshot]:  # add hold
-                result = {snapshot: {name: 'held'}}
-                if not __opts__['test']:
-                    result = __salt__['zfs.hold'](name, snapshot, **{'recursive': recursive})
+            ret['comment'] = result[snapshot]
+        elif result[snapshot] == 'no holds' or name not in result[snapshot]:  # add hold
+            result = {snapshot: {name: 'held'}}
+            if not __opts__['test']:
+                result = __salt__['zfs.hold'](name, snapshot, **{'recursive': recursive})
 
-                ret['result'] = snapshot in result and name in result[snapshot]
-                if ret['result']:
-                    ret['changes'] = result[snapshot]
-                    ret['comment'] = 'hold {0} added to {1}'.format(name, snapshot)
-                else:
-                    ret['comment'] = 'failed to add hold {0}'.format(name)
-                    if snapshot in result:
-                        ret['comment'] = result[snapshot]
-            else:  # hold present
-                ret['comment'] = 'hold already exists'
+            ret['result'] = snapshot in result and name in result[snapshot]
+            if ret['result']:
+                ret['changes'] = result[snapshot]
+                ret['comment'] = 'hold {0} added to {1}'.format(name, snapshot)
+            else:
+                ret['comment'] = 'failed to add hold {0}'.format(name)
+                if snapshot in result:
+                    ret['comment'] = result[snapshot]
+        else:  # hold present
+            ret['comment'] = 'hold already exists'
 
     return ret
 
@@ -348,77 +358,84 @@ def filesystem_present(name, create_parent=False, properties=None, cloned_from=N
     if not properties:
         properties = {}
 
-    log.debug('zfs.filesystem_present::{0}::config::create_parent = {1}'.format(name, create_parent))
-    log.debug('zfs.filesystem_present::{0}::config::cloned_from = {1}'.format(name, cloned_from))
-    log.debug('zfs.filesystem_present::{0}::config::properties = {1}'.format(name, properties))
+    log.debug('zfs.filesystem_present::%s::config::create_parent = %s',
+              name, create_parent)
+    log.debug('zfs.filesystem_present::%s::config::cloned_from = %s',
+              name, cloned_from)
+    log.debug('zfs.filesystem_present::%s::config::properties = %s',
+              name, properties)
 
-    for prop in properties.keys():  # salt breaks the on/off/yes/no properties
-        if isinstance(properties[prop], bool):
-            properties[prop] = 'on' if properties[prop] else 'off'
+    for prop in properties:
+        properties[prop] = _conform_value(properties[prop], True)
 
     if '@' in name or '#' in name:
         ret['result'] = False
         ret['comment'] = 'invalid filesystem or volume name: {0}'.format(name)
+        return ret
 
     if cloned_from:
-        cloned_parent = cloned_from[:cloned_from.index('@')]
         if '@' not in cloned_from:
             ret['result'] = False
             ret['comment'] = '{0} is not a snapshot'.format(cloned_from)
-        elif cloned_from not in __salt__['zfs.list'](cloned_from, **{'type': 'snapshot'}):
+            return ret
+
+        if not __salt__['zfs.exists'](cloned_from, **{'type': 'snapshot'}):
             ret['result'] = False
             ret['comment'] = 'snapshot {0} does not exist'.format(cloned_from)
-        elif cloned_parent not in __salt__['zfs.list'](cloned_parent, **{'type': 'filesystem'}):
+            return ret
+
+        cloned_parent = cloned_from[:cloned_from.index('@')]
+        if not __salt__['zfs.exists'](cloned_parent, **{'type': 'filesystem'}):
             ret['result'] = False
             ret['comment'] = 'snapshot {0} is not from a filesystem'.format(cloned_from)
+            return ret
 
-    if ret['result']:
-        if name in __salt__['zfs.list'](name, **{'type': 'filesystem'}):  # update properties if needed
-            result = {}
-            if len(properties) > 0:
-                result = __salt__['zfs.get'](name, **{'properties': ','.join(properties.keys()), 'fields': 'value', 'depth': 1})
+    if __salt__['zfs.exists'](name, **{'type': 'filesystem'}):  # update properties if needed
+        result = {}
+        if len(properties) > 0:
+            result = __salt__['zfs.get'](name, **{'properties': ','.join(properties.keys()), 'fields': 'value', 'depth': 1, 'parsable': True})
 
-            for prop in properties.keys():
-                if properties[prop] != result[name][prop]['value']:
-                    if name not in ret['changes']:
-                        ret['changes'][name] = {}
-                    ret['changes'][name][prop] = properties[prop]
+        for prop in properties:
+            if properties[prop] != result[name][prop]['value']:
+                if name not in ret['changes']:
+                    ret['changes'][name] = {}
+                ret['changes'][name][prop] = properties[prop]
 
-            if len(ret['changes']) > 0:
-                if not __opts__['test']:
-                    result = __salt__['zfs.set'](name, **ret['changes'][name])
-                    if name not in result:
-                        ret['result'] = False
-                    else:
-                        for prop in result[name].keys():
-                            if result[name][prop] != 'set':
-                                ret['result'] = False
-
-                if ret['result']:
-                    ret['comment'] = 'filesystem {0} was updated'.format(name)
-                else:
-                    ret['changes'] = {}
-                    ret['comment'] = 'filesystem {0} failed to be updated'.format(name)
-            else:
-                ret['comment'] = 'filesystem {0} is up to date'.format(name)
-        else:  # create filesystem
-            result = {name: 'created'}
+        if len(ret['changes']) > 0:
             if not __opts__['test']:
-                if not cloned_from:
-                    result = __salt__['zfs.create'](name, **{'create_parent': create_parent, 'properties': properties})
+                result = __salt__['zfs.set'](name, **ret['changes'][name])
+                if name not in result:
+                    ret['result'] = False
                 else:
-                    result = __salt__['zfs.clone'](cloned_from, name, **{'create_parent': create_parent, 'properties': properties})
+                    for prop in result[name]:
+                        if result[name][prop] != 'set':
+                            ret['result'] = False
 
-            ret['result'] = name in result
             if ret['result']:
-                ret['result'] = result[name] == 'created' or result[name].startswith('cloned')
-            if ret['result']:
-                ret['changes'][name] = properties if len(properties) > 0 else result[name]
-                ret['comment'] = 'filesystem {0} was created'.format(name)
+                ret['comment'] = 'filesystem {0} was updated'.format(name)
             else:
-                ret['comment'] = 'failed to create filesystem {0}'.format(name)
-                if name in result:
-                    ret['comment'] = result[name]
+                ret['changes'] = {}
+                ret['comment'] = 'filesystem {0} failed to be updated'.format(name)
+        else:
+            ret['comment'] = 'filesystem {0} is up to date'.format(name)
+    else:  # create filesystem
+        result = {name: 'created'}
+        if not __opts__['test']:
+            if not cloned_from:
+                result = __salt__['zfs.create'](name, **{'create_parent': create_parent, 'properties': properties})
+            else:
+                result = __salt__['zfs.clone'](cloned_from, name, **{'create_parent': create_parent, 'properties': properties})
+
+        ret['result'] = name in result
+        if ret['result']:
+            ret['result'] = result[name] == 'created' or result[name].startswith('cloned')
+        if ret['result']:
+            ret['changes'][name] = properties if len(properties) > 0 else result[name]
+            ret['comment'] = 'filesystem {0} was created'.format(name)
+        else:
+            ret['comment'] = 'failed to create filesystem {0}'.format(name)
+            if name in result:
+                ret['comment'] = result[name]
     return ret
 
 
@@ -464,83 +481,93 @@ def volume_present(name, volume_size, sparse=False, create_parent=False, propert
     if not properties:
         properties = {}
 
-    log.debug('zfs.volume_present::{0}::config::volume_size = {1}'.format(name, volume_size))
-    log.debug('zfs.volume_present::{0}::config::sparse = {1}'.format(name, sparse))
-    log.debug('zfs.volume_present::{0}::config::create_parent = {1}'.format(name, create_parent))
-    log.debug('zfs.volume_present::{0}::config::cloned_from = {1}'.format(name, cloned_from))
-    log.debug('zfs.volume_present::{0}::config::properties = {1}'.format(name, properties))
+    log.debug('zfs.volume_present::%s::config::volume_size = %s',
+              name, volume_size)
+    log.debug('zfs.volume_present::%s::config::sparse = %s',
+              name, sparse)
+    log.debug('zfs.volume_present::%s::config::create_parent = %s',
+              name, create_parent)
+    log.debug('zfs.volume_present::%s::config::cloned_from = %s',
+              name, cloned_from)
+    log.debug('zfs.volume_present::%s::config::properties = %s',
+              name, properties)
 
-    for prop in properties.keys():  # salt breaks the on/off/yes/no properties
-        if isinstance(properties[prop], bool):
-            properties[prop] = 'on' if properties[prop] else 'off'
+    volume_size = _conform_value(volume_size, True)
+    for prop in properties:
+        properties[prop] = _conform_value(properties[prop], True)
 
     if '@' in name or '#' in name:
         ret['result'] = False
         ret['comment'] = 'invalid filesystem or volume name: {0}'.format(name)
+        return ret
 
     if cloned_from:
-        cloned_parent = cloned_from[:cloned_from.index('@')]
         if '@' not in cloned_from:
             ret['result'] = False
             ret['comment'] = '{0} is not a snapshot'.format(cloned_from)
-        elif cloned_from not in __salt__['zfs.list'](cloned_from, **{'type': 'snapshot'}):
+            return ret
+
+        if not __salt__['zfs.exists'](cloned_from, **{'type': 'snapshot'}):
             ret['result'] = False
             ret['comment'] = 'snapshot {0} does not exist'.format(cloned_from)
-        elif cloned_parent not in __salt__['zfs.list'](cloned_parent, **{'type': 'volume'}):
+            return ret
+
+        cloned_parent = cloned_from[:cloned_from.index('@')]
+        if not __salt__['zfs.exists'](cloned_parent, **{'type': 'volume'}):
             ret['result'] = False
             ret['comment'] = 'snapshot {0} is not from a volume'.format(cloned_from)
+            return ret
 
-    if ret['result']:
-        if name in __salt__['zfs.list'](name, **{'type': 'volume'}):  # update properties if needed
-            properties['volsize'] = volume_size  # add volume_size to properties
-            result = __salt__['zfs.get'](name, **{'properties': ','.join(properties.keys()), 'fields': 'value', 'depth': 1})
+    if __salt__['zfs.exists'](name, **{'type': 'volume'}):  # update properties if needed
+        properties['volsize'] = volume_size  # add volume_size to properties
+        result = __salt__['zfs.get'](name, **{'properties': ','.join(properties.keys()), 'fields': 'value', 'depth': 1, 'parsable': True})
 
-            for prop in properties.keys():
-                if properties[prop] != result[name][prop]['value']:
-                    if name not in ret['changes']:
-                        ret['changes'][name] = {}
-                    ret['changes'][name][prop] = properties[prop]
+        for prop in properties:
+            if properties[prop] != result[name][prop]['value']:
+                if name not in ret['changes']:
+                    ret['changes'][name] = {}
+                ret['changes'][name][prop] = properties[prop]
 
-            if len(ret['changes']) > 0:
-                if not __opts__['test']:
-                    result = __salt__['zfs.set'](name, **ret['changes'][name])
-                    if name not in result:
-                        ret['result'] = False
-                    else:
-                        for prop in result[name].keys():
-                            if result[name][prop] != 'set':
-                                ret['result'] = False
-
-                if ret['result']:
-                    ret['comment'] = 'volume {0} was updated'.format(name)
-                else:
-                    ret['changes'] = {}
-                    ret['comment'] = 'volume {0} failed to be updated'.format(name)
-            else:
-                ret['comment'] = 'volume {0} is up to date'.format(name)
-        else:  # create volume
-            result = {name: 'created'}
+        if len(ret['changes']) > 0:
             if not __opts__['test']:
-                if not cloned_from:
-                    result = __salt__['zfs.create'](name, **{
-                        'volume_size': volume_size,
-                        'sparse': sparse,
-                        'create_parent': create_parent,
-                        'properties': properties
-                    })
+                result = __salt__['zfs.set'](name, **ret['changes'][name])
+                if name not in result:
+                    ret['result'] = False
                 else:
-                    result = __salt__['zfs.clone'](cloned_from, name, **{'create_parent': create_parent, 'properties': properties})
+                    for prop in result[name]:
+                        if result[name][prop] != 'set':
+                            ret['result'] = False
 
-            ret['result'] = name in result
             if ret['result']:
-                ret['result'] = result[name] == 'created' or result[name].startswith('cloned')
-            if ret['result']:
-                ret['changes'][name] = properties if len(properties) > 0 else result[name]
-                ret['comment'] = 'volume {0} was created'.format(name)
+                ret['comment'] = 'volume {0} was updated'.format(name)
             else:
-                ret['comment'] = 'failed to create volume {0}'.format(name)
-                if name in result:
-                    ret['comment'] = result[name]
+                ret['changes'] = {}
+                ret['comment'] = 'volume {0} failed to be updated'.format(name)
+        else:
+            ret['comment'] = 'volume {0} is up to date'.format(name)
+    else:  # create volume
+        result = {name: 'created'}
+        if not __opts__['test']:
+            if not cloned_from:
+                result = __salt__['zfs.create'](name, **{
+                    'volume_size': volume_size,
+                    'sparse': sparse,
+                    'create_parent': create_parent,
+                    'properties': properties
+                })
+            else:
+                result = __salt__['zfs.clone'](cloned_from, name, **{'create_parent': create_parent, 'properties': properties})
+
+        ret['result'] = name in result
+        if ret['result']:
+            ret['result'] = result[name] == 'created' or result[name].startswith('cloned')
+        if ret['result']:
+            ret['changes'][name] = properties if len(properties) > 0 else result[name]
+            ret['comment'] = 'volume {0} was created'.format(name)
+        else:
+            ret['comment'] = 'failed to create volume {0}'.format(name)
+            if name in result:
+                ret['comment'] = result[name]
     return ret
 
 
@@ -559,11 +586,13 @@ def bookmark_present(name, snapshot):
            'result': True,
            'comment': ''}
 
-    log.debug('zfs.bookmark_present::{0}::config::snapshot = {1}'.format(name, snapshot))
+    log.debug('zfs.bookmark_present::%s::config::snapshot = %s',
+              name, snapshot)
 
     if '@' not in snapshot:
         ret['result'] = False
         ret['comment'] = '{0} is not a snapshot'.format(snapshot)
+        return ret
 
     if '#' not in name:
         if '/' not in name:
@@ -571,21 +600,21 @@ def bookmark_present(name, snapshot):
         else:
             ret['result'] = False
             ret['comment'] = '{0} is not a bookmark'.format(name)
+            return ret
 
-    if ret['result']:
-        if name in __salt__['zfs.list'](**{'type': 'bookmark'}):
-            ret['comment'] = 'bookmark already exists'
-        else:  # create bookmark
-            result = {snapshot: 'bookmarked'}
-            if not __opts__['test']:
-                result = __salt__['zfs.bookmark'](snapshot, name)
+    if __salt__['zfs.exists'](name, **{'type': 'bookmark'}):
+        ret['comment'] = 'bookmark already exists'
+    else:  # create bookmark
+        result = {snapshot: 'bookmarked'}
+        if not __opts__['test']:
+            result = __salt__['zfs.bookmark'](snapshot, name)
 
-            ret['result'] = snapshot in result and result[snapshot].startswith('bookmarked')
-            if ret['result']:
-                ret['changes'] = result
-                ret['comment'] = 'snapshot {0} was bookmarked as {1}'.format(snapshot, name)
-            else:
-                ret['comment'] = 'failed to create bookmark {0}'.format(name)
+        ret['result'] = snapshot in result and result[snapshot].startswith('bookmarked')
+        if ret['result']:
+            ret['changes'] = result
+            ret['comment'] = 'snapshot {0} was bookmarked as {1}'.format(snapshot, name)
+        else:
+            ret['comment'] = 'failed to create bookmark {0}'.format(name)
     return ret
 
 
@@ -613,33 +642,34 @@ def snapshot_present(name, recursive=False, properties=None):
     if not properties:
         properties = {}
 
-    log.debug('zfs.snapshot_present::{0}::config::recursive = {1}'.format(name, recursive))
-    log.debug('zfs.snapshot_present::{0}::config::properties = {1}'.format(name, properties))
+    log.debug('zfs.snapshot_present::%s::config::recursive = %s',
+              name, recursive)
+    log.debug('zfs.snapshot_present::%s::config::properties = %s',
+              name, properties)
 
-    for prop in properties.keys():  # salt breaks the on/off/yes/no properties
-        if isinstance(properties[prop], bool):
-            properties[prop] = 'on' if properties[prop] else 'off'
+    for prop in properties:
+        properties[prop] = _conform_value(properties[prop], True)
 
     if '@' not in name:
         ret['result'] = False
         ret['comment'] = 'invalid snapshot name: {0}'.format(name)
+        return ret
 
-    if ret['result']:
-        if name in __salt__['zfs.list'](name, **{'type': 'snapshot'}):  # we are all good
-            ret['comment'] = 'snapshot already exists'
-        else:  # create snapshot
-            result = {name: 'snapshotted'}
-            if not __opts__['test']:
-                result = __salt__['zfs.snapshot'](name, **{'recursive': recursive, 'properties': properties})
+    if __salt__['zfs.exists'](name, **{'type': 'snapshot'}):  # we are all good
+        ret['comment'] = 'snapshot already exists'
+    else:  # create snapshot
+        result = {name: 'snapshotted'}
+        if not __opts__['test']:
+            result = __salt__['zfs.snapshot'](name, **{'recursive': recursive, 'properties': properties})
 
-            ret['result'] = name in result and result[name] == 'snapshotted'
-            if ret['result']:
-                ret['changes'][name] = properties if len(properties) > 0 else result[name]
-                ret['comment'] = 'snapshot {0} was created'.format(name)
-            else:
-                ret['comment'] = 'failed to create snapshot {0}'.format(name)
-                if name in result:
-                    ret['comment'] = result[name]
+        ret['result'] = name in result and result[name] == 'snapshotted'
+        if ret['result']:
+            ret['changes'][name] = properties if len(properties) > 0 else result[name]
+            ret['comment'] = 'snapshot {0} was created'.format(name)
+        else:
+            ret['comment'] = 'failed to create snapshot {0}'.format(name)
+            if name in result:
+                ret['comment'] = result[name]
 
     return ret
 
@@ -665,32 +695,32 @@ def promoted(name):
     if '@' in name or '#' in name:
         ret['result'] = False
         ret['comment'] = 'invalid filesystem or volume name: {0}'.format(name)
+        return ret
 
-    if ret['result']:
-        if name in __salt__['zfs.list'](name):
-            origin = '-'
+    if __salt__['zfs.exists'](name):
+        origin = '-'
+        if not __opts__['test']:
+            origin = __salt__['zfs.get'](name, **{'properties': 'origin', 'fields': 'value', 'parsable': True})[name]['origin']['value']
+
+        if origin == '-':
+            ret['comment'] = '{0} already promoted'.format(name)
+        else:
+            result = {name: 'promoted'}
             if not __opts__['test']:
-                origin = __salt__['zfs.get'](name, **{'properties': 'origin', 'fields': 'value'})[name]['origin']['value']
+                result = __salt__['zfs.promote'](name)
 
-            if origin == '-':
-                ret['comment'] = '{0} already promoted'.format(name)
+            ret['result'] = name in result and result[name] == 'promoted'
+            ret['changes'] = result if ret['result'] else {}
+            if ret['result']:
+                ret['comment'] = '{0} was promoted'.format(name)
             else:
-                result = {name: 'promoted'}
-                if not __opts__['test']:
-                    result = __salt__['zfs.promote'](name)
+                ret['comment'] = 'failed to promote {0}'.format(name)
+                if name in result:
+                    ret['comment'] = result[name]
 
-                ret['result'] = name in result and result[name] == 'promoted'
-                ret['changes'] = result if ret['result'] else {}
-                if ret['result']:
-                    ret['comment'] = '{0} was promoted'.format(name)
-                else:
-                    ret['comment'] = 'failed to promote {0}'.format(name)
-                    if name in result:
-                        ret['comment'] = result[name]
-
-        else:  # we don't have the dataset
-            ret['result'] = False
-            ret['comment'] = 'dataset {0} does not exist'.format(name)
+    else:  # we don't have the dataset
+        ret['result'] = False
+        ret['comment'] = 'dataset {0} does not exist'.format(name)
 
     return ret
 
@@ -717,6 +747,10 @@ def scheduled_snapshot(name, prefix, recursive=True, schedule=None):
         a schedule must be setup to automatically run the state. this means that if
         you run the state daily the hourly snapshot will only be made once per day!
 
+    .. versionchanged:: 2018.3.0
+
+        switched to localtime from gmtime so times now take into account timezones.
+
     '''
     ret = {'name': name,
            'changes': {},
@@ -733,12 +767,12 @@ def scheduled_snapshot(name, prefix, recursive=True, schedule=None):
         'month': 0,
         'year': 0,
     }
-    for hold in state_schedule.keys():
+    for hold in state_schedule:
         if hold not in schedule:
             del state_schedule[hold]
     schedule.update(state_schedule)
     # check name
-    if name not in __salt__['zfs.list'](name, **{'type': 'filesystem'}) and name not in __salt__['zfs.list'](name, **{'type': 'volume'}):
+    if not __salt__['zfs.exists'](name, **{'type': 'filesystem'}) and not __salt__['zfs.exists'](name, **{'type': 'volume'}):
         ret['comment'] = '{0} is not a filesystem or a volume or does not exist'.format(name)
         ret['result'] = False
     # check prefix
@@ -747,7 +781,7 @@ def scheduled_snapshot(name, prefix, recursive=True, schedule=None):
         ret['result'] = False
     # check schedule
     snap_count = 0
-    for hold in schedule.keys():
+    for hold in schedule:
         if not isinstance(schedule[hold], int):
             ret['comment'] = 'schedule values must be integers'
             ret['result'] = False
@@ -758,16 +792,19 @@ def scheduled_snapshot(name, prefix, recursive=True, schedule=None):
         ret['result'] = False
 
     # print debug info
-    log.debug('zfs.scheduled_snapshot::{0}::config::recursive = {1}'.format(name, recursive))
-    log.debug('zfs.scheduled_snapshot::{0}::config::prefix = {1}'.format(name, prefix))
-    log.debug('zfs.scheduled_snapshot::{0}::config::schedule = {1}'.format(name, schedule))
+    log.debug('zfs.scheduled_snapshot::%s::config::recursive = %s',
+              name, recursive)
+    log.debug('zfs.scheduled_snapshot::%s::config::prefix = %s',
+              name, prefix)
+    log.debug('zfs.scheduled_snapshot::%s::config::schedule = %s',
+              name, schedule)
 
     ## manage snapshots
     if ret['result']:
         # retreive snapshots
         prunable = []
         snapshots = {}
-        for key in schedule.keys():
+        for key in schedule:
             snapshots[key] = []
 
         for snap in sorted(__salt__['zfs.list'](name, **{'recursive': True, 'depth': 1, 'type': 'snapshot'}).keys()):
@@ -780,17 +817,18 @@ def scheduled_snapshot(name, prefix, recursive=True, schedule=None):
                 if snap not in holds or holds[snap] == 'no holds':
                     prunable.append(snap)
                     continue
-                for hold in holds[snap].keys():
+                for hold in holds[snap]:
                     hold = hold.strip()
                     if hold not in snapshots.keys():
                         continue
                     snapshots[hold].append(snap)
-        log.debug('zfs.scheduled_snapshot::{0}::snapshots = {1}'.format(name, snapshots))
+        log.debug('zfs.scheduled_snapshot::%s::snapshots = %s',
+                  name, snapshots)
 
         # create snapshot
         needed_holds = []
-        current_timestamp = gmtime()
-        for hold in snapshots.keys():
+        current_timestamp = localtime()
+        for hold in snapshots:
             # check if we need need to consider hold
             if schedule[hold] == 0:
                 continue
@@ -825,7 +863,8 @@ def scheduled_snapshot(name, prefix, recursive=True, schedule=None):
                     if current_timestamp.tm_year <= timestamp.tm_year:
                         continue
                 else:
-                    log.debug('zfs.scheduled_snapshot::{0}::hold_unknown = {1}'.format(name, hold))
+                    log.debug('zfs.scheduled_snapshot::%s::hold_unknown = %s',
+                              name, hold)
 
             # mark snapshot for hold as needed
             needed_holds.append(hold)
@@ -834,7 +873,8 @@ def scheduled_snapshot(name, prefix, recursive=True, schedule=None):
             prefix=prefix,
             timestamp=strftime('%Y%m%d_%H%M%S')
         )
-        log.debug('zfs.scheduled_snapshot::{0}::needed_holds = {1}'.format(name, needed_holds))
+        log.debug('zfs.scheduled_snapshot::%s::needed_holds = %s',
+                  name, needed_holds)
         if len(needed_holds) > 0:
             snap = '{dataset}@{snapshot}'.format(dataset=name, snapshot=snap_name)
             res = __salt__['zfs.snapshot'](snap, **{'recursive': recursive})
@@ -862,7 +902,7 @@ def scheduled_snapshot(name, prefix, recursive=True, schedule=None):
                 ret['changes']['pruned'] = []
 
         # prune snapshots
-        for hold in schedule.keys():
+        for hold in schedule:
             if hold not in snapshots.keys():
                 continue
             while len(snapshots[hold]) > schedule[hold]:
