@@ -7,7 +7,7 @@ Wire protocol: "len(payload) msgpack({'head': SOMEHEADER, 'body': SOMEBODY})"
 '''
 
 # Import Python Libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 import msgpack
 import socket
@@ -19,10 +19,11 @@ import errno
 
 # Import Salt Libs
 import salt.crypt
-import salt.utils
-import salt.utils.verify
-import salt.utils.event
 import salt.utils.async
+import salt.utils.event
+import salt.utils.platform
+import salt.utils.process
+import salt.utils.verify
 import salt.payload
 import salt.exceptions
 import salt.transport.frame
@@ -30,7 +31,7 @@ import salt.transport.ipc
 import salt.transport.client
 import salt.transport.server
 import salt.transport.mixins.auth
-import salt.ext.six as six
+from salt.ext import six
 from salt.exceptions import SaltReqTimeoutError, SaltClientError
 from salt.transport import iter_transport_opts
 
@@ -51,11 +52,16 @@ else:
 
 # Import third party libs
 try:
-    from Cryptodome.Cipher import PKCS1_OAEP
+    from M2Crypto import RSA
+    HAS_M2 = True
 except ImportError:
-    from Crypto.Cipher import PKCS1_OAEP
+    HAS_M2 = False
+    try:
+        from Cryptodome.Cipher import PKCS1_OAEP
+    except ImportError:
+        from Crypto.Cipher import PKCS1_OAEP
 
-if six.PY3 and salt.utils.is_windows():
+if six.PY3 and salt.utils.platform.is_windows():
     USE_LOAD_BALANCER = True
 else:
     USE_LOAD_BALANCER = False
@@ -221,17 +227,18 @@ class AsyncTCPReqChannel(salt.transport.client.ReqChannel):
         loop_instance_map = cls.instance_map[io_loop]
 
         key = cls.__key(opts, **kwargs)
-        if key not in loop_instance_map:
-            log.debug('Initializing new AsyncTCPReqChannel for {0}'.format(key))
+        obj = loop_instance_map.get(key)
+        if obj is None:
+            log.debug('Initializing new AsyncTCPReqChannel for %s', key)
             # we need to make a local variable for this, as we are going to store
             # it in a WeakValueDictionary-- which will remove the item if no one
             # references it-- this forces a reference while we return to the caller
-            new_obj = object.__new__(cls)
-            new_obj.__singleton_init__(opts, **kwargs)
-            loop_instance_map[key] = new_obj
+            obj = object.__new__(cls)
+            obj.__singleton_init__(opts, **kwargs)
+            loop_instance_map[key] = obj
         else:
-            log.debug('Re-using AsyncTCPReqChannel for {0}'.format(key))
-        return loop_instance_map[key]
+            log.debug('Re-using AsyncTCPReqChannel for %s', key)
+        return obj
 
     @classmethod
     def __key(cls, opts, **kwargs):
@@ -264,12 +271,14 @@ class AsyncTCPReqChannel(salt.transport.client.ReqChannel):
         resolver = kwargs.get('resolver')
 
         parse = urlparse.urlparse(self.opts['master_uri'])
-        host, port = parse.netloc.rsplit(':', 1)
-        self.master_addr = (host, int(port))
+        master_host, master_port = parse.netloc.rsplit(':', 1)
+        self.master_addr = (master_host, int(master_port))
         self._closing = False
-        self.message_client = SaltMessageClient(
-            self.opts, host, int(port), io_loop=self.io_loop,
-            resolver=resolver)
+        self.message_client = SaltMessageClientPool(self.opts,
+                                                    args=(self.opts, master_host, int(master_port),),
+                                                    kwargs={'io_loop': self.io_loop, 'resolver': resolver,
+                                                            'source_ip': self.opts.get('source_ip'),
+                                                            'source_port': self.opts.get('source_ret_port')})
 
     def close(self):
         if self._closing:
@@ -292,8 +301,11 @@ class AsyncTCPReqChannel(salt.transport.client.ReqChannel):
             yield self.auth.authenticate()
         ret = yield self.message_client.send(self._package_load(self.auth.crypticle.dumps(load)), timeout=timeout)
         key = self.auth.get_keys()
-        cipher = PKCS1_OAEP.new(key)
-        aes = cipher.decrypt(ret['key'])
+        if HAS_M2:
+            aes = key.private_decrypt(ret['key'], RSA.pkcs1_oaep_padding)
+        else:
+            cipher = PKCS1_OAEP.new(key)
+            aes = cipher.decrypt(ret['key'])
         pcrypt = salt.crypt.Crypticle(self.opts, aes)
         data = pcrypt.loads(ret[dictkey])
         if six.PY3:
@@ -404,7 +416,7 @@ class AsyncTCPPubChannel(salt.transport.mixins.auth.AESPubClientMixin, salt.tran
         def _do_transfer():
             msg = self._package_load(self.auth.crypticle.dumps(load))
             package = salt.transport.frame.frame_msg(msg, header=None)
-            yield self.message_client._stream.write(package)
+            yield self.message_client.write_to_stream(package)
             raise tornado.gen.Return(True)
 
         if force_auth or not self.auth.authenticated:
@@ -471,9 +483,7 @@ class AsyncTCPPubChannel(salt.transport.mixins.auth.AESPubClientMixin, salt.tran
             except salt.exceptions.SaltReqTimeoutError:
                 log.info('fire_master failed: master could not be contacted. Request timed out.')
             except Exception:
-                log.info('fire_master failed: {0}'.format(
-                    traceback.format_exc())
-                )
+                log.info('fire_master failed: %s', traceback.format_exc())
         else:
             self._reconnected = True
 
@@ -490,24 +500,25 @@ class AsyncTCPPubChannel(salt.transport.mixins.auth.AESPubClientMixin, salt.tran
     def connect(self):
         try:
             self.auth = salt.crypt.AsyncAuth(self.opts, io_loop=self.io_loop)
-            self.tok = self.auth.gen_token('salt')
+            self.tok = self.auth.gen_token(b'salt')
             if not self.auth.authenticated:
                 yield self.auth.authenticate()
             if self.auth.authenticated:
-                self.message_client = SaltMessageClient(
+                self.message_client = SaltMessageClientPool(
                     self.opts,
-                    self.opts['master_ip'],
-                    int(self.auth.creds['publish_port']),
-                    io_loop=self.io_loop,
-                    connect_callback=self.connect_callback,
-                    disconnect_callback=self.disconnect_callback)
+                    args=(self.opts, self.opts['master_ip'], int(self.auth.creds['publish_port']),),
+                    kwargs={'io_loop': self.io_loop,
+                            'connect_callback': self.connect_callback,
+                            'disconnect_callback': self.disconnect_callback,
+                            'source_ip': self.opts.get('source_ip'),
+                            'source_port': self.opts.get('source_publish_port')})
                 yield self.message_client.connect()  # wait for the client to be connected
                 self.connected = True
         # TODO: better exception handling...
         except KeyboardInterrupt:
             raise
         except Exception as exc:
-            if '-|RETRY|-' not in str(exc):
+            if '-|RETRY|-' not in six.text_type(exc):
                 raise SaltClientError('Unable to sign_in to master: {0}'.format(exc))  # TODO: better error message
 
     def on_recv(self, callback):
@@ -569,7 +580,7 @@ class TCPReqServerChannel(salt.transport.mixins.auth.AESReqServerMixin, salt.tra
             process_manager.add_process(
                 LoadBalancerServer, args=(self.opts, self.socket_queue)
             )
-        elif not salt.utils.is_windows():
+        elif not salt.utils.platform.is_windows():
             self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
             _set_tcp_keepalive(self._socket, self.opts)
@@ -592,7 +603,7 @@ class TCPReqServerChannel(salt.transport.mixins.auth.AESReqServerMixin, salt.tra
                                                  io_loop=self.io_loop,
                                                  ssl_options=self.opts.get('ssl'))
         else:
-            if salt.utils.is_windows():
+            if salt.utils.platform.is_windows():
                 self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                 self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                 _set_tcp_keepalive(self._socket, self.opts)
@@ -611,49 +622,72 @@ class TCPReqServerChannel(salt.transport.mixins.auth.AESReqServerMixin, salt.tra
         Handle incoming messages from underylying tcp streams
         '''
         try:
-            payload = self._decode_payload(payload)
-        except Exception:
-            stream.write(salt.transport.frame.frame_msg('bad load', header=header))
-            raise tornado.gen.Return()
+            try:
+                payload = self._decode_payload(payload)
+            except Exception:
+                stream.write(salt.transport.frame.frame_msg('bad load', header=header))
+                raise tornado.gen.Return()
 
-        # TODO helper functions to normalize payload?
-        if not isinstance(payload, dict) or not isinstance(payload.get('load'), dict):
-            yield stream.write(salt.transport.frame.frame_msg(
-                'payload and load must be a dict', header=header))
-            raise tornado.gen.Return()
+            # TODO helper functions to normalize payload?
+            if not isinstance(payload, dict) or not isinstance(payload.get('load'), dict):
+                yield stream.write(salt.transport.frame.frame_msg(
+                    'payload and load must be a dict', header=header))
+                raise tornado.gen.Return()
 
-        # intercept the "_auth" commands, since the main daemon shouldn't know
-        # anything about our key auth
-        if payload['enc'] == 'clear' and payload.get('load', {}).get('cmd') == '_auth':
-            yield stream.write(salt.transport.frame.frame_msg(
-                self._auth(payload['load']), header=header))
-            raise tornado.gen.Return()
+            try:
+                id_ = payload['load'].get('id', '')
+                if '\0' in id_:
+                    log.error('Payload contains an id with a null byte: %s', payload)
+                    stream.send(self.serial.dumps('bad load: id contains a null byte'))
+                    raise tornado.gen.Return()
+            except TypeError:
+                log.error('Payload contains non-string id: %s', payload)
+                stream.send(self.serial.dumps('bad load: id {0} is not a string'.format(id_)))
+                raise tornado.gen.Return()
 
-        # TODO: test
-        try:
-            ret, req_opts = yield self.payload_handler(payload)
-        except Exception as e:
-            # always attempt to return an error to the minion
-            stream.write('Some exception handling minion payload')
-            log.error('Some exception handling a payload from minion', exc_info=True)
-            stream.close()
-            raise tornado.gen.Return()
+            # intercept the "_auth" commands, since the main daemon shouldn't know
+            # anything about our key auth
+            if payload['enc'] == 'clear' and payload.get('load', {}).get('cmd') == '_auth':
+                yield stream.write(salt.transport.frame.frame_msg(
+                    self._auth(payload['load']), header=header))
+                raise tornado.gen.Return()
+
+            # TODO: test
+            try:
+                ret, req_opts = yield self.payload_handler(payload)
+            except Exception as e:
+                # always attempt to return an error to the minion
+                stream.write('Some exception handling minion payload')
+                log.error('Some exception handling a payload from minion', exc_info=True)
+                stream.close()
+                raise tornado.gen.Return()
+
+            req_fun = req_opts.get('fun', 'send')
+            if req_fun == 'send_clear':
+                stream.write(salt.transport.frame.frame_msg(ret, header=header))
+            elif req_fun == 'send':
+                stream.write(salt.transport.frame.frame_msg(self.crypticle.dumps(ret), header=header))
+            elif req_fun == 'send_private':
+                stream.write(salt.transport.frame.frame_msg(self._encrypt_private(ret,
+                                                             req_opts['key'],
+                                                             req_opts['tgt'],
+                                                             ), header=header))
+            else:
+                log.error('Unknown req_fun %s', req_fun)
+                # always attempt to return an error to the minion
+                stream.write('Server-side exception handling payload')
+                stream.close()
+        except tornado.gen.Return:
+            raise
+        except tornado.iostream.StreamClosedError:
+            # Stream was closed. This could happen if the remote side
+            # closed the connection on its end (eg in a timeout or shutdown
+            # situation).
+            log.error('Connection was unexpectedly closed', exc_info=True)
+        except Exception as exc:  # pylint: disable=broad-except
+            # Absorb any other exceptions
+            log.error('Unexpected exception occurred: %s', exc, exc_info=True)
 
-        req_fun = req_opts.get('fun', 'send')
-        if req_fun == 'send_clear':
-            stream.write(salt.transport.frame.frame_msg(ret, header=header))
-        elif req_fun == 'send':
-            stream.write(salt.transport.frame.frame_msg(self.crypticle.dumps(ret), header=header))
-        elif req_fun == 'send_private':
-            stream.write(salt.transport.frame.frame_msg(self._encrypt_private(ret,
-                                                         req_opts['key'],
-                                                         req_opts['tgt'],
-                                                         ), header=header))
-        else:
-            log.error('Unknown req_fun {0}'.format(req_fun))
-            # always attempt to return an error to the minion
-            stream.write('Server-side exception handling payload')
-            stream.close()
         raise tornado.gen.Return()
 
 
@@ -673,7 +707,7 @@ class SaltMessageServer(tornado.tcpserver.TCPServer, object):
         '''
         Handle incoming streams and add messages to the incoming queue
         '''
-        log.trace('Req client {0} connected'.format(address))
+        log.trace('Req client %s connected', address)
         self.clients.append((stream, address))
         unpacker = msgpack.Unpacker()
         try:
@@ -689,10 +723,10 @@ class SaltMessageServer(tornado.tcpserver.TCPServer, object):
                     self.io_loop.spawn_callback(self.message_handler, stream, header, framed_msg['body'])
 
         except tornado.iostream.StreamClosedError:
-            log.trace('req client disconnected {0}'.format(address))
+            log.trace('req client disconnected %s', address)
             self.clients.remove((stream, address))
         except Exception as e:
-            log.trace('other master-side exception: {0}'.format(e))
+            log.trace('other master-side exception: %s', e)
             self.clients.remove((stream, address))
             stream.close()
 
@@ -745,7 +779,14 @@ class TCPClientKeepAlive(tornado.tcpclient.TCPClient):
         super(TCPClientKeepAlive, self).__init__(
             resolver=resolver, io_loop=io_loop)
 
-    def _create_stream(self, max_buffer_size, af, addr):
+    def _create_stream(self, max_buffer_size, af, addr, **kwargs):  # pylint: disable=unused-argument
+        '''
+        Override _create_stream() in TCPClient.
+
+        Tornado 4.5 added the kwargs 'source_ip' and 'source_port'.
+        Due to this, use **kwargs to swallow these and any future
+        kwargs to maintain compatibility.
+        '''
         # Always connect in plaintext; we'll convert to ssl if necessary
         # after one connection has completed.
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -757,6 +798,43 @@ class TCPClientKeepAlive(tornado.tcpclient.TCPClient):
         return stream.connect(addr)
 
 
+class SaltMessageClientPool(salt.transport.MessageClientPool):
+    '''
+    Wrapper class of SaltMessageClient to avoid blocking waiting while writing data to socket.
+    '''
+    def __init__(self, opts, args=None, kwargs=None):
+        super(SaltMessageClientPool, self).__init__(SaltMessageClient, opts, args=args, kwargs=kwargs)
+
+    def __del__(self):
+        self.close()
+
+    def close(self):
+        for message_client in self.message_clients:
+            message_client.close()
+        self.message_clients = []
+
+    @tornado.gen.coroutine
+    def connect(self):
+        futures = []
+        for message_client in self.message_clients:
+            futures.append(message_client.connect())
+        for future in futures:
+            yield future
+        raise tornado.gen.Return(None)
+
+    def on_recv(self, *args, **kwargs):
+        for message_client in self.message_clients:
+            message_client.on_recv(*args, **kwargs)
+
+    def send(self, *args, **kwargs):
+        message_clients = sorted(self.message_clients, key=lambda x: len(x.send_queue))
+        return message_clients[0].send(*args, **kwargs)
+
+    def write_to_stream(self, *args, **kwargs):
+        message_clients = sorted(self.message_clients, key=lambda x: len(x.send_queue))
+        return message_clients[0]._stream.write(*args, **kwargs)
+
+
 # TODO consolidate with IPCClient
 # TODO: limit in-flight messages.
 # TODO: singleton? Something to not re-create the tcp connection so much
@@ -765,10 +843,13 @@ class SaltMessageClient(object):
     Low-level message sending client
     '''
     def __init__(self, opts, host, port, io_loop=None, resolver=None,
-                 connect_callback=None, disconnect_callback=None):
+                 connect_callback=None, disconnect_callback=None,
+                 source_ip=None, source_port=None):
         self.opts = opts
         self.host = host
         self.port = port
+        self.source_ip = source_ip
+        self.source_port = source_port
         self.connect_callback = connect_callback
         self.disconnect_callback = disconnect_callback
 
@@ -789,6 +870,7 @@ class SaltMessageClient(object):
         self._on_recv = None
         self._closing = False
         self._connecting_future = self.connect()
+        self._stream_return_future = tornado.concurrent.Future()
         self.io_loop.spawn_callback(self._stream_return)
 
     # TODO: timeout inflight sessions
@@ -806,6 +888,24 @@ class SaltMessageClient(object):
                 # the next message and the associated read future is marked
                 # 'StreamClosedError' when the stream is closed.
                 self._read_until_future.exc_info()
+                if (not self._stream_return_future.done() and
+                        self.io_loop != tornado.ioloop.IOLoop.current(
+                            instance=False)):
+                    # If _stream_return() hasn't completed, it means the IO
+                    # Loop is stopped (such as when using
+                    # 'salt.utils.async.SyncWrapper'). Ensure that
+                    # _stream_return() completes by restarting the IO Loop.
+                    # This will prevent potential errors on shutdown.
+                    orig_loop = tornado.ioloop.IOLoop.current()
+                    self.io_loop.make_current()
+                    try:
+                        self.io_loop.add_future(
+                            self._stream_return_future,
+                            lambda future: self.io_loop.stop()
+                        )
+                        self.io_loop.start()
+                    finally:
+                        orig_loop.make_current()
         self._tcp_client.close()
         # Clear callback references to allow the object that they belong to
         # to be deleted.
@@ -845,9 +945,21 @@ class SaltMessageClient(object):
             if self._closing:
                 break
             try:
-                self._stream = yield self._tcp_client.connect(self.host,
-                                                              self.port,
-                                                              ssl_options=self.opts.get('ssl'))
+                if (self.source_ip or self.source_port) and tornado.version_info >= (4, 5):
+                    ### source_ip and source_port are supported only in Tornado >= 4.5
+                    # See http://www.tornadoweb.org/en/stable/releases/v4.5.0.html
+                    # Otherwise will just ignore these args
+                    self._stream = yield self._tcp_client.connect(self.host,
+                                                                  self.port,
+                                                                  ssl_options=self.opts.get('ssl'),
+                                                                  source_ip=self.source_ip,
+                                                                  source_port=self.source_port)
+                else:
+                    if self.source_ip or self.source_port:
+                        log.warning('If you need a certain source IP/port, consider upgrading Tornado >= 4.5')
+                    self._stream = yield self._tcp_client.connect(self.host,
+                                                                  self.port,
+                                                                  ssl_options=self.opts.get('ssl'))
                 self._connecting_future.set_result(True)
                 break
             except Exception as e:
@@ -856,66 +968,69 @@ class SaltMessageClient(object):
 
     @tornado.gen.coroutine
     def _stream_return(self):
-        while not self._closing and (
-                not self._connecting_future.done() or
-                self._connecting_future.result() is not True):
-            yield self._connecting_future
-        unpacker = msgpack.Unpacker()
-        while not self._closing:
-            try:
-                self._read_until_future = self._stream.read_bytes(4096, partial=True)
-                wire_bytes = yield self._read_until_future
-                unpacker.feed(wire_bytes)
-                for framed_msg in unpacker:
-                    if six.PY3:
-                        framed_msg = salt.transport.frame.decode_embedded_strs(
-                            framed_msg
-                        )
-                    header = framed_msg['head']
-                    body = framed_msg['body']
-                    message_id = header.get('mid')
+        try:
+            while not self._closing and (
+                    not self._connecting_future.done() or
+                    self._connecting_future.result() is not True):
+                yield self._connecting_future
+            unpacker = msgpack.Unpacker()
+            while not self._closing:
+                try:
+                    self._read_until_future = self._stream.read_bytes(4096, partial=True)
+                    wire_bytes = yield self._read_until_future
+                    unpacker.feed(wire_bytes)
+                    for framed_msg in unpacker:
+                        if six.PY3:
+                            framed_msg = salt.transport.frame.decode_embedded_strs(
+                                framed_msg
+                            )
+                        header = framed_msg['head']
+                        body = framed_msg['body']
+                        message_id = header.get('mid')
 
-                    if message_id in self.send_future_map:
-                        self.send_future_map.pop(message_id).set_result(body)
-                        self.remove_message_timeout(message_id)
-                    else:
-                        if self._on_recv is not None:
-                            self.io_loop.spawn_callback(self._on_recv, header, body)
+                        if message_id in self.send_future_map:
+                            self.send_future_map.pop(message_id).set_result(body)
+                            self.remove_message_timeout(message_id)
                         else:
-                            log.error('Got response for message_id {0} that we are not tracking'.format(message_id))
-            except tornado.iostream.StreamClosedError as e:
-                log.debug('tcp stream to {0}:{1} closed, unable to recv'.format(self.host, self.port))
-                for future in six.itervalues(self.send_future_map):
-                    future.set_exception(e)
-                self.send_future_map = {}
-                if self._closing:
-                    return
-                if self.disconnect_callback:
-                    self.disconnect_callback()
-                # if the last connect finished, then we need to make a new one
-                if self._connecting_future.done():
-                    self._connecting_future = self.connect()
-                yield self._connecting_future
-            except TypeError:
-                # This is an invalid transport
-                if 'detect_mode' in self.opts:
-                    log.info('There was an error trying to use TCP transport; '
-                             'attempting to fallback to another transport')
-                else:
-                    raise SaltClientError
-            except Exception as e:
-                log.error('Exception parsing response', exc_info=True)
-                for future in six.itervalues(self.send_future_map):
-                    future.set_exception(e)
-                self.send_future_map = {}
-                if self._closing:
-                    return
-                if self.disconnect_callback:
-                    self.disconnect_callback()
-                # if the last connect finished, then we need to make a new one
-                if self._connecting_future.done():
-                    self._connecting_future = self.connect()
-                yield self._connecting_future
+                            if self._on_recv is not None:
+                                self.io_loop.spawn_callback(self._on_recv, header, body)
+                            else:
+                                log.error('Got response for message_id %s that we are not tracking', message_id)
+                except tornado.iostream.StreamClosedError as e:
+                    log.debug('tcp stream to %s:%s closed, unable to recv', self.host, self.port)
+                    for future in six.itervalues(self.send_future_map):
+                        future.set_exception(e)
+                    self.send_future_map = {}
+                    if self._closing:
+                        return
+                    if self.disconnect_callback:
+                        self.disconnect_callback()
+                    # if the last connect finished, then we need to make a new one
+                    if self._connecting_future.done():
+                        self._connecting_future = self.connect()
+                    yield self._connecting_future
+                except TypeError:
+                    # This is an invalid transport
+                    if 'detect_mode' in self.opts:
+                        log.info('There was an error trying to use TCP transport; '
+                                 'attempting to fallback to another transport')
+                    else:
+                        raise SaltClientError
+                except Exception as e:
+                    log.error('Exception parsing response', exc_info=True)
+                    for future in six.itervalues(self.send_future_map):
+                        future.set_exception(e)
+                    self.send_future_map = {}
+                    if self._closing:
+                        return
+                    if self.disconnect_callback:
+                        self.disconnect_callback()
+                    # if the last connect finished, then we need to make a new one
+                    if self._connecting_future.done():
+                        self._connecting_future = self.connect()
+                    yield self._connecting_future
+        finally:
+            self._stream_return_future.set_result(True)
 
     @tornado.gen.coroutine
     def _stream_send(self):
@@ -1157,7 +1272,7 @@ class PubServer(tornado.tcpserver.TCPServer, object):
                     client.id_ = load['id']
                     self._add_client_present(client)
             except tornado.iostream.StreamClosedError as e:
-                log.debug('tcp stream to {0} closed, unable to recv'.format(client.address))
+                log.debug('tcp stream to %s closed, unable to recv', client.address)
                 client.close()
                 self._remove_client_present(client)
                 self.clients.discard(client)
@@ -1167,7 +1282,7 @@ class PubServer(tornado.tcpserver.TCPServer, object):
                 continue
 
     def handle_stream(self, stream, address):
-        log.trace('Subscriber at {0} connected'.format(address))
+        log.trace('Subscriber at %s connected', address)
         client = Subscriber(stream, address)
         self.clients.add(client)
         self.io_loop.spawn_callback(self._stream_read, client)
@@ -1175,7 +1290,7 @@ class PubServer(tornado.tcpserver.TCPServer, object):
     # TODO: ACK the publish through IPC
     @tornado.gen.coroutine
     def publish_payload(self, package, _):
-        log.debug('TCP PubServer sending payload: {0}'.format(package))
+        log.debug('TCP PubServer sending payload: %s', package)
         payload = salt.transport.frame.frame_msg(package['payload'])
 
         to_remove = []
@@ -1196,7 +1311,7 @@ class PubServer(tornado.tcpserver.TCPServer, object):
                         except tornado.iostream.StreamClosedError:
                             to_remove.append(client)
                 else:
-                    log.debug('Publish target {0} not connected'.format(topic))
+                    log.debug('Publish target %s not connected', topic)
         else:
             for client in self.clients:
                 try:
@@ -1206,7 +1321,7 @@ class PubServer(tornado.tcpserver.TCPServer, object):
                 except tornado.iostream.StreamClosedError:
                     to_remove.append(client)
         for client in to_remove:
-            log.debug('Subscriber at {0} has disconnected from publisher'.format(client.address))
+            log.debug('Subscriber at %s has disconnected from publisher', client.address)
             client.close()
             self._remove_client_present(client)
             self.clients.discard(client)
@@ -1221,6 +1336,7 @@ class TCPPubServerChannel(salt.transport.server.PubServerChannel):
     def __init__(self, opts):
         self.opts = opts
         self.serial = salt.payload.Serial(self.opts)  # TODO: in init?
+        self.ckminions = salt.utils.minions.CkMinions(opts)
         self.io_loop = None
 
     def __setstate__(self, state):
@@ -1235,7 +1351,7 @@ class TCPPubServerChannel(salt.transport.server.PubServerChannel):
         '''
         Bind to the interface specified in the configuration file
         '''
-        salt.utils.appendproctitle(self.__class__.__name__)
+        salt.utils.process.appendproctitle(self.__class__.__name__)
 
         if log_queue is not None:
             salt.log.setup.set_multiprocessing_logging_queue(log_queue)
@@ -1269,7 +1385,7 @@ class TCPPubServerChannel(salt.transport.server.PubServerChannel):
         )
 
         # Securely create socket
-        log.info('Starting the Salt Puller on {0}'.format(pull_uri))
+        log.info('Starting the Salt Puller on %s', pull_uri)
         old_umask = os.umask(0o177)
         try:
             pull_sock.start()
@@ -1289,7 +1405,7 @@ class TCPPubServerChannel(salt.transport.server.PubServerChannel):
         do the actual publishing
         '''
         kwargs = {}
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             kwargs['log_queue'] = (
                 salt.log.setup.get_multiprocessing_logging_queue()
             )
@@ -1325,6 +1441,16 @@ class TCPPubServerChannel(salt.transport.server.PubServerChannel):
 
         # add some targeting stuff for lists only (for now)
         if load['tgt_type'] == 'list':
-            int_payload['topic_lst'] = load['tgt']
+            if isinstance(load['tgt'], six.string_types):
+                # Fetch a list of minions that match
+                _res = self.ckminions.check_minions(load['tgt'],
+                                                    tgt_type=load['tgt_type'])
+                match_ids = _res['minions']
+
+                log.debug("Publish Side Match: %s", match_ids)
+                # Send list of miions thru so zmq can target them
+                int_payload['topic_lst'] = match_ids
+            else:
+                int_payload['topic_lst'] = load['tgt']
         # Send it over IPC!
         pub_sock.send(int_payload)
