@@ -4,22 +4,22 @@ Helpful decorators for module writing
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import inspect
 import logging
+import sys
 import time
 from functools import wraps
 from collections import defaultdict
 
 # Import salt libs
-import salt.utils
 import salt.utils.args
-from salt.exceptions import CommandNotFoundError, CommandExecutionError, SaltConfigurationError
-from salt.version import SaltStackVersion, __saltstack_version__
+import salt.utils.data
+from salt.exceptions import CommandExecutionError, SaltConfigurationError
 from salt.log import LOG_LEVELS
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -52,9 +52,8 @@ class Depends(object):
         '''
 
         log.trace(
-            'Depends decorator instantiated with dep list of {0}'.format(
-                dependencies
-            )
+            'Depends decorator instantiated with dep list of %s',
+            dependencies
         )
         self.dependencies = dependencies
         self.fallback_function = kwargs.get('fallback_function')
@@ -78,8 +77,10 @@ class Depends(object):
                 self.dependency_dict[kind][dep][(mod_name, fun_name)] = \
                         (frame, self.fallback_function)
         except Exception as exc:
-            log.error('Exception encountered when attempting to inspect frame in '
-                      'dependency decorator: {0}'.format(exc))
+            log.error(
+                'Exception encountered when attempting to inspect frame in '
+                'dependency decorator: %s', exc
+            )
         return function
 
     @classmethod
@@ -95,30 +96,21 @@ class Depends(object):
                 # check if dependency is loaded
                 if dependency is True:
                     log.trace(
-                        'Dependency for {0}.{1} exists, not unloading'.format(
-                            mod_name,
-                            func_name
-                        )
+                        'Dependency for %s.%s exists, not unloading',
+                        mod_name, func_name
                     )
                     continue
                 # check if you have the dependency
                 if dependency in frame.f_globals \
                         or dependency in frame.f_locals:
                     log.trace(
-                        'Dependency ({0}) already loaded inside {1}, '
-                        'skipping'.format(
-                            dependency,
-                            mod_name
-                        )
+                        'Dependency (%s) already loaded inside %s, skipping',
+                        dependency, mod_name
                     )
                     continue
                 log.trace(
-                    'Unloading {0}.{1} because dependency ({2}) is not '
-                    'imported'.format(
-                        mod_name,
-                        func_name,
-                        dependency
-                    )
+                    'Unloading %s.%s because dependency (%s) is not imported',
+                    mod_name, func_name, dependency
                 )
                 # if not, unload the function
                 if frame:
@@ -140,7 +132,7 @@ class Depends(object):
                             del functions[mod_key]
                     except AttributeError:
                         # we already did???
-                        log.trace('{0} already removed, skipping'.format(mod_key))
+                        log.trace('%s already removed, skipping', mod_key)
                         continue
 
 
@@ -154,84 +146,20 @@ def timing(function):
     @wraps(function)
     def wrapped(*args, **kwargs):
         start_time = time.time()
-        ret = function(*args, **salt.utils.clean_kwargs(**kwargs))
+        ret = function(*args, **salt.utils.args.clean_kwargs(**kwargs))
         end_time = time.time()
         if function.__module__.startswith('salt.loaded.int.'):
             mod_name = function.__module__[16:]
         else:
             mod_name = function.__module__
-        log.profile(
-            'Function {0}.{1} took {2:.20f} seconds to execute'.format(
-                mod_name,
-                function.__name__,
-                end_time - start_time
-            )
+        fstr = 'Function %s.%s took %.{0}f seconds to execute'.format(
+            sys.float_info.dig
         )
+        log.profile(fstr, mod_name, function.__name__, end_time - start_time)
         return ret
     return wrapped
 
 
-def which(exe):
-    '''
-    Decorator wrapper for salt.utils.which
-    '''
-    def wrapper(function):
-        def wrapped(*args, **kwargs):
-            if salt.utils.which(exe) is None:
-                raise CommandNotFoundError(
-                    'The \'{0}\' binary was not found in $PATH.'.format(exe)
-                )
-            return function(*args, **kwargs)
-        return identical_signature_wrapper(function, wrapped)
-    return wrapper
-
-
-def which_bin(exes):
-    '''
-    Decorator wrapper for salt.utils.which_bin
-    '''
-    def wrapper(function):
-        def wrapped(*args, **kwargs):
-            if salt.utils.which_bin(exes) is None:
-                raise CommandNotFoundError(
-                    'None of provided binaries({0}) was not found '
-                    'in $PATH.'.format(
-                        ['\'{0}\''.format(exe) for exe in exes]
-                    )
-                )
-            return function(*args, **kwargs)
-        return identical_signature_wrapper(function, wrapped)
-    return wrapper
-
-
-def identical_signature_wrapper(original_function, wrapped_function):
-    '''
-    Return a function with identical signature as ``original_function``'s which
-    will call the ``wrapped_function``.
-    '''
-    context = {'__wrapped__': wrapped_function}
-    function_def = compile(
-        'def {0}({1}):\n'
-        '    return __wrapped__({2})'.format(
-            # Keep the original function name
-            original_function.__name__,
-            # The function signature including defaults, i.e., 'timeout=1'
-            inspect.formatargspec(
-                *salt.utils.args.get_function_argspec(original_function)
-            )[1:-1],
-            # The function signature without the defaults
-            inspect.formatargspec(
-                formatvalue=lambda val: '',
-                *salt.utils.args.get_function_argspec(original_function)
-            )[1:-1]
-        ),
-        '<string>',
-        'exec'
-    )
-    six.exec_(function_def, context)
-    return wraps(original_function)(context[original_function.__name__])
-
-
 def memoize(func):
     '''
     Memoize aka cache the return output of a function
@@ -248,12 +176,11 @@ def memoize(func):
         str_args = []
         for arg in args:
             if not isinstance(arg, six.string_types):
-                str_args.append(str(arg))
+                str_args.append(six.text_type(arg))
             else:
                 str_args.append(arg)
-        args = str_args
 
-        args_ = ','.join(list(args) + ['{0}={1}'.format(k, kwargs[k]) for k in sorted(kwargs)])
+        args_ = ','.join(list(str_args) + ['{0}={1}'.format(k, kwargs[k]) for k in sorted(kwargs)])
         if args_ not in cache:
             cache[args_] = func(*args, **kwargs)
         return cache[args_]
@@ -278,7 +205,7 @@ class _DeprecationDecorator(object):
         :param version: Expiration version
         :return:
         '''
-
+        from salt.version import SaltStackVersion, __saltstack_version__
         self._globals = globals
         self._exp_version_name = version
         self._exp_version = SaltStackVersion.from_name(self._exp_version_name)
@@ -322,14 +249,17 @@ class _DeprecationDecorator(object):
             try:
                 return self._function(*args, **kwargs)
             except TypeError as error:
-                error = str(error).replace(self._function, self._orig_f_name)  # Hide hidden functions
-                log.error('Function "{f_name}" was not properly called: {error}'.format(f_name=self._orig_f_name,
-                                                                                        error=error))
+                error = six.text_type(error).replace(self._function, self._orig_f_name)  # Hide hidden functions
+                log.error(
+                    'Function "%s" was not properly called: %s',
+                    self._orig_f_name, error
+                )
                 return self._function.__doc__
             except Exception as error:
-                log.error('Unhandled exception occurred in '
-                          'function "{f_name}: {error}'.format(f_name=self._function.__name__,
-                                                               error=error))
+                log.error(
+                    'Unhandled exception occurred in function "%s: %s',
+                    self._function.__name__, error
+                )
                 raise error
         else:
             raise CommandExecutionError("Function is deprecated, but the successor function was not found.")
@@ -542,8 +472,14 @@ class _WithDeprecated(_DeprecationDecorator):
                 f_name=function.__name__))
 
         opts = self._globals.get('__opts__', '{}')
-        use_deprecated = full_name in opts.get(self.CFG_USE_DEPRECATED, list())
-        use_superseded = full_name in opts.get(self.CFG_USE_SUPERSEDED, list())
+        pillar = self._globals.get('__pillar__', '{}')
+
+        use_deprecated = (full_name in opts.get(self.CFG_USE_DEPRECATED, list()) or
+                          full_name in pillar.get(self.CFG_USE_DEPRECATED, list()))
+
+        use_superseded = (full_name in opts.get(self.CFG_USE_SUPERSEDED, list()) or
+                          full_name in pillar.get(self.CFG_USE_SUPERSEDED, list()))
+
         if use_deprecated and use_superseded:
             raise SaltConfigurationError("Function '{0}' is mentioned both in deprecated "
                                          "and superseded sections. Please remove any of that.".format(full_name))
@@ -565,8 +501,11 @@ class _WithDeprecated(_DeprecationDecorator):
             f_name=self._orig_f_name)
 
         return func_path in self._globals.get('__opts__').get(
+            self.CFG_USE_DEPRECATED, list()) or func_path in self._globals.get('__pillar__').get(
             self.CFG_USE_DEPRECATED, list()) or (self._policy == self.OPT_IN
                                                  and not (func_path in self._globals.get('__opts__', {}).get(
+                                                          self.CFG_USE_SUPERSEDED, list()))
+                                                 and not (func_path in self._globals.get('__pillar__', {}).get(
                                                           self.CFG_USE_SUPERSEDED, list()))), func_path
 
     def __call__(self, function):
@@ -641,3 +580,19 @@ def ignores_kwargs(*kwarg_names):
             return fn(*args, **kwargs_filtered)
         return __ignores_kwargs
     return _ignores_kwargs
+
+
+def ensure_unicode_args(function):
+    '''
+    Decodes all arguments passed to the wrapped function
+    '''
+    @wraps(function)
+    def wrapped(*args, **kwargs):
+        if six.PY2:
+            return function(
+                *salt.utils.data.decode_list(args),
+                **salt.utils.data.decode_dict(kwargs)
+            )
+        else:
+            return function(*args, **kwargs)
+    return wrapped
