@@ -1,7 +1,10 @@
 # -*- coding: utf-8 -*-
+'''
+Functions for daemonizing and otherwise modifying running processes
+'''
 
 # Import python libs
-from __future__ import absolute_import, with_statement
+from __future__ import absolute_import, with_statement, print_function, unicode_literals
 import copy
 import os
 import sys
@@ -20,13 +23,15 @@ import socket
 
 # Import salt libs
 import salt.defaults.exitcodes
-import salt.utils
+import salt.utils.files
+import salt.utils.path
+import salt.utils.platform
 import salt.log.setup
 import salt.defaults.exitcodes
 from salt.log.mixins import NewStyleClassMixIn
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import queue, range  # pylint: disable=import-error,redefined-builtin
 from tornado import gen
 
@@ -40,6 +45,84 @@ try:
 except ImportError:
     pass
 
+try:
+    import setproctitle
+    HAS_SETPROCTITLE = True
+except ImportError:
+    HAS_SETPROCTITLE = False
+
+
+def appendproctitle(name):
+    '''
+    Append "name" to the current process title
+    '''
+    if HAS_SETPROCTITLE:
+        setproctitle.setproctitle(setproctitle.getproctitle() + ' ' + name)
+
+
+def daemonize(redirect_out=True):
+    '''
+    Daemonize a process
+    '''
+    # Avoid circular import
+    import salt.utils.crypt
+    try:
+        pid = os.fork()
+        if pid > 0:
+            # exit first parent
+            salt.utils.crypt.reinit_crypto()
+            sys.exit(salt.defaults.exitcodes.EX_OK)
+    except OSError as exc:
+        log.error('fork #1 failed: %s (%s)', exc.errno, exc)
+        sys.exit(salt.defaults.exitcodes.EX_GENERIC)
+
+    # decouple from parent environment
+    os.chdir('/')
+    # noinspection PyArgumentList
+    os.setsid()
+    os.umask(18)
+
+    # do second fork
+    try:
+        pid = os.fork()
+        if pid > 0:
+            salt.utils.crypt.reinit_crypto()
+            sys.exit(salt.defaults.exitcodes.EX_OK)
+    except OSError as exc:
+        log.error('fork #2 failed: %s (%s)', exc.errno, exc)
+        sys.exit(salt.defaults.exitcodes.EX_GENERIC)
+
+    salt.utils.crypt.reinit_crypto()
+
+    # A normal daemonization redirects the process output to /dev/null.
+    # Unfortunately when a python multiprocess is called the output is
+    # not cleanly redirected and the parent process dies when the
+    # multiprocessing process attempts to access stdout or err.
+    if redirect_out:
+        with salt.utils.files.fopen('/dev/null', 'r+') as dev_null:
+            # Redirect python stdin/out/err
+            # and the os stdin/out/err which can be different
+            os.dup2(dev_null.fileno(), sys.stdin.fileno())
+            os.dup2(dev_null.fileno(), sys.stdout.fileno())
+            os.dup2(dev_null.fileno(), sys.stderr.fileno())
+            os.dup2(dev_null.fileno(), 0)
+            os.dup2(dev_null.fileno(), 1)
+            os.dup2(dev_null.fileno(), 2)
+
+
+def daemonize_if(opts):
+    '''
+    Daemonize a module function process if multiprocessing is True and the
+    process is not being called by salt-call
+    '''
+    if 'salt-call' in sys.argv[0]:
+        return
+    if not opts.get('multiprocessing', True):
+        return
+    if sys.platform.startswith('win'):
+        return
+    daemonize(False)
+
 
 def systemd_notify_call(action):
     process = subprocess.Popen(['systemd-notify', action], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
@@ -55,7 +138,8 @@ def notify_systemd():
     try:
         import systemd.daemon
     except ImportError:
-        if salt.utils.which('systemd-notify') and systemd_notify_call('--booted'):
+        if salt.utils.path.which('systemd-notify') \
+                and systemd_notify_call('--booted'):
             # Notify systemd synchronously
             notify_socket = os.getenv('NOTIFY_SOCKET')
             if notify_socket:
@@ -88,13 +172,13 @@ def set_pidfile(pidfile, user):
     if not os.path.isdir(pdir) and pdir:
         os.makedirs(pdir)
     try:
-        with salt.utils.fopen(pidfile, 'w+') as ofile:
-            ofile.write(str(os.getpid()))
+        with salt.utils.files.fopen(pidfile, 'w+') as ofile:
+            ofile.write(str(os.getpid()))  # future lint: disable=blacklisted-function
     except IOError:
         pass
 
-    log.debug(('Created pidfile: {0}').format(pidfile))
-    if salt.utils.is_windows():
+    log.debug('Created pidfile: %s', pidfile)
+    if salt.utils.platform.is_windows():
         return True
 
     import pwd  # after confirming not running Windows
@@ -125,10 +209,10 @@ def set_pidfile(pidfile, user):
                 pidfile, user
             )
         )
-        log.debug('{0} Traceback follows:\n'.format(msg), exc_info=True)
+        log.debug('%s Traceback follows:', msg, exc_info=True)
         sys.stderr.write('{0}\n'.format(msg))
         sys.exit(err.errno)
-    log.debug('Chowned pidfile: {0} to user: {1}'.format(pidfile, user))
+    log.debug('Chowned pidfile: %s to user: %s', pidfile, user)
 
 
 def check_pidfile(pidfile):
@@ -142,12 +226,12 @@ def get_pidfile(pidfile):
     '''
     Return the pid from a pidfile as an integer
     '''
-    with salt.utils.fopen(pidfile) as pdf:
-        pid = pdf.read()
-    if pid:
+    try:
+        with salt.utils.files.fopen(pidfile) as pdf:
+            pid = pdf.read().strip()
         return int(pid)
-    else:
-        return
+    except (OSError, IOError, TypeError, ValueError):
+        return None
 
 
 def clean_proc(proc, wait_for_kill=10):
@@ -164,11 +248,7 @@ def clean_proc(proc, wait_for_kill=10):
             waited += 1
             time.sleep(0.1)
             if proc.is_alive() and (waited >= wait_for_kill):
-                log.error(
-                    'Process did not die with terminate(): {0}'.format(
-                        proc.pid
-                    )
-                )
+                log.error('Process did not die with terminate(): %s', proc.pid)
                 os.kill(proc.pid, signal.SIGKILL)
     except (AssertionError, AttributeError):
         # Catch AssertionError when the proc is evaluated inside the child
@@ -255,8 +335,10 @@ class ThreadPool(object):
                 # order to avoid an unclean shutdown. Le sigh.
                 continue
             try:
-                log.debug('ThreadPool executing func: {0} with args:{1}'
-                          ' kwargs{2}'.format(func, args, kwargs))
+                log.debug(
+                    'ThreadPool executing func: %s with args=%s kwargs=%s',
+                    func, args, kwargs
+                )
                 func(*args, **kwargs)
             except Exception as err:
                 log.debug(err, exc_info=True)
@@ -293,7 +375,7 @@ class ProcessManager(object):
         if kwargs is None:
             kwargs = {}
 
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             # Need to ensure that 'log_queue' is correctly transferred to
             # processes that inherit from 'MultiprocessingProcess'.
             if type(MultiprocessingProcess) is type(tgt) and (
@@ -319,7 +401,7 @@ class ProcessManager(object):
             else:
                 name = '{0}{1}.{2}'.format(
                     tgt.__module__,
-                    '.{0}'.format(tgt.__class__) if str(tgt.__class__) != "<type 'type'>" else '',
+                    '.{0}'.format(tgt.__class__) if six.text_type(tgt.__class__) != "<type 'type'>" else '',
                     tgt.__name__,
                 )
 
@@ -333,7 +415,7 @@ class ProcessManager(object):
                 process.start()
         else:
             process.start()
-        log.debug("Started '{0}' with pid {1}".format(name, process.pid))
+        log.debug("Started '%s' with pid %s", name, process.pid)
         self._process_map[process.pid] = {'tgt': tgt,
                                           'args': args,
                                           'kwargs': kwargs,
@@ -346,10 +428,12 @@ class ProcessManager(object):
         '''
         if self._restart_processes is False:
             return
-        log.info('Process {0} ({1}) died with exit status {2},'
-                 ' restarting...'.format(self._process_map[pid]['tgt'],
-                                         pid,
-                                         self._process_map[pid]['Process'].exitcode))
+        log.info(
+            'Process %s (%s) died with exit status %s, restarting...',
+            self._process_map[pid]['tgt'],
+            pid,
+            self._process_map[pid]['Process'].exitcode
+        )
         # don't block, the process is already dead
         self._process_map[pid]['Process'].join(1)
 
@@ -363,7 +447,7 @@ class ProcessManager(object):
         self._restart_processes = False
 
     def send_signal_to_processes(self, signal_):
-        if (salt.utils.is_windows() and
+        if (salt.utils.platform.is_windows() and
                 signal_ in (signal.SIGTERM, signal.SIGINT)):
             # On Windows, the subprocesses automatically have their signal
             # handlers invoked. If you send one of these signals while the
@@ -393,7 +477,7 @@ class ProcessManager(object):
         Load and start all available api modules
         '''
         log.debug('Process Manager starting!')
-        salt.utils.appendproctitle(self.name)
+        appendproctitle(self.name)
 
         # make sure to kill the subprocesses if the parent is killed
         if signal.getsignal(signal.SIGTERM) is signal.SIG_DFL:
@@ -434,7 +518,7 @@ class ProcessManager(object):
         if self._restart_processes is True:
             for pid, mapping in six.iteritems(self._process_map):
                 if not mapping['Process'].is_alive():
-                    log.trace('Process restart of {0}'.format(pid))
+                    log.trace('Process restart of %s', pid)
                     self.restart_process(pid)
 
     def kill_children(self, *args, **kwargs):
@@ -454,7 +538,7 @@ class ProcessManager(object):
                 return signal.default_int_handler(signal.SIGTERM)(*args)
             else:
                 return
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             if multiprocessing.current_process().name != 'MainProcess':
                 # Since the main process will kill subprocesses by tree,
                 # no need to do anything in the subprocesses.
@@ -462,18 +546,18 @@ class ProcessManager(object):
                 # call 'taskkill', it will leave a 'taskkill' zombie process.
                 # We want to avoid this.
                 return
-            with salt.utils.fopen(os.devnull, 'wb') as devnull:
+            with salt.utils.files.fopen(os.devnull, 'wb') as devnull:
                 for pid, p_map in six.iteritems(self._process_map):
                     # On Windows, we need to explicitly terminate sub-processes
                     # because the processes don't have a sigterm handler.
                     subprocess.call(
-                        ['taskkill', '/F', '/T', '/PID', str(pid)],
+                        ['taskkill', '/F', '/T', '/PID', six.text_type(pid)],
                         stdout=devnull, stderr=devnull
                         )
                     p_map['Process'].terminate()
         else:
             for pid, p_map in six.iteritems(self._process_map.copy()):
-                log.trace('Terminating pid {0}: {1}'.format(pid, p_map['Process']))
+                log.trace('Terminating pid %s: %s', pid, p_map['Process'])
                 if args:
                     # escalate the signal to the process
                     try:
@@ -497,7 +581,7 @@ class ProcessManager(object):
         log.trace('Waiting to kill process manager children')
         while self._process_map and time.time() < end_time:
             for pid, p_map in six.iteritems(self._process_map.copy()):
-                log.trace('Joining pid {0}: {1}'.format(pid, p_map['Process']))
+                log.trace('Joining pid %s: %s', pid, p_map['Process'])
                 p_map['Process'].join(0)
 
                 if not p_map['Process'].is_alive():
@@ -521,7 +605,7 @@ class ProcessManager(object):
                         # This is a race condition if a signal was passed to all children
                         pass
                     continue
-                log.trace('Killing pid {0}: {1}'.format(pid, p_map['Process']))
+                log.trace('Killing pid %s: %s', pid, p_map['Process'])
                 try:
                     os.kill(pid, signal.SIGKILL)
                 except OSError as exc:
@@ -542,7 +626,7 @@ class ProcessManager(object):
                 log.info(
                     'Some processes failed to respect the KILL signal: %s',
                         '; '.join(
-                            'Process: {0} (Pid: {1})'.format(v['Process'], k) for
+                            'Process: {0} (Pid: {1})'.format(v['Process'], k) for  # pylint: disable=str-format-in-logging
                             (k, v) in self._process_map.items()
                         )
                 )
@@ -553,7 +637,7 @@ class ProcessManager(object):
                 log.warning(
                     'Failed to kill the following processes: %s',
                     '; '.join(
-                        'Process: {0} (Pid: {1})'.format(v['Process'], k) for
+                        'Process: {0} (Pid: {1})'.format(v['Process'], k) for  # pylint: disable=str-format-in-logging
                         (k, v) in self._process_map.items()
                     )
                 )
@@ -575,7 +659,7 @@ class MultiprocessingProcess(multiprocessing.Process, NewStyleClassMixIn):
         return instance
 
     def __init__(self, *args, **kwargs):
-        if (salt.utils.is_windows() and
+        if (salt.utils.platform.is_windows() and
                 not hasattr(self, '_is_child') and
                 self.__setstate__.__code__ is
                 MultiprocessingProcess.__setstate__.__code__):
@@ -606,7 +690,7 @@ class MultiprocessingProcess(multiprocessing.Process, NewStyleClassMixIn):
         # 'log_queue' from kwargs.
         super(MultiprocessingProcess, self).__init__(*args, **kwargs)
 
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             # On Windows, the multiprocessing.Process object is reinitialized
             # in the child process via the constructor. Due to this, methods
             # such as ident() and is_alive() won't work properly. So we use
@@ -677,7 +761,7 @@ class MultiprocessingProcess(multiprocessing.Process, NewStyleClassMixIn):
 class SignalHandlingMultiprocessingProcess(MultiprocessingProcess):
     def __init__(self, *args, **kwargs):
         super(SignalHandlingMultiprocessingProcess, self).__init__(*args, **kwargs)
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             if hasattr(self, '_is_child'):
                 # On Windows, no need to call register_after_fork().
                 # register_after_fork() would only work on Windows if called
