@@ -2,7 +2,7 @@
 '''
 A few checks to make sure the environment is sane
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Original Author: Jeff Schroeder <jeffschroeder@computer.org>
 
@@ -27,10 +27,15 @@ from salt.log.setup import LOG_LEVELS
 from salt.exceptions import SaltClientError, SaltSystemExit, \
     CommandExecutionError
 import salt.defaults.exitcodes
-import salt.utils
+import salt.utils.files
+import salt.utils.path
+import salt.utils.platform
+import salt.utils.user
 
 log = logging.getLogger(__name__)
 
+ROOT_DIR = 'c:\\salt' if salt.utils.platform.is_windows() else '/'
+
 
 def zmq_version():
     '''
@@ -145,7 +150,7 @@ def verify_files(files, user):
     '''
     Verify that the named files exist and are owned by the named user
     '''
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         return True
     import pwd  # after confirming not running Windows
     try:
@@ -167,8 +172,8 @@ def verify_files(files, user):
                     if err.errno != errno.EEXIST:
                         raise
             if not os.path.isfile(fn_):
-                with salt.utils.fopen(fn_, 'w+') as fp_:
-                    fp_.write('')
+                with salt.utils.files.fopen(fn_, 'w'):
+                    pass
 
         except IOError as err:
             if os.path.isfile(dirname):
@@ -192,19 +197,28 @@ def verify_files(files, user):
     return True
 
 
-def verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False):
+def verify_env(
+        dirs,
+        user,
+        permissive=False,
+        pki_dir='',
+        skip_extra=False,
+        root_dir=ROOT_DIR):
     '''
     Verify that the named directories are in place and that the environment
     can shake the salt
     '''
-    if salt.utils.is_windows():
-        return win_verify_env(dirs, permissive, pki_dir, skip_extra)
+    if salt.utils.platform.is_windows():
+        return win_verify_env(root_dir,
+                              dirs,
+                              permissive=permissive,
+                              skip_extra=skip_extra)
     import pwd  # after confirming not running Windows
     try:
         pwnam = pwd.getpwnam(user)
         uid = pwnam[2]
         gid = pwnam[3]
-        groups = salt.utils.get_gid_list(user, include_default=False)
+        groups = salt.utils.user.get_gid_list(user, include_default=False)
 
     except KeyError:
         err = ('Failed to prepare the Salt environment for user '
@@ -243,7 +257,7 @@ def verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False):
                 fsubdir = os.path.join(dir_, subdir)
                 if '{0}jobs'.format(os.path.sep) in fsubdir:
                     continue
-                for root, dirs, files in os.walk(fsubdir):
+                for root, dirs, files in salt.utils.path.os_walk(fsubdir):
                     for name in files:
                         if name.startswith('.'):
                             continue
@@ -298,9 +312,9 @@ def check_user(user):
     '''
     Check user and assign process uid/gid.
     '''
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         return True
-    if user == salt.utils.get_user():
+    if user == salt.utils.user.get_user():
         return True
     import pwd  # after confirming not running Windows
     try:
@@ -309,7 +323,7 @@ def check_user(user):
             if hasattr(os, 'initgroups'):
                 os.initgroups(user, pwuser.pw_gid)  # pylint: disable=minimum-python-version
             else:
-                os.setgroups(salt.utils.get_gid_list(user, include_default=False))
+                os.setgroups(salt.utils.user.get_gid_list(user, include_default=False))
             os.setgid(pwuser.pw_gid)
             os.setuid(pwuser.pw_uid)
 
@@ -381,7 +395,7 @@ def check_path_traversal(path, user='root', skip_perm_errors=False):
             if not os.path.exists(tpath):
                 msg += ' Path does not exist.'
             else:
-                current_user = salt.utils.get_user()
+                current_user = salt.utils.user.get_user()
                 # Make the error message more intelligent based on how
                 # the user invokes salt-call or whatever other script.
                 if user != current_user:
@@ -416,9 +430,8 @@ def check_max_open_files(opts):
     accepted_count = len(os.listdir(accepted_keys_dir))
 
     log.debug(
-        'This salt-master instance has accepted {0} minion keys.'.format(
-            accepted_count
-        )
+        'This salt-master instance has accepted %s minion keys.',
+        accepted_count
     )
 
     level = logging.INFO
@@ -523,18 +536,33 @@ def verify_log(opts):
         log.warning('Insecure logging configuration detected! Sensitive data may be logged.')
 
 
-def win_verify_env(dirs, permissive=False, pki_dir='', skip_extra=False):
+def win_verify_env(
+        path,
+        dirs,
+        permissive=False,
+        pki_dir='',
+        skip_extra=False):
     '''
     Verify that the named directories are in place and that the environment
     can shake the salt
     '''
     import salt.utils.win_functions
     import salt.utils.win_dacl
+    import salt.utils.path
 
-    # Get the root path directory where salt is installed
-    path = dirs[0]
-    while os.path.basename(path) not in ['salt', 'salt-tests-tmpdir']:
-        path, base = os.path.split(path)
+    # Make sure the file_roots is not set to something unsafe since permissions
+    # on that directory are reset
+
+    # `salt.utils.path.safe_path` will consider anything inside `C:\Windows` to
+    # be unsafe. In some instances the test suite uses
+    # `C:\Windows\Temp\salt-tests-tmpdir\rootdir` as the file_roots. So, we need
+    # to consider anything in `C:\Windows\Temp` to be safe
+    system_root = os.environ.get('SystemRoot', r'C:\Windows')
+    allow_path = '\\'.join([system_root, 'TEMP'])
+    if not salt.utils.path.safe_path(path=path, allow_path=allow_path):
+        raise CommandExecutionError(
+            '`file_roots` set to a possibly unsafe location: {0}'.format(path)
+        )
 
     # Create the root path directory if missing
     if not os.path.isdir(path):
